<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Error;
use Util\Validation;

class Inventarios extends \Zeedhi\Framework\Controller\Simple {

	protected $inventarios;

	public function __construct(\SCM\Services\Inventarios $inventarios) {
        $this->inventarios = $inventarios;
    }

	public function retornaUltimoInventario(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $ultimoInventario = $this->inventarios->retornaUltimoInventario($nrorg, $cdfilial);

			$response->addDataSet(new DataSet('UltimoInventario', $ultimoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoInventario(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $proximoCodigoInventario = $this->inventarios->retornaProximoCodigoInventario($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('ProximoCodigoInventario', $proximoCodigoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoProdutoInventario(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $proximoCodigoProdutoInventario = $this->inventarios->retornaProximoCodigoProdutoInventario($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('ProximoCodigoProdutoInventario', $proximoCodigoProdutoInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaDatasInventarios(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $datasInventario = $this->inventarios->retornaDatasInventarios($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('DatasInventario', $datasInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosInventarios(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filter[0]['value'];
            $cdfilial = $filter[1]['value'];

            $produtosInventario = $this->inventarios->retornaProdutosInventarios($nrorg, $cdfilial);

            $response->addDataSet(new DataSet('ProdutosInventario', $produtosInventario));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function criarInventario(Request\Dataset $request, Response $response) {
        try {

            $rows = $request->getDataSet()->getRows();

            $inventariosSalvos = $this->inventarios->criarInventario($rows);

            if(sizeof($inventariosSalvos) > 0) {
                    
                $this->inventarios->adicionarProdutos($inventariosSalvos);
            }

        } catch (\Exception $e) {
            
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}