<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class Produtos extends \Zeedhi\Framework\Controller\Simple {

	protected $produtos;
	protected $environment;

	public function __construct(\SCM\Services\Produtos $produtos, $environment) {
        $this->produtos = $produtos;
        $this->environment = $environment;
    }

    public function retornaProdutos(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            $nrorg = $filter[0]['value'];
            $filiais = $filter[1]['value'];
            $cdgrupprod = $filter[2]['value'];
            $qtdInicial = $filter[3]['value'];
            $qtdFinal = $filter[4]['value'];
            
            $todosProdutos = $this->produtos->retornaProdutos($nrorg, $filiais, $cdgrupprod, $qtdInicial, $qtdFinal);

			$response->addDataSet(new DataSet('Produtos', $todosProdutos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosInventarios(Request $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filter[0]['value'];
            $filiais = $filter[1]['value'];

            $todosProdutos = $this->produtos->retornaProdutosInventarios($nrorg, $filiais);

            $response->addDataSet(new DataSet('InventarioProdutos', $todosProdutos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function retornaGruposProduto(Request $request, Response $response){

        try {

            // $filter = $request->getFilterCriteria()->getConditions();

            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrOrg();

            $gruposProduto = $this->produtos->retornaGruposProduto($nrorg);

            $response->addDataSet(new DataSet('GruposProduto', $gruposProduto));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }

    }

    public function retornaMascaraProduto(Request $request, Response $response){

        try {

            $filter = $request->getFilterCriteria()->getConditions();

            $nrorg = $filter[0]['value'];

            $mascaraProduto = $this->produtos->retornaMascaraProduto($nrorg);

            $response->addDataSet(new DataSet('MascaraProduto', $mascaraProduto[0]));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }

    public function retornaQtdTotalProdutos(Request $request, Response $response) {
        try {          
            $filter = $request->getFilterCriteria()->getConditions();
            $nrorg = $filter[0]['value'];
            $filiais = $filter[1]['value'];
            $cdgrupprod = $filter[2]['value'];
            
            $qtdTotalProdutos = $this->produtos->retornaQtdTotalProdutos($nrorg, $filiais, $cdgrupprod);
            $response->addDataSet(new DataSet('qtdTotalProdutos', $qtdTotalProdutos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

}