<?php

namespace SCM\Services;

class Inventarios {

    private $entityManager;
    private $novoCodigo = array();
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function retornaUltimoInventario($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaUltimoInventario($nrorg, $cdfilial);
    }

    public function retornaProximoCodigoInventario($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProximoCodigoInventario($nrorg, $cdfilial);
    }

    public function retornaProximoCodigoProdutoInventario($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProximoCodigoProdutoInventario($nrorg, $cdfilial);
    }

    public function retornaDatasInventarios($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaDatasInventarios($nrorg, $cdfilial);
    }

    public function retornaProdutosInventarios($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Inventarios')->retornaProdutosInventarios($nrorg, $cdfilial);
    }

    public function criarInventario($inventarios) {

        $inventarioreti = new \SCM\Factory\Inventarioreti();
        $this->entityManager->getConnection()->beginTransaction();

        $inventariosSalvos = array();
        
        foreach ($inventarios as $inventario) {

            $ultimoCodigoInventario = $this->retornaCodigoInventario($inventario['CDFILIAL']);
            
            if(empty($ultimoCodigoInventario)) {

                $novoCodigoInventario = $this->entityManager->getRepository('SCM\Model\Inventarioreti')->codigoInventario($inventario['CDFILIAL'], $inventario['NRORG']);
                $this->setNovoCodigo($novoCodigoInventario, $inventario['CDFILIAL']);

            } else {

                $novoCodigoInventario = $this->retornaNovoCodigoInventario($inventario['CDFILIAL']);
            }

            $novoInventario = $inventarioreti->criaInventario($inventario, $novoCodigoInventario);

            $this->entityManager->persist($novoInventario);

            $inventario['NRINVENTRETI'] = $novoCodigoInventario;
            $inventario['PRODUTOS'] = $this->atualizaProdutosInventario($novoCodigoInventario, $inventario['PRODUTOS']);

            $inventariosSalvos[] = $inventario;

        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();
        $this->novoCodigo = array();

        return $inventariosSalvos;


    }

    private function atualizaProdutosInventario($codigoInventario, $produtos) {
        
        for ($i = 0; $i < sizeof($produtos); $i++) {
            
            $produtos[$i]['NRINVENTRETI'] = $codigoInventario;
        }

        return $produtos;
    }

    private function retornaCodigoInventario($cdfilial) {

        if(array_key_exists($cdfilial, $this->novoCodigo)) {

            return $this->novoCodigo[$cdfilial];

        } else {

            return array();
        }
    } 

    private function setNovoCodigo($novoCodigoInventario, $cdfilial) {

        $this->novoCodigo[$cdfilial] = $novoCodigoInventario;
    }

    private function retornaNovoCodigoInventario($cdfilial) {

        $codigoAtual = intval($this->novoCodigo[$cdfilial]);
        $codigoAtual++;
        $codigoAtual = '0000000000'.$codigoAtual;
        $codigoAtual = substr($codigoAtual, -10);

        $this->novoCodigo[$cdfilial] = $codigoAtual;

        return $codigoAtual;
    }

    public function adicionarProdutos($inventariosSalvos) {

        $item = new \SCM\Factory\Iteminventreti();

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($inventariosSalvos as $inventario) {
            
            foreach ($inventario['PRODUTOS'] as $produto) {

                $novoProduto = $item->adicionarProdutos($produto);

                $this->entityManager->persist($novoProduto);
            }
        }
        
        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

    }
}