import { IDictionary, dayjs as moment, I18n } from '@zeedhi/core';
import { AlertService } from '@zeedhi/common';
// import * as localforage from 'localforage';

export class Utilities {
  public getItemLocalStorage(itemName: string, key?: string, value?: string) {
    const allRows = JSON.parse(<string>localStorage.getItem(itemName)) || [];
    if (key && value) {
      return allRows.find((el: IDictionary) => el[`${key}`] === value);
    }
    return allRows;
  }

  public setItemLocalStorage(itemName: string, values: object[]) {
    const result = localStorage.getItem(itemName);
    if (result !== null) {
      const obj = JSON.parse(result);
      obj.push(values);
      localStorage.setItem(itemName, JSON.stringify(obj));
    } else {
      localStorage.setItem(itemName, JSON.stringify(values));
    }
  }
  /*
  public getIDBItem(elementName: string, raw?: boolean) {
    new Promise((resolve) => {
      localforage.getItem(elementName, () => {}).then((item: any) => {
        let result;
        if (!raw) {
          result = JSON.parse(item);
        } else result = item;
        resolve(result);
      });
    });
  }

  public setIDBItem(elementName: string, content: object[], raw?: boolean) {
    let result;
    if (!raw) {
      result = JSON.stringify(content);
    } else result = content;
    return localforage.setItem(elementName, result);
  }
  */

  public showAlert(name: string, error: any, color = 'green') {
    const text = error?.response?.data;
    let msg;
    if (text) {
      msg = I18n.translate(`${text}`);
    } else {
      msg = I18n.translate(error);
    }

    const alertType = {
      name,
      color,
      text: msg,
      bottom: false,
      top: true,
      left: false,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
    AlertService.show(alertType);
  }

  public validateInterval(ini: string, fim: string) {
    if (ini && fim) {
      const iniValue = moment(ini, 'DD/MM/YYYY');
      const fimValue = moment(fim, 'DD/MM/YYYY');
      if (fimValue <= iniValue) {
        this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
        return false;
      }
      // if (fimValue.diff(iniValue, 'days') === 0) {
      //   this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
      //   return false;
      // }
      if (fimValue.diff(iniValue, 'days') > 31) {
        this.showAlert('intervalTooBig', 'SELECT_31_DAYS_INTERVAL', 'error');
        return false;
      }

      return true;
    }
    this.showAlert('invalidInterval', 'SELECT_COMPLETE_INTERVAL', 'error');
    return false;
  }

  public spliceOnCondition(array: object[], condition: (el: any, index: number) => boolean) {
    let i = array.length - 1;
    while (i) {
      if (condition(array[i], i)) {
        array.splice(i, 1);
      }
      i -= 1;
    }
  }

  public buildFilterArray(filterObj: IDictionary) {
    const filterArray: IDictionary = [];

    Object.keys(filterObj).forEach((columnName: string) => {
      filterArray.push({
        name: columnName,
        operator: '=',
        value: filterObj[columnName],
      });
    });

    return filterArray;
  }

  get getUnidadeLogada() {
    const cdfilial = localStorage.getItem('CDFILIALLOGADA');
    const nmfilial = localStorage.getItem('NMFILIALLOGADA');
    return `${cdfilial} - ${nmfilial}`;
  }
}
