module.exports = function (grunt) {

    var srcFiles = {
        less: ['css/**/*.less', 'css/*.less','schoolmeals_components/**/css/*.less'],
        libs: ['js/libs/*.js'],
        templates: ['schoolmeals_components/*/js/*.js'],
        js: ['js/util/*.js', 'js/repositories/*.js', 'js/services/*.js', 'js/controllers/*.js', '!js/zeedhiMain.js']
    };

    grunt.initConfig({
        less: {
            development: {
                options: {
                    compress: true
                },
                files: {
                    'css/css-compile.css': "css/import-ordering.less"
                }
            }
        },
        concat: {
            templates: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.templates,
                dest: 'dist/templates.js'
            },
            js: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.js,
                dest: 'dist/app.js'
            },
            libs: {
                options: {
                    process: function (src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.libs,
                dest: 'dist/libs.js'
            }
        },
        jsbeautifier: {
            final: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 0,
                        preserveNewlines: false
                    }
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        maxPreserveNewlines: 2
                    }
                }
            }
        },
        watch: {
            templates: {
                files: srcFiles.templates,
                tasks: ['jsbeautifier:dev', 'concat:templates']
            },
            js: {
                files: srcFiles.js,
                tasks: ['jsbeautifier:dev', 'concat:js', 'buildEnvironment']
            },
            less: {
                files: srcFiles.less,
                tasks: ['less']
            }
        },
        jshint: {
            merge: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": {
                    reporter: require('jshint-html-reporter'),
                    reporterOutput: './logs/dev.html',
                    esversion: 6
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": {
                    reporter: require('jshint-html-reporter'),
                    reporterOutput: './logs/dev.html',
                    esversion: 6
                }
            }
        },
        buildEnvironment: {
            development: {
                configFile: "config/devEnvironment.json"
            },
            production: {
                configFile: "config/prodEnvironment.json"
            },
            test: {
                configFile: "config/testEnvironment.json"
            },
            presentationDB: {
                configFile: "config/presentationDB.json"
            },
            srcFolders: ["./js/", "./config", "./", "../backend"]
        }
    });

    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks("grunt-jsbeautifier");
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks("grunt-notify");


    // Default task.
    grunt.registerTask('default', ['less', 'jsbeautifier:dev', 'jshint:dev', 'concat', 'buildEnvironment']);
    // Watch task.
    grunt.registerTask('devCoPilot', ['watch', 'notify']);
    // Other tasks.
    grunt.registerTask('merge', ['jsbeautifier:final', 'jshint:merge', 'concat']);
    grunt.registerTask('dev', ['jsbeautifier:dev', 'jshint:dev', 'concat']);
    grunt.registerTask('prod', ['jsbeautifier:final', 'concat']);
    grunt.registerTask('test', ['jsbeautifier:final', 'concat']);
};