// FILE: js/util/PDF.js
let PDFUtil = function(ScreenService) {
    window._ss = ScreenService;

    const titleHEXString = '#000000';
    const subTitleHEXString = '#111111';
    const textHEXString = '#333333';
    const xOffset = 4;
    let currentY = 10;

    this.resetY = () => currentY = 10;

    this.renderTitle = (jsPDFDoc, title, fontSize = 22) => {
        preventOverflow(jsPDFDoc);
        jsPDFDoc.setFontSize(fontSize);
        jsPDFDoc.setTextColor(titleHEXString);
        var splitTitle = jsPDFDoc.splitTextToSize(title, 100);
        jsPDFDoc.text(xOffset, currentY, splitTitle);
        currentY += (7 * splitTitle.length) + 7;
    };

    this.renderSubTitle = (jsPDFDoc, subTitle, verticalOffset = 5, fontSize = 14) => {
        preventOverflow(jsPDFDoc);
        jsPDFDoc.setFontSize(fontSize);
        jsPDFDoc.setTextColor(subTitleHEXString);
        var splitSubTitle = jsPDFDoc.splitTextToSize(subTitle, 100);
        jsPDFDoc.text(xOffset, currentY, splitSubTitle);
        currentY += (verticalOffset * splitSubTitle.length) + verticalOffset;
    };

    this.renderText = (jsPDFDoc, text, verticalOffset = 5, fontSize = 11) => {
        preventOverflow(jsPDFDoc);
        jsPDFDoc.setFontSize(fontSize);
        jsPDFDoc.setTextColor(textHEXString);
        var splitText = jsPDFDoc.splitTextToSize(text, 100);
        jsPDFDoc.text(xOffset, currentY, splitText);
        currentY += (verticalOffset * splitText.length) + verticalOffset;
    };

    var preventOverflow = (jsPDFDoc) => {
        var pageHeight = jsPDFDoc.internal.pageSize.height;
        if (currentY >= pageHeight) {
            jsPDFDoc.addPage();
            currentY = 10; // Restart height position
        }
    };

    this.fromDataArray = (data) => {
        let newDoc = new jsPDF('portrait', 'mm', [297.64, 500.53]);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PDFUtil', PDFUtil);
});


// FILE: js/util/UIManager.js
let UIManager = function(eventAggregator, templateManager, ScreenService) {

    window._UIM = this;
    /**
     * @type {Array<String|Function>}
     */
    let stateStack = [];

    Object.defineProperty(window, '_stack', {
        get: () => stateStack
    });

    let isListenerAttached = false;
    let activeWidget;
    let sf = this;
    /**
     * Called every time a container is open
     */
    this.init = () => {
        // stateStack = [];
        activeWidget = this.activeWidget = templateManager.container.getActiveWidget();
    };

    if (!isListenerAttached) {
        eventAggregator.subscribe('onContainerOpen', this.init);
        isListenerAttached = true;
    }

    /**
     * Displays the given widget
     * @param {ZhWidgetOjbect} widgetObject
     * @param {boolean} [shouldReload] 
     * @param {function} [callbackFunction] 
     */
    this.goToWidget = (widgetObject, shouldReload = false, callbackFunction = null) => {
        if (widgetObject && widgetObject.name !== activeWidget.name) {
            const previousActiveWidget = activeWidget.name;
            widgetObject.activate();
            activeWidget.isVisible = false;
            widgetObject.isVisible = true;
            activeWidget = widgetObject;
            stateStack.push({
                type: "widget",
                name: activeWidget.name,
                parentWidget: previousActiveWidget,
                parentScrollInfo: _getScrollPosition(),
                callbackFunction: callbackFunction
            });
            if (shouldReload) {
                return widgetObject.reload();
            } else {
                return Promise.resolve();
            }
        }
    };

    /**
     * Tryes to find a widget by it's name and display it
     * @param {string} widgetName 
     * @param {boolean} shouldReload 
     */
    this.goToWidgetByName = (widgetName, shouldReload = false, shouldCarryCurRow = false) => {
        let widgetToOpen = templateManager.container.getWidget(widgetName);
        if (widgetToOpen && widgetToOpen.name !== activeWidget.name) {
            widgetToOpen.activate();
            stateStack.push(activeWidget.name);
            activeWidget.isVisible = false;
            widgetToOpen.isVisible = true;
            if (shouldCarryCurRow) {
                widgetToOpen.currentRow = activeWidget.currentRow;
            }
            activeWidget = widgetToOpen;

            if (shouldReload) {
                return widgetToOpen.reload();
            }
        } else {
            console.log('%can attempt to open the widget: ' + widgetName + " was made. But this widget wasn't found", "font-size: 15px");
        }
    };

    /**
     * Displays a widget that is already in the stack 
     * and removes all others in the path
     * 
     * homeScreen -> list -> listItem -> listItemDetails
     * 
     * goBackToStackedWidget('list')
     * 
     * homeScreen -> list
     *  
     * @param {string} widgetName 
     */
    this.goBackToStackedWidget = widgetName => {
        let widgetToGoBack = templateManager.container.getWidget(widgetName);
        let widgetNameIndex = stateStack.findIndex(widget => widget.name === widgetName);
        if (widgetToGoBack && widgetNameIndex !== -1 && stateStack.length > 0) {
            stateStack = stateStack.slice(0, widgetNameIndex + 1);
            widgetToGoBack.activate();
            activeWidget.isVisible = false;
            widgetToGoBack.isVisible = true;
            activeWidget = widgetToGoBack;
        }
    };

    /**
     * Displays the last widget in the stack
     * or triggers the last callback state in the stack 
     */
    this.goBack = () => {
        if (stateStack.length > 0) {
            if (typeof stateStack[stateStack.length - 1] === 'string') {
                let widgetToGoBackName = stateStack.pop();
                let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackName);
                widgetToGoBack.activate();
                widgetToGoBack.isVisible = true;
                activeWidget.isVisible = false;

                activeWidget = widgetToGoBack;
            } else if (typeof stateStack[stateStack.length - 1] === 'function') {
                (stateStack.pop())();
            } else if (typeof stateStack[stateStack.length - 1] === 'object') {
                let currentState = stateStack.pop();
                if (currentState.type === 'container') {
                    let parentState = stateStack[stateStack.length - 1];
                    if (!parentState || (parentState.type === 'container' && !parentState.skipContainer)) {
                        let parentContainerArguments = ((parentState && parentState.args) || {});
                        ScreenService.openWindow(currentState.parentContainer, parentContainerArguments);
                    } else {
                        this.goBack();
                    }
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                } else if (currentState.type === 'widget') {
                    let widgetToGoBackData = currentState;
                    let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackData.parentWidget);
                    widgetToGoBack.activate();
                    widgetToGoBack.isVisible = true;
                    activeWidget.isVisible = false;
                    activeWidget = widgetToGoBack;
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                } else if (currentState.type === 'popUp') {
                    let widgetToGoBackData = currentState;
                    let widgetToGoBack = templateManager.container.getWidget(widgetToGoBackData.parentWidget);
                    widgetToGoBack.activate();
                    widgetToGoBack.isVisible = true;
                    // activeWidget.isVisible = false;

                    activeWidget = widgetToGoBack;
                    ScreenService.closePopup();
                    _scrollIntoPosition(currentState && currentState.parentScrollInfo, 100);
                }
            }
        }
    };

    /**
     * hides the editor of the active widget, (if this widget is editable)
     */
    this.hideEdit = () => {
        if (activeWidget.hasOwnProperty('isEditing')) {
            this.allowScroll();
            activeWidget.isEditing = false;
            // activeWidget.restoreHeaderActions();
        }
    };

    this.preventScroll = () => document.body.classList.add('no-scroll');

    this.allowScroll = () => document.body.classList.remove('no-scroll');

    /**
     * @param {Function} callback
     */
    this.pushState = (callback) => stateStack.push(callback);

    this.openContainer = (containerName, args, keepInHistory = true) => {
        ScreenService.openWindow(containerName, args);
        // if (keepInHistory) {
        let previouseContainer = templateManager.container.name;
        // stateStack.push(() => {
        //     ScreenService.openWindow(previouseContainer);
        // });
        stateStack.push({
            type: "container",
            name: containerName,
            parentContainer: previouseContainer,
            skipContainer: !keepInHistory,
            args: args || {},
            parentScrollInfo: _getScrollPosition()
        });
        // }
    };

    this.clearStack = () => stateStack = [];

    this.popState = () => stateStack.pop();

    this.openPopUp = (widgetToOpen) => {
        const previousActiveWidget = templateManager.container.getActiveWidget().name;
        ScreenService.openPopup(widgetToOpen);
        stateStack.push({
            type: "popUp",
            name: widgetToOpen.name,
            parentWidget: previousActiveWidget,
            parentScrollInfo: _getScrollPosition()
        });
        // widgetToOpen.isVisible = true;
    };

    let _getScrollPosition = () => {
        let documentScroll = document.documentElement.scrollTop;
        let scrollInfo = {
            elementSelector: null,
            scrollValue: null
        };
        if (documentScroll !== 0) {
            scrollInfo.scrollValue = documentScroll;
            scrollInfo.elementSelector = 'html';
        } else {
            let scrollable = document.querySelector('.scrollable');
            if (scrollable && scrollable.scrollTop !== 0) {
                scrollInfo.scrollValue = scrollTop;
                scrollInfo.elementSelector = '.scrollable';
            }
        }
        return scrollInfo;
    };

    let _scrollIntoPosition = (scrollInfo, delay = 50) => {
        if (scrollInfo && scrollInfo.elementSelector && scrollInfo.scrollValue) {
            let element = document.querySelector(scrollInfo.elementSelector);
            setTimeout(() => element.scrollTop = scrollInfo.scrollValue, delay);
        }
    };

    this.openHeaderMenu = () => templateManager.container.openHeaderMenu();

};

Configuration(function(ContextRegister) {
    ContextRegister.register('UIManager', UIManager);
});


// FILE: js/util/Util.js
var SchoolMealsUtil = function(localStorageEngine, restEngine, templateManager) {

    /**
     * Get factored widgets in container.
     */
    Container.prototype.getFactoredWidget = function(widgetName) {
        return this.factoredWidgets.find(widget => widget.name == widgetName);
    };

    /**
     * Prevent long click to open 
     */
    window.oncontextmenu = function(event) {
        event.preventDefault();
        event.stopPropagation();
        return false;
    };

    /**
     * Remove array element if testFunction results
     * become true.
     * 
     * @param {Array} array Array containing elements
     * @param {Function} testFunction Function to execute.
     */
    spliceOnCondition = function(array, testFunction) {
        var i = array.length;
        while (i--) {
            if (testFunction(array[i], i)) {
                array.splice(i, 1);
            }
        }
    };

    /**
     * Reads cookie from browser document memory.
     * 
     * @param {String} name Cookie name.
     * 
     * @return {String|Null}
     */
    this.readCookie = function(name) {
        var nameEQ = name + "=";
        var ca = document.cookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') c = c.substring(1, c.length);
            if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length, c.length);
        }
        return null;
    };

    /**
     * Remove cookie from browser document memory.
     * 
     * @param {String} name Cookie name.
     */
    this.removeCookie = function(name) {
        document.cookie = name + "=;  path=/;";
        var d = new Date();
        d.setTime(d.getTime() + (0 * 60 * 1000));
        var expires = "expires=" + d.toUTCString();
        document.cookie = name + "=" + '' + ";" + expires + ";path=/";
    };

    /**
     * Check if a string is a valid e-mail address.
     * 
     * @param {String} email Email string.
     * 
     * @returns {Boolean}
     */
    this.emailValidation = function(email) {
        var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        return re.test(String(email).toLowerCase());
    };

    /**
     * 
     * @param {} pass
     * 
     * @returns {Boolean}
     */
    this.passValidation = function(pass) {
        return pass.length > 0;
    };

    /**
     * Get back to te previous screen at system
     * history.
     * 
     * @param {Widget} widget
     */
    this.backToPreviousScreen = function(widget) {
        let steps = widget.container.steps;
        if (steps && steps.length > 0) {
            let cont = 0;
            let cond = false;
            while (cont <= steps.length && !cond) {
                let step = steps[cont];
                if (step.active) {
                    cond = true;
                } else {
                    cont = cont + 1;
                }
            }
            if (cont > 0) {
                steps[cont].active = false;
                steps[cont - 1].active = true;
                widget.container.getWidget(steps[cont].name).isVisible = false;
                widget.container.getWidget(steps[cont - 1].name).activate();
                widget.container.getWidget(steps[cont - 1].name).isVisible = true;

                let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
                let stepElements = document.getElementsByClassName("footer-wizard-step");
                let curStep = stepElements[cont - 1];
                let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
                footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
            }
        }
    };

    /**
     * Get max level from product code mask
     * of the product's tree.
     * 
     * @returns {Number} product code level.
     */
    this.getNivelMaxProduto = () => {
        var mascaraProduto = localStorage.getItem('CDPICTPROD');
        if (mascaraProduto) {
            return mascaraProduto.split('.').length;
        }
    };

    /**
     * Get max level from plates code mask
     * of the plate's tree.
     * 
     * @returns {Number} plates code level.
     */
    this.getNivelMaxPrato = () => {
        var mascaraProduto = localStorage.getItem('CDPICTPRAT');
        if (mascaraProduto) {
            return mascaraProduto.split('.').length;
        }
    };

    /**
     * Set value on dataSource of a widget according
     * to parameters.
     * 
     * @param {Widget} widget
     * @param {String} name column name to filter.
     * @param {*} value Value to use as filter.
     */
    this.setValueOnFilter = (widget, name, value) => {
        let simF = widget.dataSourceFilter.find(dsf => dsf.name === name);
        if (simF) {
            simF.value = value;
        } else {
            widget.dataSourceFilter.push({
                "name": name,
                "operator": "=",
                "value": value
            });
        }
    };

    /**
     * Returns Indexed DB var according to name.
     * Parse it to object if raw is falsy.
     * 
     * @param {String} varName Name of indexed DB key.
     * @param {Boolean} raw
     * 
     * @returns {Promise}
     */
    this.getIDBVar = (varName, raw = true) =>
        new Promise((resolve) => {
            localforage.getItem(varName, () => {}).then(item => {
                if (!raw) {
                    item = JSON.parse(item);
                }
                resolve(item);
            });
        });

    /**
     * Set Indexed DB var according to name.
     * Parse it to string if raw is truly.
     * 
     * @param {String} varName Name of indexed DB key.
     * @param {Boolean} raw
     * 
     * @returns {Promise}
     */
    this.setIDBVar = (varName, content, raw = true) => {
        if (!raw) {
            content = JSON.stringify(content);
        }
        return localforage.setItem(varName, content);
    };

    /**
     * Overwrite localStorageEngine __prepareData function to
     * verify if storaged data is string. Original function parse 
     * it regardless of type.
     * Performance of array data storagedData is better than string
     * data.
     */
    localStorageEngine._prepareData = function(storagedData) {
        let data = storagedData;

        if (typeof storagedData === "string") {
            data = JSON.parse(storagedData);
        }

        if (Util.isArray(data)) {
            data.forEach(item => {
                if (item) {
                    item.__is_new = !!item.__is_new;
                    return item;
                }
            });
        }

        return data;
    };

    /**
     * Clear all data from current user at indexed DB.
     */
    this.clearUserIDBData = () => {
        this.setIDBVar('Units', null, true);
        this.setIDBVar('productsGroups', null, true);
    };

    /**
     * Build filter array using object as reference.
     * 
     * @param {Object} filterObj Object containing data to
     * filter. Where key is column name and value is column
     * value.
     * 
     * @return {Array} Filter on Zeedhi Format
     */
    const buildFilterArray = filterObj => {
        let filterArray = [];

        Object.keys(filterObj).forEach(columnName => {
            filterArray.push({
                name: columnName,
                operator: '=',
                value: filterObj[columnName]
            });
        });

        return filterArray;
    };

    /**
     * Make a filter data request to route passed as argument.
     * 
     * @param {String} route
     * @param {Object} params Filter Data
     */
    this.doFilterDataRequest = (route, params) => restEngine.request(route, {
        "requestType": "FilterData",
        "filter": buildFilterArray(params)
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset;
    });

    /**
     * Change current logged working unit
     * to selected parameter unit.
     * 
     * @param {String} unitCode Unit code to
     * change.
     * 
     * @return {Promise}
     */
    this.changeUnit = unitCode => {
        let userData = getLocalVar('userData');

        return this.getIDBVar('Units', true).then(units => {
            let unitData = units.find(unit => unit.CDFILIAL == unitCode);

            if (unitData) {
                userData = Object.assign(userData, unitData);
            }

            setLocalVar('userData', userData);
            setLocalVar('CDFILIAL', unitCode);

            return userData;
        });
    };

    /**
     * Insert register into IDB var array.
     * If key not exists, return a warning to developer.
     * If key is not an array, return a warning to developer.
     * 
     * @param {String} dataSourceName Key name to insert.
     * 
     * @returns {Promise}
     */
    this.insertIntoDataSource = (dataSourceName, value) => {
        return this.getIDBVar(dataSourceName).then(dataSet => {
            let invalidDataset = isInvalidIDBDataSource(dataSet);

            if (!invalidDataset) {

                dataSet.push(value);
                return this.setIDBVar(dataSourceName, dataSet);

            } else
                return Promise.reject(invalidDataset);
        });
    };

    /**
     * Check if object is a valid array to manage
     * register at IDB.
     * 
     * Show a console warning to feedback developer
     * what's wrong.
     * 
     * @param {*} objToTest Object to check
     * 
     * @returns {Boolean} Is valid.
     */
    const isInvalidIDBDataSource = objToTest => {
        if (objToTest) {
            if (Array.isArray(objToTest)) {
                return false;
            } else {
                console.warn('IDB key does not reference an array.');
                return 'IDB_KEY_IS_NOT_ARRAY';
            }
        } else {
            console.warn('IDB key does not exists.');
            return 'IDB_KEY_NOT_EXISTS';
        }
    };

    /**
     * Remove itens from parameter dataSource. Target to be
     * removed can be a value or a test function. If it's a 
     * function, all registers that pass the test gonna be
     * removed.
     * 
     * @param {String} dataSourceName Name of working datasource.
     * @param {*|Function} target Exclusion target value, or test
     * function to remove more than a value.
     * 
     * @returns {Promise}
     */
    this.removeFromDataSource = (dataSourceName, target) => {
        let targetFunction = () => {};

        if (typeof target != 'function') {
            let _targetValue = Object.assign(target);
            targetFunction = dataValue => JSON.stringify(dataValue) == JSON.stringify(_targetValue);
        } else {
            targetFunction = target;
        }

        return this.getIDBVar(dataSourceName).then(dataSet => {
            let invalidDataset = isInvalidIDBDataSource(dataSet);

            if (!invalidDataset) {

                spliceOnCondition(dataSet, targetFunction);
                return this.setIDBVar(dataSourceName, dataSet).then(() => dataSet.filter(targetFunction).length);

            } else
                return Promise.reject(invalidDataset);
        });
    };

    /**
     * Check if user has a logged unit for the
     * sytem.
     * 
     * @return {Boolean}
     */
    this.hasLoggedUnit = () => {
        return getLocalVar('CDFILIAL');
    };

    /**
     * Check if user has imported data to system.
     * 
     * @return {Boolean}
     */
    this.hasImportedData = () => {
        return getLocalVar('hasImportedData');
    };

    /**
     * Execute array of promises sequentially, and
     * then return thenable from last promise;
     * 
     * Array.reduce method is used here to execute
     * all promise in sequence, without implement 
     * inherited .then's
     * 
     * @param {Array<Function>} promiseArray Array of
     * promises funcions.
     * 
     * @returns {Promise}
     */
    this.executePromisesSequentialy = promiseArray => promiseArray.reduce(
        (executedPromise, toExecutePromise) => executedPromise.then(promiseParam => toExecutePromise(promiseParam)),
        Promise.resolve()
    );

    /**
     * Show current logged unit into sub header
     * text span.
     */
    this.showUnitOnSubLabel = () => {
        let userData = getLocalVar('userData');
        templateManager.container.setSubHeaderText(`Unidade: ${userData.CDFILIAL} - ${userData.NMFILIAL}`);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('SchoolMealsUtil', SchoolMealsUtil);
});


// FILE: js/services/DataExportService.js
var DataExportService = function(restEngine, SchoolMealsUtil, ScreenService, UIManager) {

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     * 
     * @return {Promise}
     */
    this.validateActiveUser = () => {
        let userMail = getLocalVar('userData').DSEMAILOPER;
        let nrorg = getLocalVar('userData').NRORG;

        return checkActiveUser(userMail, nrorg).then(isActive => {
            if (isActive) {
                return Promise.resolve();
            } else {
                return Promise.reject();
            }
        });
    };

    /**
     * Check if user is active.
     * 
     * @param {String} mail User mail.
     * @param {String|Number} nrorg
     * 
     * @return {Promise}
     */
    const checkActiveUser = function(mail, nrorg) {
        return restEngine.request('/UsuarioAtivo', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "DSEMAILOPER",
                    "operator": "=",
                    "value": mail
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }
            ]
        }, null, null, null, null, 300000).then(function(result) {
            return result.dataset.usuario.ativo;
        });
    };

    /**
     * Bind inventory's products to inventory register.
     * Set product into inventory object and save into
     * indexed db.
     * 
     * @return {Promise} 
     */
    this.bindProductListToInventory = () => {
        let promises = [
            SchoolMealsUtil.getIDBVar('ProdutosInventario'),
            SchoolMealsUtil.getIDBVar('Inventarios')
        ];

        return Promise.all(promises).then(result => {
            let productInventory = result[0];
            let inventoryRegisters = result[1];

            inventoryRegisters.forEach(inventory => {
                inventory.PRODUTOS = productInventory.filter(product =>
                    product.NRINVENTRETI === inventory.NRINVENTRETI &&
                    product.CDFILIAL === inventory.CDFILIAL
                );
            });

            return SchoolMealsUtil.setIDBVar('Inventarios', inventoryRegisters);
        });
    };

    /**
     * Set on deleted products datasource only products
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeDeletedProductsFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('ProdutosExcluidos').then(deletedProducts => {
            deletedProducts = deletedProducts.filter(product => !!product.NRSOLICMP);
            return SchoolMealsUtil.setIDBVar('ProdutosExcluidos', deletedProducts);
        });
    };

    /**
     * Set on deleted products datasource only products
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeEditedProductsFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(editedProducts => {
            editedProducts = editedProducts.filter(product => !!product.NRSOLICMP && product.edited);
            return SchoolMealsUtil.setIDBVar('ProdutosEditados', editedProducts);
        });
    };

    /**
     * Set on MOVOPSOL datasource only registers
     * from requests that exists in remote server.
     * 
     * @returns {Promise}
     */
    this.removeLogRegisterFromInexistentRequest = () => {
        return SchoolMealsUtil.getIDBVar('Movopsol').then(logRegisterList => {
            logRegisterList = logRegisterList.filter(logMov => !!logMov.NRSOLIMOV);
            return SchoolMealsUtil.setIDBVar('Movopsol', logRegisterList);
        });
    };

    /**
     * Update request list and set extra request registers
     * according to extra products added to indexed db
     * datasource.
     */
    this.updateRequestList = () => {
        let promises = [
            SchoolMealsUtil.getIDBVar('ProdutosSolicitacao'),
            SchoolMealsUtil.getIDBVar('NovaSolicitacao'),
            SchoolMealsUtil.getIDBVar('Movopsol')
        ];

        return Promise.all(promises)
            .then(result => Object({
                requestProducts: result[0],
                newRequest: result[1],
                movopsol_log: result[2]
            }))
            .then(updateExtrasRequest)
            .then(updateRequestData)
            .then(setRequestDataSource);
    };

    /**
     * Update products and logs that are related
     * with each request register. Set requests data
     * to be exported.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     * 
     */
    const updateRequestData = dataObject => {
        dataObject.newRequest.forEach(request => {
            let products = getProductsByRequest(request, dataObject.requestProducts);

            if (products.length) {
                let logs = getLogsByRequest(request, dataObject.movopsol_log);

                request.QTITEMSOLIC = products.length;
                request.PRODUTOS = products;
                request.LOG = logs;
            }

            // request.DTSOLCMP = moment(request.DTSOLCMP).format('DD/MM/YYYY');
        });

        dataObject.newRequest = dataObject.newRequest.filter(request => request.QTITEMSOLIC);

        return Promise.resolve(dataObject);
    };

    /**
     * Update products and logs that are related
     * with each request register.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     */
    const updateExtrasRequest = dataObject => {
        let extraRequests = [];

        dataObject.newRequest.forEach(request => {
            let products = getProductsByExtraRequest(request, dataObject.requestProducts);

            if (products.length) {
                let logs = getLogsByExtraRequest(request, dataObject.movopsol_log);

                // DTSOLCMP: moment(request.DTSOLCMP).format('DD/MM/YYYY'),
                extraRequests.push(Object.assign({}, request, {
                    IDSOLEXTRA: "S",
                    QTITEMSOLIC: products.length,
                    PRODUTOS: products,
                    LOG: logs
                }));
            }
        });

        dataObject.newRequest = dataObject.newRequest.concat(extraRequests);

        return Promise.resolve(dataObject);
    };

    /**
     * Return product that are related with
     * request register and are not from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestProducts 
     */
    const getProductsByRequest = (request, requestProducts) => {
        let filteredProducts = requestProducts.filter(product =>
            product.CDFILSOLI == request.CDFILSOLI &&
            !product.NRSOLICMP &&
            product.IDITMSOLEXTRA == 'N' &&
            product.QTSOLIPROD > 0 &&
            product.inserted
        );

        filteredProducts.forEach(product => {
            product.DTPROGENTR = moment(product.DTPROGENTR).format('DD/MM/YYYY');
            product.DTUTILIZA = moment(product.DTUTILIZA).format('DD/MM/YYYY');
        });

        return filteredProducts;
    };

    /**
     * Return product that are related with
     * request register and are from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestProducts 
     */
    const getProductsByExtraRequest = (request, requestProducts) => {
        let filteredProducts = requestProducts.filter(product =>
            product.CDFILSOLI == request.CDFILSOLI &&
            !product.NRSOLICMP &&
            product.IDITMSOLEXTRA == 'S' &&
            product.inserted
        );

        filteredProducts.forEach(product => {
            product.DTPROGENTR = moment(product.DTPROGENTR).format('DD/MM/YYYY');
            product.DTUTILIZA = moment(product.DTUTILIZA).format('DD/MM/YYYY');
        });

        return filteredProducts;
    };

    /**
     * Return log registers that are related with
     * request register and are not from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestLogs 
     */
    const getLogsByRequest = (request, requestLogs) => {
        let filteredLog = requestLogs.filter(log =>
            log.CDFILSOLMOV == request.CDFILSOLI &&
            log.IDITMSOLEXTRA == "N" &&
            !log.NRSOLIMOV
        );

        filteredLog.forEach(log => {
            log.DTPROGENTR = moment(log.DTPROGENTR).format('DD/MM/YYYY');
        });

        return filteredLog;
    };

    /**
     * Return log registers that are related with
     * request register and are from extra request.
     * 
     * @param {Object} request 
     * @param {Array} requestLogs 
     */
    const getLogsByExtraRequest = (request, requestLogs) => {
        let filteredLog = requestLogs.filter(log =>
            log.CDFILSOLMOV == request.CDFILSOLI &&
            log.IDITMSOLEXTRA == "S" &&
            !log.NRSOLIMOV
        );

        filteredLog.forEach(log => {
            log.DTPROGENTR = moment(log.DTPROGENTR).format('DD/MM/YYYY');
        });

        return filteredLog;
    };

    /**
     * Set updated request registers into correct
     * datasource.
     * 
     * @param {Object} dataObject Object containing
     * data from 'ProdutosSolicitacao', 'NovaSolicitacao'
     * and 'Movopsol' dataSources.
     * 
     * @returns {Promise}
     */
    const setRequestDataSource = dataObject => SchoolMealsUtil.setIDBVar('NovaSolicitacao', dataObject.newRequest);

    /**
     * Save local dataSource chages and insertions
     * to remote server.
     * 
     * @param {String} dataSourceName 
     * 
     * @returns {Promise}
     */
    this.saveLocalDataSourceToRemote = dataSourceName => {
        return getDataToExport(dataSourceName)
            .then(dataToExport => saveRemoteData(dataSourceName, dataToExport));
    };

    /**
     * Save local dataSource to remote.
     * 
     * @param {String} dataSourceName
     * @param {Array} data 
     * 
     * @returns {Promise}
     */
    const saveRemoteData = (dataSourceName, data) => {
        if (data.length) {
            let saveRequestUrl = `/${dataSourceName}/save`;
            return restEngine.request(saveRequestUrl, {
                "requestType": "DataSet",
                "dataset": data
            });
        } else {
            return Promise.resolve();
        }
    };

    /**
     * Get edited and inserted data existent
     * in dataSource to export to remote storage.
     * 
     * @param {String} dataSourceName 
     * 
     * @returns {Promise}
     */
    const getDataToExport = dataSourceName => {
        return SchoolMealsUtil.getIDBVar(dataSourceName).then(dataSet => (dataSet && dataSet.filter(data => data.edited || data.inserted)) || []);
    };

    /**
     * Set locally dataSource's value to
     * empty array.
     * 
     * @param {String} dataSourceName
     * 
     * @returns {Promise}
     */
    const clearDataSource = dataSourceName => SchoolMealsUtil.setIDBVar(dataSourceName, []);

    /**
     * Export all datasources from local to remote 
     * storage by doing a request and save data.
     * After export, set all dataSources as empty arrays.
     * 
     * @return {Promise} 
     */
    this.exportAllDataSources = () => {
        let exportPromiseArray = [];
        let dataSourcesToExport = ['NovaSolicitacao', 'ProdutosEditados', 'ProdutosExcluidos', 'Inventarios',
            'Movopsol', 'ServicosCardapioEfetivos', 'ClientesDoServicoEfetivos'
        ];

        dataSourcesToExport.forEach(dataSourceName => {
            exportPromiseArray.push(
                this.saveLocalDataSourceToRemote(dataSourceName)
                .then(() => clearDataSource(dataSourceName))
            );
        });

        return Promise.all(exportPromiseArray);
    };

    /**
     * After export dataSources, import again secondary
     * datasources using same paramters used in importation
     * routine.
     * 
     * @param {DataImportService} DataImportService 
     * 
     * @returns {Promise}
     */
    this.afterExportDataSource = (exportResultDataset, DataImportService) => SchoolMealsUtil.executePromisesSequentialy([
        removeSecondaryDataSources,
        () => refreshSecondaryDataSources(DataImportService),
        updateRemoteUserData,
        updateLocalRequestProducts,
        () => finalizeExportation(exportResultDataset, DataImportService)
    ]);

    /**
     * Remove unexported secondary dataSources, to export
     * them again later.
     * 
     * @returns {Promise}
     */
    const removeSecondaryDataSources = () => {
        let promises = [];
        let dataSourcesToRemove = [
            "DatasInventario",
            "UltimoInventario",
            "ProdutosSolicitacao",
            "Solicitacao",
            "ProximoCodigoSolicitacao",
            "Parametros"
        ];

        dataSourcesToRemove.forEach(dataSourceName => {
            promises.push(
                clearDataSource(dataSourceName)
            );
        });

        return Promise.all(promises);
    };

    /**
     * Refresh secondary dataSources, get all data
     * using import parameters.
     * 
     * @param {DataImportService<Prototype>} DataImportService
     * Import data service.
     * 
     * @returns {Promise}
     */
    const refreshSecondaryDataSources = DataImportService => {
        return getParametersToRefreshData().then(params => Promise.all([
            DataImportService.getLastInventory(params),
            DataImportService.getInventoryDates(params),
            DataImportService.getSystemParameters(params),
            DataImportService.getRequestDeliveryDate(params),
            DataImportService.getRequestsList(params),
            DataImportService.getNextCodeRequest(params)
        ]));
    };

    /** 
     * Return parameters to request data from
     * secondary dataSources.
     * 
     * @returns {Promise}
     */
    const getParametersToRefreshData = () => getLastRequestDeliveryDate().then(lastRequestDeliveryDate =>
        SchoolMealsUtil.getIDBVar('availableUnitsToChange').then(availableUnitsToChange => {
            let userData = getLocalVar('userData');

            return Object.assign(userData, {
                mappedRelatedUnits: availableUnitsToChange.map(unit => unit.CDFILIAL),
                INTERVAL_START: userData.INTERVAL_START,
                INTERVAL_END: lastRequestDeliveryDate
            });
        }));

    /**
     * Return most recent request delivery date.
     * 
     * @returns {Promise}
     */
    const getLastRequestDeliveryDate = () => SchoolMealsUtil.getIDBVar('DatasEntregaSolicitacao')
        .then(requestDeliveryDates => requestDeliveryDates.map(reg => reg.DATA).reduce((date1, date2) => {
            return moment(date1) > moment(date2) ? date1 : date2;
        }, getLocalVar('userData').INTERVAL_END));

    /**
     * Update remote user data. 
     * Set unit code, user code and organization
     * number.
     * 
     * @returns {Promise}
     */
    const updateRemoteUserData = () => {
        let userData = getLocalVar('userData');

        return restEngine.request('/User/save', {
            "requestType": "Row",
            "row": {
                "NRORG": userData.NRORG,
                "CDOPERADOR": userData.CDOPERADOR,
                "CDFILIAL": userData.CDFILIAL
            }
        });
    };

    /**
     * Refresh local request products data of
     * all parametrized unit code.
     */
    const updateLocalRequestProducts = () => {
        return SchoolMealsUtil.executePromisesSequentialy([
            getAvailableUnitsToChange,
            getImportedUnitsRequestProducts
        ]);
    };

    /**
     * Get available units to change from indexed
     * db local storage.
     * 
     * @returns {Promise}
     */
    const getAvailableUnitsToChange = () =>
        SchoolMealsUtil.getIDBVar('availableUnitsToChange')
        .then(units => units.map(unit => unit.CDFILIAL));

    /**
     * Returns purchasing request products
     * according to unit.
     * 
     * @param {Array} unitArray Array of unit's
     * codes.
     * 
     * @returns {Promise}
     */
    const getImportedUnitsRequestProducts = unitArray => {
        let promises = [];

        unitArray.forEach(unit => {
            promises.push(getRequestProductsByUnit(unit));
        });

        return Promise.all(promises)
            .then(data => SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', data.flat()));
    };

    /**
     * Get request products related with unit
     * register.
     * 
     * @param {String} unitCode 
     */
    const getRequestProductsByUnit = unitCode =>
        getParametersToRefreshData().then((params) => {
            return SchoolMealsUtil.doFilterDataRequest(
                "/ProdutosSolicitacao", {
                    filial: unitCode,
                    nrorg: params.NRORG,
                    dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
                    dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
                    periodo: 0
                }
            );
        });

    /**
     * Finalize exportation proccess.
     * Show success message according to approved
     * request's products.
     * 
     * @param {Array} exportResultDataset 
     * @param {DataImportService} DataImportService 
     */
    const finalizeExportation = (exportResultDataset, DataImportService) => {
        let approvedProducts = exportResultDataset[1].concat(exportResultDataset[2]);
        if (approvedProducts.length) {
            buildRequestLog(approvedProducts).then(() => {
                showSuccesLogMessage();
            });
        } else {
            ScreenService.showMessage('SUCCESSFUL_EXPORTATION');
        }
    };

    /**
     * Show custom message to bring user the
     * option to get to the log page and check 
     * what services are not changed.
     */
    const showSuccesLogMessage = () => {
        ScreenService.showCustomDialog('SUCCESSFUL_EXPORTATION_WITH_NO_PROD_CHANGES',
            'SUCCESS', 'exclamation', [{
                    label: "OK",
                    code: () => {}
                },
                {
                    label: "Visualizar",
                    code: () => {}
                }
            ]
        );
    };

    /**
     * Build request logs after export all
     * data.
     * 
     * @param {Array} approvedProducts 
     * 
     * @returns {Promise}
     */
    const buildRequestLog = approvedProducts => {
        return Util.getIDBVar('LogSolicitacao').then(requestLog => {
            if (!requestLog) {
                requestLog = [];
            }

            let currentDate = moment();
            let objLog = {
                "ID": "SM" + (requestLog.length + 1),
                "DATA": currentDate.format('DD/MM/YYYY'),
                "HORA": currentDate.format('H:mm:ss'),
                "DATAHORA": currentDate.format('DD/MM/YYYY - H:mm:ss'),
                "STATUS": "ERROR"
            };

            requestLog.pop();
            requestLog.unshift(objLog);

            return Util.setIDBVar('LogSolicitacao', requestLog).then(() => buildRequetsProductsLog(approvedProducts, objLog.ID));

        });
    };

    /**
     * Save products logs related with request register
     * that call this function.
     * 
     * @param {Array} approvedProducts 
     * @param {String} parentId
     * 
     * @returns {Promise}
     */
    const buildRequetsProductsLog = (approvedProducts, parentId) => {
        return Util.getIDBVar('LogProdutosSolicitacao').then(productRequestLog => {
            approvedProducts.forEach((product, index) => {
                product.ID = 'SM' + (index + 1);
                product.PARENT = parentId;
            });

            productRequestLog.push(approvedProducts);

            return Util.setIDBVar('LogProdutosSolicitacao', productRequestLog);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataExportService', DataExportService);
});


// FILE: js/services/DataImportService.js
var DataImportService = function(restEngine, SchoolMealsUtil) {

    /**
     * Clear all imported data from app.
     */
    this.clearImportData = () => {
        const importedDataToClean = [
            "AF", "CardapioEfetivos", "ClientesDoServicoEfetivos",
            "DatasCardapioEfetivos", "DatasEntregaSolicitacao", "DatasInventario",
            "Justificativas", "Parametros", "ProximoCodigoInventario", "ProximoCodigoProdutoInventario",
            "ServicosCardapioEfetivos", "Solicitacao", "UltimoInventario",
            "availableUnitsToChange", "reducedUniqueRequestedProducts", "requestedProducts"
        ];

        let promiseQueue = [];
        importedDataToClean.forEach(idbIndex => {
            promiseQueue.push(SchoolMealsUtil.setIDBVar(idbIndex, null));
        });

        return Promise.all(promiseQueue);
    };

    /**
     * Import requested products data.
     * 
     * @param {Object} params Data search parameters
     * @param {Array} units Selected units
     * 
     * @return {Promise}
     */
    this.importRequestProductsData = (params, units) => {
        let requestQueue = [];

        units.forEach(unit => {
            params.unit = unit.CDFILIAL;
            requestQueue.push(this.getRequestedProducts(params));
        });

        return Promise.all(requestQueue).then(allData => allData.flat());
    };

    /**
     * Return units that are relateds with current
     * user.
     * 
     * @param {String} cdoperador User code.
     * @param {Array<String>} filiais Units codes.
     * 
     * @return {Promise}
     */
    this.getUserRelatedUnits = (cdoperador, filiais) => restEngine.request('/FiliaisOperador', {
        "requestType": "Row",
        "row": {
            "CDOPERADOR": cdoperador,
            "CDFILIAIS": filiais
        }
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset.Units;
    });

    /**
     * Import products data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.importProductsData = params => {
        return this.getProductAmount(params).then(productAmount => {
            let requestParams = {
                nrorg: getLocalVar('userData').NRORG,
                cdgrupprod: params.CDGRUPPROD || "T",
                initialAmount: 1
            };

            let requestQueue = [];

            params.relatedUnits.forEach(unit => {
                requestParams.unit = unit.CDFILIAL;
                requestParams.finalAmount = productAmount[unit.CDFILIAL];

                requestQueue.push(this.getProducts(requestParams));
            });

            return Promise.all(requestQueue).then(allData => allData.flat());
        });
    };

    /**
     * Import general system data after import all products.
     * 
     * @param {Object} params
     * 
     * @return {Promise}
     */
    this.importGeneralData = params => Promise.all([
        this.getSystemParameters(params),
        this.getJustifications(params),
        this.getRequestDeliveryDate(params),
        this.getRequestsList(params),
        this.getMenusDates(params),
        this.getMenusTypes(params),
        this.getMenusServices(params),
        this.getMenusClients(params),
        this.getMenusList(params),
        this.getLastInventory(params),
        this.getNextInventoryCode(params),
        this.getNextInventoryProductCode(params),
        this.getInventoryDates(params),
        this.getSupplyAuthorization(params)
    ]).then(allGeneralData => {
        let generalDataObject = {};

        allGeneralData.forEach(dataObject => {
            generalDataObject = Object.assign(generalDataObject, dataObject);
        });

        return generalDataObject;
    });

    /**
     * 
     * ALL DATA FILTER FUNCTIONS
     * USED BY AHEAD FUNCTIONS
     * 
     */

    /**
     * Return requested products data.
     * 
     * @param {Object} params Parameters to filter
     * data result.
     * 
     * @return {Promise}
     */
    this.getRequestedProducts = (params) => SchoolMealsUtil.doFilterDataRequest('/ProdutosSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filial: params.unit,
        dataInicial: params.start,
        dataFinal: params.end,
        periodo: params.period
    }).then(dataset => dataset.ProdutosSolicitacao);

    /**
     * Get products total amount.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getProductAmount = params => SchoolMealsUtil.doFilterDataRequest('/QtdTotalProdutos', {
        NRORG: getLocalVar('userData').NRORG,
        CDFILIAL: params.mappedRelatedUnits,
        CDGRUPPROD: params.CDGRUPPROD || "T"
    }).then(dataset => dataset.qtdTotalProdutos);

    /**
     * Get products data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getProducts = params => SchoolMealsUtil.doFilterDataRequest('/Produtos', {
        nrorg: getLocalVar('userData').NRORG,
        filiais: params.unit,
        cdgrupprod: params.cdgrupprod,
        qtdInicial: params.initialAmount,
        qtdFinal: params.finalAmount
    }).then(dataset => dataset.Produtos);

    /**
     * Get system's parameters data.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getSystemParameters = params => SchoolMealsUtil.doFilterDataRequest('/Parametros', {
        cdfilial: params.mappedRelatedUnits,
        nrorg: getLocalVar('userData').NRORG
    });

    /**
     * Get system's default justifications for
     * changes in process registers.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getJustifications = params => SchoolMealsUtil.doFilterDataRequest('/Justificativas', {
        nrorg: getLocalVar('userData').NRORG
    });

    /**
     * Get system's request delivery date.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getRequestDeliveryDate = params => SchoolMealsUtil.doFilterDataRequest('/DatasEntregaSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get system's requests registers.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getRequestsList = params => SchoolMealsUtil.doFilterDataRequest('/Solicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period,
        userMail: getLocalVar('userData').DSEMAILOPER
    });

    /**
     * Get all menus dates according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusDates = params => SchoolMealsUtil.doFilterDataRequest('/DatasCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all menus types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusTypes = params => SchoolMealsUtil.doFilterDataRequest('/TiposCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all services types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusServices = params => SchoolMealsUtil.doFilterDataRequest('/ServicosCardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get all Clients types according to initial and final dates, and selected
     * units.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusClients = params => SchoolMealsUtil.doFilterDataRequest('/ClientesDoServicoEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get menu list.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getMenusList = params => SchoolMealsUtil.doFilterDataRequest('/CardapioEfetivos', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits,
        dataInicial: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
        dataFinal: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
        PERIODO: params.period
    });

    /**
     * Get last inventory register according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getLastInventory = params => SchoolMealsUtil.doFilterDataRequest('/UltimoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next inventory code according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextInventoryCode = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next inventory product code according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextInventoryProductCode = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoProdutoInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get inventory dates according to units selected
     * by user.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getInventoryDates = params => SchoolMealsUtil.doFilterDataRequest('/DatasInventario', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get next purchasing request registers code.
     * 
     * @param {Array} params Search data params.
     * 
     * @returns {Promise}
     */
    this.getNextCodeRequest = params => SchoolMealsUtil.doFilterDataRequest('/ProximoCodigoSolicitacao', {
        nrorg: getLocalVar('userData').NRORG,
        filiaisImportacao: params.mappedRelatedUnits
    });

    /**
     * Get supply authorization registers
     * 
     * @param {String} cdoperador User code.
     * @param {Array<String>} filiais Units codes.
     * 
     * @return {Promise}
     */
    this.getSupplyAuthorization = params => restEngine.request('/AF', {
        "requestType": "Row",
        "row": {
            "NRORG": getLocalVar('userData').NRORG,
            "FILIAIS": params.mappedRelatedUnits,
            "DATAINICIAL": moment(params.INTERVAL_START).format('DD/MM/YYYY'),
            "DATAFINAL": moment(params.INTERVAL_END).format('DD/MM/YYYY'),
            "PERIODO": params.period
        }
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset;
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataImportService', DataImportService);
});


// FILE: js/services/LoginService.js
var LoginService = function(restEngine, ScreenService, SchoolMealsUtil) {

    this.login = function(email, password) {
        var concate_usr = [{
                'name': 'email',
                'value': email,
                'operator': '='
            },
            {
                'name': 'password',
                'value': password,
                'operator': '='
            }
        ];
        return restEngine.request("/Login", {
            requestType: "FilterData",
            filter: concate_usr
        });
    };

    this.changeUnit = function(userData) {
        return restEngine.request("/changeFilial", {
            requestType: "Row",
            row: userData
        });
    };

    this.checkLoginValidity = function(userData) {
        return restEngine.request("/checkLoginValidity", {
            requestType: "Row",
            row: userData
        });
    };

    this.checkServerRepository = () => restEngine.request('/ServerRepository', {
        requestType: "Row",
        row: {}
    });

    this.getProductsGroups = () => restEngine.request('/GruposProduto', {
        requestType: "FilterData",
        filter: [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('userData').NRORG
        }]
    });

    /**
     * Get product code mask from server.
     * 
     * @returns {Promise}
     */
    this.getProductMask = () => restEngine.request('/MascaraProduto', {
        requestType: "FilterData",
        filter: [{
            "name": "NRORG",
            "operator": "=",
            "value": getLocalVar('userData').NRORG
        }]
    }).then(result => result.dataset.MascaraProduto.CDPICTPROD.replace(/([^\.])/g, "A"));

    /**
     * Logout current user, and clear
     * it data at system memory.
     * 
     * @return {Promise}
     */
    this.logout = () => {
        setLocalVar('isLogged', false);
        setLocalVar('userData', '');
        SchoolMealsUtil.clearUserIDBData();
        ScreenService.openWindow('login');
    };

    /**
     * Check if user is active.
     * 
     * @param {String} mail User mail.
     * @param {String|Number} nrorg
     * 
     * @return {Promise}
     */
    this.checkActiveUser = function(mail, nrorg) {
        return restEngine.request('/UsuarioAtivo', {
            "requestType": "FilterData",
            "filter": [{
                    "name": "DSEMAILOPER",
                    "operator": "=",
                    "value": mail
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }
            ]
        }, null, null, null, null, 300000).then(function(result) {
            return result.dataset.usuario.ativo;
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginService', LoginService);
});


// FILE: js/services/ManageInventoryService.js
var ManageInventoryService = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, ZhUtil) {
    const util = SchoolMealsUtil;

    /**
     * Show message according to words file code, using i18n 
     * function.
     * 
     * @param {String} messageCode Message code according to
     * words file.
     */
    this.showMessage = messageCode => ScreenService.showMessage(ScreenService.i18n(messageCode));

    /**
     * Check if current unit is a school meals center.
     * 
     * @return {Boolean}
     */
    this.isCurrentUnitMealsCenter = () => {
        let userData = getLocalVar('userData');
        return userData.CDFILIAL == userData.CDFILCENTPME;
    };

    /**
     * Return if working unit have a meals center
     * (CDFILCENTPME).
     * 
     * @return {String} Meals center unit code.
     */
    this.currentUnitHaveMealsCenter = () => {
        return getLocalVar('userData').CDFILCENTPME;
    };

    /**
     * Check if all required fields are filled
     * when adding a new inventory register.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     */
    this.isInventoryRequiredFieldsFilled = inventoryRow => {
        return inventoryRow.DATE && inventoryRow.DSINVENTRETI;
    };

    /**
     * Check if inventory date is less than or equal to
     * importation date.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     */
    this.isInventoryDateValid = inventoryRow => {
        return moment(inventoryRow.DATE) <= moment(getLocalVar('userData').INTERVAL_END);
    };

    /**
     * Check if already exists an inventory register by checking
     * inventory date and related unit.
     * If remote is setted to true, checks exported
     * dataset, else checks at local dataset.
     * 
     * @param {Object} inventoryRow Object containing
     * inventory data.
     * 
     * @return {Promise}
     */
    this.existsInventory = (inventoryRow, remote = false) => {
        let idbKey = remote ? 'DatasInventario' : 'Inventarios';
        let loggedUnit = getLocalVar('CDFILIAL');

        return util.getIDBVar(idbKey).then(inventoryList => {
            return inventoryList && !!inventoryList.find(inventory =>
                moment(inventory.DTINVENTRETI, "DD/MM/YYYY").isSame(moment(inventoryRow.DATE)) &&
                inventory.CDFILIAL == loggedUnit
            );
        });
    };

    /**
     * Insert a new inventory register in related indexed db.
     * If not exists IDB key for inventory datasource, create
     * an empty datasource to save inventory registers.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     */
    this.insertInventoryRegister = inventoryRow => {
        setupInventoryRegisterToInsert(inventoryRow).then(() => {

            util.insertIntoDataSource('Inventarios', inventoryRow).then(() => {

                setInventoryIsEmptyObject(inventoryRow, true).then(() => {
                    afterInsertInventoryRegister(inventoryRow);
                });

            }).catch(errorCode => {

                if (errorCode == 'IDB_KEY_NOT_EXISTS') {
                    errorHandlerInventoryInsertion(inventoryRow);
                }

            });

        });
    };

    /**
     * Set object in indexed db used to check if a inventory
     * is empty. Implemented instead of a loop verification of
     * all products and inventorys registers because have a lower
     * processing cost.
     * 
     * @param {object} inventory
     * @param {Boolean} empty
     * 
     * @returns {Promise}
     */
    const setInventoryIsEmptyObject = (inventory, empty) => {
        return util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmptyObject => {
            let toSaveObject = {};

            if (inventoryIsEmptyObject) {
                toSaveObject = inventoryIsEmptyObject;
            }

            let objKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            toSaveObject[objKey] = empty;

            return util.setIDBVar('inventoryIsEmpty', toSaveObject);
        });
    };

    /**
     * Set inventory row properties to save correctly.
     * Set unit code, user code, inventory's date and description,
     * organization number and generate a new code to inventory.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     * 
     * @returns {Promise}
     */
    const setupInventoryRegisterToInsert = inventoryRow => {
        const userData = getLocalVar('userData');

        inventoryRow.CDFILIAL = userData.CDFILIAL;
        inventoryRow.DTINVENTRETI = moment(inventoryRow.DATE).format("DD/MM/YYYY");
        inventoryRow.CDOPERINCLUSAO = userData.CDOPERADOR;
        inventoryRow.NRORG = userData.NRORGTRAB;
        inventoryRow.inserted = true;
        return generateInventoryCode().then(nextCode => {
            inventoryRow.NRINVENTRETI = nextCode;
        });
    };

    /**
     * Generate new code to use in a inventory register insertion.
     * Generate according to inventory unit and date.
     * 
     * @returns {Promise}
     */
    const generateInventoryCode = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;

        return util.getIDBVar('ProximoCodigoInventario').then(nextCodeDataSource => {
            let nextCode = false;

            nextCodeDataSource.forEach(nextCodeReg => {
                if (nextCodeReg.CDFILIAL == unitCode) {
                    nextCode = nextCodeReg.PROXIMOCODIGO;
                    nextCodeReg.PROXIMOCODIGO = String(Number(nextCodeReg.PROXIMOCODIGO) + 1).padStart(10, '0');
                }
            });

            if (nextCode) {
                util.setIDBVar('ProximoCodigoInventario', nextCodeDataSource);
            }

            return nextCode;
        });
    };

    /**
     * Get back to previous list and show a success notification
     * after insert a new inventory register.
     * 
     * @param {Object} inventoryRow Object containing inventory
     * data.
     */
    const afterInsertInventoryRegister = inventoryRow => {
        setNextInventoryProductCode(inventoryRow.NRINVENTRETI);
        UIManager.goBack();
        templateManager.container.getActiveWidget().reload();
        ScreenService.notificationMessage(ScreenService.i18n('INVENTORY_ADDED'));
    };

    /**
     * Set next inventory's product code.
     * 
     * @param {String} inventoryCode 
     */
    const setNextInventoryProductCode = inventoryCode => {
        util.getIDBVar('nextInventoryProductCode').then(nextInventoryProductCode => {
            let code = '0001';

            if (nextInventoryProductCode[inventoryCode] !== undefined) {
                code = String(Number(nextInventoryProductCode[inventoryCode]) + 1).padStart(4, '0');
            }

            nextInventoryProductCode[inventoryCode] = code;

            util.setIDBVar('nextInventoryProductCode', nextInventoryProductCode);
        });
    };

    /**
     * Handle error after try to insert a inventory register.
     * Create an IDB register to store inventory list, then
     * try to insert it again.
     * 
     * @param {Object} inventoryRow
     */
    const errorHandlerInventoryInsertion = (inventoryRow) => {
        util.setIDBVar('Inventarios', []).then(() => {
            util.insertIntoDataSource('Inventarios', inventoryRow).then(() => {
                setInventoryIsEmptyObject(inventoryRow, true).then(() => {
                    afterInsertInventoryRegister(inventoryRow);
                });
            });
        });
    };

    /**
     * Save new product register related with selected inventory.
     * Check if register is valid and if already exists before 
     * try to insert.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing inventory
     * data.
     * 
     * @returns {Promise}
     */
    this.saveNewInventoryProduct = (product, inventory) => {
        return checkProductRowValidity(product, inventory)
            .then(() => insertNewProduct(product, inventory))
            .catch(errorCode => this.showMessage(errorCode));
    };

    /**
     * Check if product register is valid to insert.
     * Check if all fields are filled, if product already
     * exists in inventory and if quantity is greater
     * than zero.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const checkProductRowValidity = (product, inventory) => {
        let invalid = false;

        if (!isProductRequiredFieldsFilled(product)) {
            invalid = 'FILL_REQUIRED_FIELDS';
        } else if (product.QTITINVENTRETI <= 0) {
            invalid = 'PRODUCT_QUANT_MUST_BE_GT_ZERO';
        }

        return existsProductInventory(product, inventory).then(exists => {
            if (exists) {
                invalid = 'ALREADY_EXISTS_PRODUCT_INVENTORY';
            }

            return !invalid ? Promise.resolve() : Promise.reject(invalid);
        });
    };

    /**
     * Check if product code and quantity are filled;
     * 
     * @param {Object} productRow Object containing product
     * data.
     * 
     * @returns {Boolean}
     */
    const isProductRequiredFieldsFilled = productRow => {
        return (productRow.CDPRODUTO !== undefined && productRow.CDPRODUTO !== null) &&
            (productRow.QTITINVENTRETI !== undefined && productRow.QTITINVENTRETI !== null);
    };

    /**
     * Check if current product code is already inserted
     * and related to current inventory.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const existsProductInventory = (product, inventory) => {
        const productCode = product.CDPRODUTO;
        const unitCode = inventory.CDFILIAL;
        const inventoryCode = inventory.NRINVENTRETI;

        return util.getIDBVar('ProdutosInventario').then(productDataset => {
            if (productDataset && productDataset.length) {
                let exists = false;

                productDataset.forEach(prod => {

                    if (prod.CDFILIAL == unitCode &&
                        prod.CDPRODUTO == productCode &&
                        prod.NRINVENTRETI == inventoryCode) {

                        exists = true;
                    }

                });

                return exists;
            } else return false;
        });
    };

    /**
     * Insert new product register in 'ProdutosInventario'
     * dataset after make all validations.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise}
     */
    const insertNewProduct = (product, inventory) => {
        return setupProductRegisterToInsert(product, inventory)
            .then(() => {

                return util.insertIntoDataSource('ProdutosInventario', product).then(() => {
                    setNextInventoryProductCode(inventory.NRINVENTRETI);
                    return setInventoryIsEmptyObject(inventory, false);
                }).catch(errorCode => {
                    if (errorCode == 'IDB_KEY_NOT_EXISTS') {
                        return errorHandlerProductInsertion(product);
                    }
                });
            });
    };

    /**
     * Insert new product register in 'ProdutosInventario'
     * dataset after make all validations.
     * 
     * @param {Object} product Object containing product
     * data.
     * @param {Object} inventory Object containing product
     * data.
     * 
     * @returns {Promise} 
     */
    const setupProductRegisterToInsert = (product, inventory) => {
        const userData = getLocalVar('userData');

        return util.getIDBVar('nextInventoryProductCode').then(nextInventoryProductCode => {
            product.CDFILIAL = inventory.CDFILIAL;
            product.NRINVENTRETI = inventory.NRINVENTRETI;
            product.NRITINVENTRETI = nextInventoryProductCode[inventory.NRINVENTRETI];
            product.QTITINVENTRETI = product.QTITINVENTRETI;
            product.CDOPERINCLUSAO = userData.CDOPERADOR;
            product.NRORG = userData.NRORGTRAB;
            product.changed = false;
            product.__is_new = false;
            product.inserted = true;
        });
    };

    /**
     * Handle error after try to insert a product register.
     * Create an IDB register to store product list, then
     * try to insert it again.
     * 
     * @param {Object} productRow
     */
    const errorHandlerProductInsertion = (productRow) => {
        return util.setIDBVar('ProdutosInventario', [])
            .then(() => util.insertIntoDataSource('ProdutosInventario', productRow))
            .then(() => setInventoryIsEmptyObject(productRow, false));
    };

    /**
     * Remove inventory register and all products related 
     * with that.
     * 
     * @param {Object} itemToExclude Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    this.removeInventoryRegister = itemToExclude => {
        util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmpty => {
            let keyToDelete = itemToExclude.NRINVENTRETI + '_' + itemToExclude.CDFILIAL;
            delete inventoryIsEmpty[keyToDelete];
            util.setIDBVar('inventoryIsEmpty', inventoryIsEmpty);
        });

        return removeAllInventoryProducts(itemToExclude).then(() => {
            return util.removeFromDataSource('Inventarios', inventory => {
                return inventory.CDFILIAL == itemToExclude.CDFILIAL &&
                    inventory.NRINVENTRETI == itemToExclude.NRINVENTRETI;
            });
        });
    };

    /**
     * Remove all products related with 
     * parameter's inventory.
     * 
     * @param {Object} inventory Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    const removeAllInventoryProducts = inventory => {
        return util.removeFromDataSource('ProdutosInventario', product => {
            return product.CDFILIAL == inventory.CDFILIAL &&
                product.NRINVENTRETI == inventory.NRINVENTRETI;
        }).then(() => {
            setInventoryIsEmptyObject(inventory, true);
        });
    };

    /**
     * Check if inventory register has no product related
     * with it.
     * 
     * @param {Object} inventory Object containing
     * inventory data.
     * 
     * @returns {Promise}
     */
    this.checkIsInventoryEmpty = inventory => {
        return util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmptyObj => {
            let objKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            return inventoryIsEmptyObj[objKey] !== undefined &&
                inventoryIsEmptyObj[objKey];
        });
    };

    /**
     * Remove product register related with inventory register.
     * 
     * @param {Object} itemToExclude Object containing
     * inventory data.
     */
    this.removeInventoryProductRegister = productToExclude => {
        return util.removeFromDataSource('ProdutosInventario', product => {
            return product.CDFILIAL == productToExclude.CDFILIAL &&
                product.NRINVENTRETI == productToExclude.NRINVENTRETI &&
                product.CDPRODUTO == productToExclude.CDPRODUTO;
        }).then(newDataSourceLength => {
            if (newDataSourceLength === 0) {
                setInventoryIsEmptyObject(productToExclude, true);
            }

            let productList = templateManager.container.getWidget('inventoryProductList');
            productList.reload();
        });
    };

    /**
     * Remove user's selected inventorys.
     * 
     * @param {Array} inventoryArray
     * 
     * @returns {Promise}
     */
    this.removeSelectedInventorys = inventoryArray => {
        let mappedInventoryArray = inventoryArray.map(function(inventory) {
            return inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
        });

        return util.removeFromDataSource('Inventarios', inventory => {

            let inventoryKey = inventory.NRINVENTRETI + '_' + inventory.CDFILIAL;
            return mappedInventoryArray.includes(inventoryKey);

        }).then(util.removeFromDataSource('ProdutosInventario', product => {

            let productKey = product.NRINVENTRETI + '_' + product.CDFILIAL;
            return mappedInventoryArray.includes(productKey);

        })).then(util.getIDBVar('inventoryIsEmpty').then(inventoryIsEmpty => {

            mappedInventoryArray.forEach(key => delete inventoryIsEmpty[key]);
            util.setIDBVar('inventoryIsEmpty', inventoryIsEmpty);

        }));
    };

    /**
     * Remove selected inventory's products registers.
     * 
     * @param {Array} productArray
     * 
     * @returns {Promise}
     */
    this.removeSelectedProducts = productArray => {
        let mappedProductArray = productArray.map(function(product) {
            return product.CDPRODUTO + '_' + product.NRINVENTRETI + '_' + product.CDFILIAL;
        });

        return util.removeFromDataSource('ProdutosInventario', product => {

            let productKey = product.CDPRODUTO + '_' + product.NRINVENTRETI + '_' + product.CDFILIAL;
            return mappedProductArray.includes(productKey);

        }).then(newDataSourceLength => {
            if (newDataSourceLength === 0) {
                setInventoryIsEmptyObject(productArray[0], true);
            }
        });
    };

    /**
     * Save edited inventory's product register. 
     * 
     * @param {Object} savedProduct Object containing
     * product data.
     * 
     * @returns {Promise}
     */
    this.saveEditedProduct = savedProduct => {
        return util.getIDBVar('ProdutosInventario').then(productDataset => {
            productDataset.forEach(product => {
                if (product.CDPRODUTO == savedProduct.CDPRODUTO &&
                    product.CDFILIAL == savedProduct.CDFILIAL &&
                    product.NRINVENTRETI == savedProduct.NRINVENTRETI) {

                    product.QTITINVENTRETI = savedProduct.QTITINVENTRETI;
                }
            });

            return util.setIDBVar('ProdutosInventario', productDataset);
        });
    };

    /**
     * Copy last exported inventory productset to the current
     * one.
     * 
     * @param {Object} currentInventory Current inventory register.
     * 
     * @returns {Promise}
     */
    this.copyLastExportedInventory = (currentInventory) => {
        return gestLastFilteredInventory().then(filteredInventoryProducts => {
            if (filteredInventoryProducts.length) {
                return removeAllInventoryProducts(currentInventory).then(() => {
                    return importProductsFromLastInventory(currentInventory, filteredInventoryProducts);
                });
            } else {
                return Promise.reject('HAS_NO_INVENTORY');
            }
        });
    };

    /**
     * Copy all products from last exported inventory to currentInventory.
     */
    const importProductsFromLastInventory = (currentInventory, filteredInventoryProducts) => {
        let insertPromises = [];

        filteredInventoryProducts.forEach(product => {
            product.QTITINVENTRETI = product.QTITINVENTRETI || 0;
            insertPromises.push(insertNewProduct(product, currentInventory));
        });

        return Promise.all(insertPromises);
    };

    /**
     * Get last inventory products filtered by unit
     * code.
     * 
     * @return {Promise}
     */
    const gestLastFilteredInventory = () => util.getIDBVar('UltimoInventario').then(lastInventory => {
        return lastInventory.filter(inventory => inventory.CDFILIAL == getLocalVar('userData').CDFILIAL);
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ManageInventoryService', ManageInventoryService);
});


// FILE: js/services/PurchaseRequestService.js
var PurchaseRequestService = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Get unit parameters inside 'Parametros'
     * dataSource according to unit code argument.
     * 
     * @param {String} unitCode Unit code
     * 
     * @return {Promise}
     */
    this.getUnitParameters = unitCode => SchoolMealsUtil.getIDBVar('Parametros').then(parameters => {
        return parameters.filter(param => param.CDFILIAL == unitCode);
    });

    /**
     * Set filter at product field in add new request
     * form.
     * 
     * @param {Object<Field>} productField
     * @param {Array} parameterArray
     */
    this.setAddProductFieldFilter = (productField, parameterArray) => {
        let dataSourceFilter = [];

        if (parameterArray.length && parameterArray[0].IDUTPRODPREF == 'N') {
            dataSourceFilter = [{
                name: 'TIPOPRODUTO',
                operator: '=',
                value: 'E'
            }];
        }
        dataSourceFilter.push({
            operator: '=',
            name: 'CDFILIAL',
            value: getLocalVar('userData').CDFILIAL
        });

        productField.dataSourceFilter = dataSourceFilter;
    };

    /**
     * Create new product purchase request with user's data
     * input if all data is valid to proceed.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    this.createNewProductPurchaseRequest = purchaseRequest => {
        return checkRequiredFieldInRequestForm(purchaseRequest)
            .then(requestDeliveryDateIsValid)
            .then(selectedQuantityIsValid)
            .then(alreadyExistsProductInSelectedDate)
            .then(createRequestRegisterIfNotExists)
            .then(insertProductPurchaseRequestRegister);
    };

    /**
     * Check if all required fields from purchase request
     * form are filled correctly.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const checkRequiredFieldInRequestForm = purchaseRequest => {
        if ((!purchaseRequest.DSJUSTOPER ||
                !purchaseRequest.CDPRODUTO ||
                (purchaseRequest.QTSOLIPROD === undefined || purchaseRequest.QTSOLIPROD === null) ||
                !purchaseRequest.DTPROGENTR) ||
            (purchaseRequest.PRODUTOEXTRA && !purchaseRequest.DSJUSTSOLEXT)) {

            return Promise.reject('FILL_REQUIRED_FIELDS');

        } else {
            return Promise.resolve(purchaseRequest);
        }
    };

    /**
     * Check if selected delivery date to purchase request 
     * is greater than today.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const requestDeliveryDateIsValid = purchaseRequest => {
        let selectedDate = moment(purchaseRequest.DTPROGENTR);
        let today = moment();

        if (selectedDate <= today) {
            return Promise.reject('SELECTED_DATE_HAVE_BE_GREATER_THAN_TODAY');
        } else {
            return Promise.resolve(purchaseRequest);
        }
    };

    /**
     * Check if selected quantity is greater than 0.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const selectedQuantityIsValid = purchaseRequest => {
        if (purchaseRequest.QTSOLIPROD > 0) {
            return Promise.resolve(purchaseRequest);
        } else {
            return Promise.reject('SELECTED_QUANTITY_HAVE_BE_GREATER_THAN_0');
        }
    };

    /**
     * Check if already exists product related with selected
     * date.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const alreadyExistsProductInSelectedDate = purchaseRequest => {
        let productCode = purchaseRequest.CDPRODUTO;
        let deliveryDate = purchaseRequest.DTPROGENTR;
        let unitCode = getLocalVar('userData').CDFILIAL;

        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(products => {
            let exists = products.filter(product =>
                product.CDPRODUTO == productCode &&
                product.DTPROGENTR == deliveryDate &&
                product.CDFILSOLI == unitCode).length;

            if (exists) {
                return Promise.reject('ALREADY_EXISTS_PRODUCT_FOR_DELIVERY_DATE');
            } else {
                return Promise.resolve(purchaseRequest);
            }
        });
    };

    /**
     * Create new a purchase register into "NovaSolicitacao"
     * datasource if it not exists for current unit.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const createRequestRegisterIfNotExists = purchaseRequest => {
        return SchoolMealsUtil.getIDBVar('NovaSolicitacao').then(requestList => {
            if (!existsRequestRegisterForCurrentUnit(requestList)) {
                let requestRegister = buildNewRequestRegister();
                return SchoolMealsUtil.insertIntoDataSource('NovaSolicitacao', requestRegister).then(() => purchaseRequest);
            } else {
                return Promise.resolve(purchaseRequest);
            }
        });
    };

    /**
     * Check if already an existent purchase register inserted
     * for current unit.
     * 
     * @param {Array} requestList 
     * 
     * @return {Boolean}
     */
    const existsRequestRegisterForCurrentUnit = requestList => {
        return requestList.find(request => {
            return request.CDFILLANCA === getLocalVar('userData').CDFILLANCA &&
                request.CDFILSOLI === getLocalVar('userData').CDFILIAL;
        }) !== undefined;
    };

    /**
     * Build new purchase request register according to current
     * unit parameters to insert into 'NovaSolicitacao' dataSource.
     * 
     * @returns {Object}
     */
    const buildNewRequestRegister = () => {
        return {
            CDFILLANCA: getLocalVar('userData').CDFILLANCA,
            CDFILSOLI: getLocalVar('userData').CDFILIAL,
            NRSOLICMP: null,
            DTSOLCMP: moment().format('DD/MM/YYYY'),
            IDTIPLAN: 'M',
            IDSOLEXTRA: "N",
            CDFILCENT: getLocalVar('userData').CDFILCENTCMP,
            CDOPERLANC: getLocalVar('userData').CDOPERADOR,
            DTHRLANCSOL: moment().format('YYYY-MM-DD HH:mm:ss'),
            QTITEMSOLIC: null,
            NRORG: getLocalVar('userData').NRORG,
            inserted: true
        };
    };

    /**
     * Insert a product purchase request register in correct
     * related request register.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Promise}
     */
    const insertProductPurchaseRequestRegister = purchaseRequest => {
        let productRequestObject = buildProductRequestRegisterToInsert(purchaseRequest);
        return SchoolMealsUtil.insertIntoDataSource('ProdutosSolicitacao', productRequestObject).then(() => {
            return createLog_MOVOPSOL(productRequestObject, 'I', 0, productRequestObject.QTSOLIPROD);
        });
    };

    /**
     * Insert a register into MOVOPSOL dataSource to store
     * operation log.
     * 
     * @param {Object} productRequestObject 
     * @param {String} operationCode
     * @param {String} previousQuantity
     * @param {String} afterQuantity
     * 
     * @returns {Promise}
     */
    const createLog_MOVOPSOL = (productRequestObject, operationCode, previousQuantity, afterQuantity) => {
        return SchoolMealsUtil.insertIntoDataSource('Movopsol', {
            CDFILSOLMOV: productRequestObject.CDFILSOLI,
            CDFILLANMOV: productRequestObject.CDFILLAN,
            NRSOLIMOV: productRequestObject.NRSOLICMP,
            CDPRODUTO: productRequestObject.CDPRODUTO,
            DTPROGENTR: productRequestObject.DTPROGENTR,
            DTOPERSOL: moment().format("YYYY-MM-DD HH:mm:ss"),
            CDOPERADOR: getLocalVar('userData').CDOPERADOR,
            DSJUSTOPER: productRequestObject.DSJUSTOPER || null,
            DSJUSTSOLEXT: productRequestObject.DSJUSTSOLEXT || null,
            QTANTOPER: previousQuantity,
            QTDEPOPER: afterQuantity,
            NRORG: productRequestObject.NRORG,
            IDOPERAMOVS: operationCode,
            IDITMSOLEXTRA: productRequestObject.IDITMSOLEXTRA,
            inserted: true
        });
    };

    /**
     * Build an object containing a new product purchase register
     * data to insert into 'ProdutosSolicitacao' dataSource.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * 
     * @returns {Object}
     */
    const buildProductRequestRegisterToInsert = purchaseRequest => {
        return {
            CDFILLAN: getLocalVar('userData').CDFILLANCA,
            CDFILSOLI: getLocalVar('userData').CDFILIAL,
            CDPRODUTO: purchaseRequest.CDPRODUTO,
            DSJUSTOPER: purchaseRequest.DSJUSTOPER,
            DTPROGENTR: purchaseRequest.DTPROGENTR,
            DTUTILIZA: purchaseRequest.DTPROGENTR,
            NMPRODUTO: purchaseRequest.NMPRODUTO,
            NRORG: getLocalVar('userData').NRORG,
            CDFILLANCA: getLocalVar('userData').CDFILLANCA,
            NRSOLICMP: null,
            IDITMSOLEXTRA: purchaseRequest.PRODUTOEXTRA ? 'S' : 'N',
            DSJUSTSOLEXT: purchaseRequest.DSJUSTSOLEXT || null,
            QTAJUSTADA: null,
            QTSOLIPROD: purchaseRequest.QTSOLIPROD,
            SGUNIDADE: purchaseRequest.SGUNIDADE,
            IDSOLIAPROV: 'N',
            CDFILIOPELOG: null,
            inserted: true
        };
    };

    /**
     * Remove swiped product register if is valid to.
     * If is not, return a message code related
     * what's wrong with operation.
     */
    this.removeValidProductPurchaseRequest = itemToExclude => {
        return getUnitParametersByPurchasingCentral()
            .then(parameters => isValidProductToExclude(parameters, itemToExclude))
            .then(isValid => {
                if (isValid) {
                    SchoolMealsUtil.insertIntoDataSource('ProdutosExcluidos', itemToExclude);
                    createLog_MOVOPSOL(itemToExclude, 'E', itemToExclude.QTSOLIPROD || itemToExclude.QTAJUSTADA, 0);
                    return SchoolMealsUtil.removeFromDataSource('ProdutosSolicitacao', product => {
                        return product.CDPRODUTO == itemToExclude.CDPRODUTO &&
                            product.DTPROGENTR == itemToExclude.DTPROGENTR &&
                            product.CDFILSOLI == getLocalVar('userData').CDFILIAL;
                    });
                } else {
                    return Promise.reject();
                }
            });
    };

    /** 
     * Get unit parameters inside 'Parametros'
     * dataSource according to unit code and central
     * code. 
     * 
     * @returns {Promise}
     */
    const getUnitParametersByPurchasingCentral = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;
        let centralCode = getLocalVar('userData').CDFILCENTCMP;

        return this.getUnitParameters(unitCode).then(parameters => {
            return parameters.find(parameter => parameter.CENTRAL == centralCode);
        });
    };

    /**
     * Check if parameter product is valid to exclude according
     * to unit and purchasing central parameters.
     * 
     * If delivery date is expired, so product just can be edited
     * or removed if it does request approvation and central 
     * parameters allow to edit approved products.
     * 
     * If date is not expired already, so product can be edited/removed
     * if it doest not request to be approved.
     * 
     * @param {Object} parameters Filtered unit and purchasing
     * central parameters object.
     * @param {Object} itemToExclude Object containing data of product
     * to exclude.
     */
    const isValidProductToExclude = (parameters, itemToExclude) => {
        let isValid = itemToExclude.IDSOLIAPROV != 'S';
        let productDeliveryDate = moment(itemToExclude.DTPROGENTR);
        let today = moment();

        if (!isValid) {
            return Promise.reject("PRODUCT_CANNOT_BE_REMOVED_REQUEST_APPROVATION");
        } else if (productDeliveryDate <= today || (itemToExclude.IDSOLIAPROV == 'S' && parameters.ALTERAR_PRODUTO_APROVADO == 'N')) {
            return Promise.reject("PRODUCT_CANNOT_BE_REMOVED_EXPIRED_DELIVERY");
        } else {
            return Promise.resolve(parameters, itemToExclude);
        }

    };

    /**
     * Check if parameter product is valid to edit according
     * to unit and purchasing central parameters.
     * 
     * If delivery date is expired, so product just can be edited
     * or removed if it does request approvation and central 
     * parameters allow to edit approved products.
     * 
     * If date is not expired already, so product can be edited/removed
     * if request is already created.
     * 
     * @param {Object} itemToEdit Object containing data of product
     * to Edit.
     * 
     * @returns {Promise}
     */
    this.isValidProductToEdit = (itemToEdit) => {
        let productDeliveryDate = moment(itemToEdit.DTPROGENTR);
        let today = moment();

        return getUnitParametersByPurchasingCentral().then(parameters => {
            return productDeliveryDate > today &&
                !(itemToEdit.IDSOLIAPROV == 'S' && parameters.ALTERAR_PRODUTO_APROVADO == 'N') &&
                (itemToEdit.NRSOLICMP == null);
        });
    };

    /**
     * Remove selected products in purchase request list.
     * 
     * @param {Array} selectedRegisters Checked products in
     * product purchase request list.
     * 
     * @returns {Promise}
     */
    this.removeSelectedProductsIfAreValid = selectedRegisters => {
        return getUnitParametersByPurchasingCentral()
            .then(parameters => validateSelectedProductsToExclude(parameters, selectedRegisters))
            .then(isValid => {
                if (isValid) {
                    insertSelectedProductsIntoRemovedDataSet(selectedRegisters);

                    selectedRegisters.forEach(purchaseRequest => {
                        createLog_MOVOPSOL(purchaseRequest, 'E', purchaseRequest.QTSOLIPROD || purchaseRequest.QTAJUSTADA, 0);
                    });

                    let mappedRegisters = selectedRegisters.map(purchaseRequest => {
                        return purchaseRequest.CDFILSOLI + '_' + purchaseRequest.CDPRODUTO + '_' + purchaseRequest.DTPROGENTR;
                    });
                    return SchoolMealsUtil.removeFromDataSource('ProdutosSolicitacao', product => {
                        let objKey = product.CDFILSOLI + '_' + product.CDPRODUTO + '_' + product.DTPROGENTR;
                        return mappedRegisters.includes(objKey);
                    });

                } else {
                    return Promise.reject();
                }
            });
    };

    /**
     * Insert selected products to remove into
     * 'ProdutosExcluidos' dataSource
     * 
     * @param {*} selectedProducts 
     */
    const insertSelectedProductsIntoRemovedDataSet = selectedProducts => {
        SchoolMealsUtil.getIDBVar('ProdutosExcluidos').then(productDataSource => {
            productDataSource = productDataSource.concat(selectedProducts);
            SchoolMealsUtil.setIDBVar('ProdutosExcluidos', productDataSource);
        });
    };

    /**
     * Check if all selected products are valid to exclude
     * according to individual product verification to remove.
     * 
     * @param {Object} parameters Filtered unit and purchasing
     * central parameters object.
     * @param {Array} selectedRegisters Checked products in
     * product purchase request list.
     * 
     * @returns {Promise}
     */
    const validateSelectedProductsToExclude = (parameters, selectedRegisters) => {
        let validationPromiseArray = [];

        selectedRegisters.forEach(product => {
            validationPromiseArray.push(isValidProductToExclude(parameters, product));
        });

        return Promise.all(validationPromiseArray).then(isValidArray => !isValidArray.includes(false));
    };

    /**
     * Save edited purchase request data.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchase request data
     */
    this.saveEditedProductPurchaseRequest = purchaseRequest => {
        let parameters = {};
        return getUnitParametersByPurchasingCentral()
            .then(unitParam => {
                parameters = unitParam;
                checkRequiredFieldInRequestForm(purchaseRequest);
            })
            .then(() => adjustedQuantityIsValid(purchaseRequest, parameters))
            .then(saveProductChangeOnDataSource)
            .then(() => createLog_MOVOPSOL(purchaseRequest, 'A', purchaseRequest.QTSOLIPROD, purchaseRequest.QTAJUSTADA));
    };

    /**
     * Check if adjusted quantity is greater than 0 and if
     * is not greater then previous quantity.
     * 
     * @param {Object} purchaseRequest Object containing
     * purchaseRequest data.
     * @param {Object} unitParam Object containing
     * unit parameters.
     * 
     * @returns {Promise}
     */
    const adjustedQuantityIsValid = (purchaseRequest, unitParam) => {
        if (purchaseRequest.QTAJUSTADA <= 0) {
            return Promise.reject('ADJUSTED_QUANTITY_HAVE_BE_GREATER_THAN_0');
        } else if (purchaseRequest.QTAJUSTADA > purchaseRequest.QTSOLIPROD &&
            unitParam.IDBLOQALTSOLAVR == 'S' && !!purchaseRequest.NRSOLICMP) {
            return Promise.reject('ADJUSTED_QUANTITY_HAVE_BE_GREATER_THAN_PREVIOUS_QUANT');
        } else {
            return Promise.resolve(purchaseRequest, unitParam);
        }
    };

    /**
     * Set changes on product to indexed db register.
     * 
     * @param {Object} purchaseRequest 
     * @param {Object} unitParam 
     */
    const saveProductChangeOnDataSource = (purchaseRequest, unitParam) => {
        if (purchaseRequest.IDSOLIAPROV == 'S' && unitParam.DESAPROVAR_PRODUTO_ALTERADO == 'S') {
            purchaseRequest.IDSOLIAPROV = 'N';
        }
        purchaseRequest.NMSOLIAPROV = purchaseRequest.IDSOLIAPROV == 'S' ? 'Sim' : 'Não';

        return SchoolMealsUtil.getIDBVar('ProdutosSolicitacao').then(productList => {
            productList.forEach(product => {
                if (product.CDPRODUTO == purchaseRequest.CDPRODUTO && product.DTPROGENTR == purchaseRequest.DTPROGENTR &&
                    product.CDFILSOLI == purchaseRequest.CDFILSOLI) {

                    Object.assign(product, purchaseRequest, {
                        edited: true
                    });
                }
            });

            return SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', productList);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PurchaseRequestService', PurchaseRequestService);
});


// FILE: js/services/StaffManagementService.js
var StaffManagementService = function(restEngine, SchoolMealsUtil) {

    /**
     * Check if logged unit is a meal center.
     * 
     * @returns {Boolean}
     */
    this.isLoggedUnitMealsCenter = () => {
        let userData = getLocalVar('userData');
        return userData.CDFILIAL == userData.CDFILCENTPME;
    };

    /**
     * Check if logged unit has a meal center.
     * 
     * @returns {Boolean}
     */
    this.loggedUnitHasMealsCenter = () => {
        let userData = getLocalVar('userData');
        return !!userData.CDFILCENTPME;
    };

    /**
     * Check if has staff registers in selected
     * import date.
     * 
     * @returns {Promise}
     */
    this.checkIfHasStaffInPeriod = () => {
        return SchoolMealsUtil.getIDBVar('TiposCardapioEfetivos').then(staff => {
            if (staff && staff.length) {
                return Promise.resolve(staff);
            } else {
                return Promise.reject('CANNOT_MANAGE_STAFF_HAS_NO_STAFF_IN_PERIOD');
            }
        });
    };

    /**
     * Check if has staff registers in logged unit.
     * 
     * @returns {Promise}
     */
    this.checkIfHasStaffInUnit = () => {
        let unitCode = getLocalVar('userData').CDFILIAL;

        return SchoolMealsUtil.getIDBVar('TiposCardapioEfetivos').then(staff => {
            staff = staff.filter(reg => reg.CDFILIAL == unitCode);

            if (staff && staff.length) {
                return Promise.resolve(staff);
            } else {
                return Promise.reject('CANNOT_MANAGE_STAFF_HAS_NO_STAFF_IN_UNIT');
            }
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * 
     * @param {DataSource} dataSource Widget's dataSource
     */
    this.sortServiceListByServiceName = dataSource => {
        dataSource.orderingAscAlpha = !dataSource.orderingAscAlpha;

        dataSource.data.sort((a, b) => {
            let nameA = a.NMSERVICO.toLowerCase();
            let nameB = b.NMSERVICO.toLowerCase();
            let order = 0;

            order = (nameA < nameB) ? -1 : 1;

            if (!dataSource.orderingAscAlpha) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * 
     * @param {DataSource} dataSource Widget's dataSource
     */
    this.sortServiceListByDate = dataSource => {
        dataSource.orderingAscDate = !dataSource.orderingAscDate;

        dataSource.data.sort((a, b) => {
            let dateA = moment(a.DTCARDIAPME, 'DD/MM/YYYY');
            let dateB = moment(b.DTCARDIAPME, 'DD/MM/YYYY');
            let order = 0;

            order = (dateA < dateB) ? -1 : 1;

            if (!dataSource.orderingAscDate) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Save staff edited register.
     * 
     * @param {Object} staffData Staff edited data.
     * 
     * @returns {Promise}
     */
    this.saveStaffEdit = staffData => {
        return SchoolMealsUtil.getIDBVar('ServicosCardapioEfetivos')
            .then(serviceList => {
                serviceList.forEach(service => {
                    if (service.CDSERVFILI == staffData.CDSERVFILI &&
                        service.CDFILIAL == staffData.CDFILIAL &&
                        service.DTCARDIAPME == staffData.DTCARDIAPME) {

                        service.QTCLIEFECPME = staffData.QTCLIEFECPME;
                        service.QTCOMEFECPME = staffData.QTCOMEFECPME;
                        service.QTDOAEFECPME = staffData.QTDOAEFECPME;
                        service.QTFUNEFECPME = staffData.QTFUNEFECPME;

                        service.edited = true;
                    }
                });

                saveStaffQuantityInMainClient(staffData);
                return SchoolMealsUtil.setIDBVar('ServicosCardapioEfetivos', serviceList);
            });
    };

    /**
     * Save service staff quantities in main client register.
     * 
     * @param {Object} staffData Staff edited data.
     * 
     * @returns {Promise}
     */
    const saveStaffQuantityInMainClient = staffData => {
        return SchoolMealsUtil.getIDBVar('ClientesDoServicoEfetivos')
            .then(clientList => {
                let staffClientList = clientList.filter(client => {
                    return client.CDFILIAL == staffData.CDFILIAL &&
                        client.DTEFETCLPME == staffData.DTCARDIAPME &&
                        client.CDSERVFILI == staffData.CDSERVFILI &&
                        client.CDTIPOCARD == staffData.CDTIPOCARD;
                });

                if (staffClientList.length == 1) {
                    staffClientList[0].QTDOAEFCLPME = staffData.QTDOAEFECPME;
                    staffClientList[0].QTFUNEFCLPME = staffData.QTFUNEFECPME;
                    staffClientList[0].QTCOMEFCLPME = staffData.QTCLIEFECPME;
                    staffClientList[0].TOTAL = staffData.QTCOMEFECPME;
                    staffClientList[0].edited = true;
                    return SchoolMealsUtil.setIDBVar('ClientesDoServicoEfetivos', clientList);
                }

            });
    };

    /**
     * Save all staff client quantities on 
     * 'ClientesDoServicoEfetivos' dataSource if
     * is valid.
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     * @param {Array} staffClientListData
     * @param {Object} staffRegister
     * 
     * @returns {Promise}
     */
    this.saveStaffClientQuantities = (staffClientListData, staffRegister) => {
        return isValidClientStaffQuantities(staffClientListData, staffRegister)
            .then(() => saveStaffClientRegisters(staffClientListData));
    };

    /**
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     * @param {Array} staffClientListData
     * @param {Object} staffRegister
     * 
     * @returns {Promise}
     */
    const isValidClientStaffQuantities = (staffClientListData, staffRegister) => {
        let sum = 0;

        if (staffClientListData.length > 1) {
            sum = staffClientListData.reduce((a, b) => a.TOTAL + b.TOTAL);
        } else if (staffClientListData.length) {
            sum = staffClientListData[0].TOTAL;
        }

        if (sum == staffRegister.QTCOMEFECPME) {
            return Promise.resolve();
        } else {
            return Promise.reject('STAFF_CLIENT_SUM_NOT_EQUAL');
        }
    };

    /**
     * Save staff client registers on 'ClientesDoServicoEfetivos'
     * dataSource.
     * 
     * @param {Array} staffClientListData 
     * 
     * @returns {Promise}
     */
    const saveStaffClientRegisters = staffClientListData => {
        return SchoolMealsUtil.getIDBVar('ClientesDoServicoEfetivos').then(staffDataSet => {
            staffClientListData.forEach(client => {
                let clientReg = staffDataSet.find(staffClient => {
                    return client.CDFILIAL == staffClient.CDFILIAL &&
                        client.DTEFETCLPME == staffClient.DTEFETCLPME &&
                        client.CDSERVFILI == staffClient.CDSERVFILI &&
                        client.CDTIPOCARD == staffClient.CDTIPOCARD &&
                        client.CDCLIENTE == staffClient.CDCLIENTE;
                });

                Object.assign(clientReg, client, {
                    edited: true
                });
            });

            return SchoolMealsUtil.setIDBVar('ClientesDoServicoEfetivos', staffDataSet);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('StaffManagementService', StaffManagementService);
});


// FILE: js/controllers/ChangeUnitController.js
var ChangeUnitController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Instructions to be executed when enter
     * in main widget.
     */
    this.onEnterChangeUnit = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.openWindow('dashboard').then(function() {
                ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
            });
        } else {
            const userData = getLocalVar('userData');
            updateUnitLabel(userData);
        }
    };

    /**
     * Confirm unit change.
     * Set data of new unit in user data
     * at localStorage.
     */
    this.confirmChangeUnit = () => {
        const changeUnitWidget = templateManager.container.getWidget('changeUnit');
        let unitCode = changeUnitWidget.currentRow.CDFILIAL;
        let userData = getLocalVar('userData');

        if (unitCode) {
            if (userData.CDFILIAL == unitCode) {
                ScreenService.showMessage(ScreenService.i18n('ALREADY_LOGGED_UNIT'));
            } else {
                SchoolMealsUtil.changeUnit(unitCode).then(() => {
                    ScreenService.showMessage(ScreenService.i18n('SUCCESSFUL_OPERATION')).then(() => {
                        ScreenService.openWindow('dashboard');
                    });
                });
            }
        } else {
            ScreenService.showMessage(ScreenService.i18n('FILL_REQUIRED_FIELDS'));
        }
    };

    /**
     * Update logged unit label text.
     * 
     * @param {Object} unitData Object containing
     * unit name and code.
     */
    const updateUnitLabel = unitData => {
        const changeUnitWidget = templateManager.container.getWidget('changeUnit');
        changeUnitWidget.getField('loggedUnit').label = unitData.CDFILIAL + ' - ' + unitData.NMFILIAL;
        templateManager.updateTemplate();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ChangeUnitController', ChangeUnitController);
});


// FILE: js/controllers/DashboardController.js
var DashboardController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Translation method.
     */
    const i18n = ScreenService.i18n;

    /**
     * Instructions to be executed when enter at 
     * dashboardWidget.
     * 
     * @param {Widget} dashboardWidget
     */
    this.onEnterMainDashboard = dashboardWidget => {
        setDefaultValuesOnCurrentRow(dashboardWidget);
        buildUnitAndLogCards();
    };

    /**
     * Set values on dashboard currentRow, used to
     * format the welcome message
     * 
     * @param {Widget} dashboardWidget
     */
    const setDefaultValuesOnCurrentRow = dashboardWidget => {
        dashboardWidget.currentRow = {
            lastImport: getLocalVar('hasImportedData') ? moment(getLocalVar('lastImport')).format('DD/MM/YYYY HH:mm') : '-'
        };
    };

    /**
     * Open unit change if have imported data.
     */
    this.openChangeUnit = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.showMessage(i18n('HAVE_TO_IMPORT_DATA')).then(() => {
                UIManager.openContainer('dataImport');
            });
        } else {
            UIManager.openContainer('changeUnit');
        }
    };

    /**
     * Build Unit and error log cards information
     * to show in main dashboard.
     * 
     * @returns {Promise}
     */
    const buildUnitAndLogCards = () => {
        const dsbWidget = templateManager.container.getWidget('mainDashboard');
        const unitCard = dsbWidget.horizontalDashboards.dashboard.dataSource.find(card => card.name == "unitCard");
        const errorLogCard = dsbWidget.horizontalDashboards.dashboard.dataSource.find(card => card.name == "errorLogCard");

        return getCurrentUnitParams().then(params => {
            unitCard.main = unitCard.main.replace('__INFO__', buildUnitCardContent(params));

            return buildLogCardContent(params).then(logCard => {
                errorLogCard.main = errorLogCard.main.replace('__INFO__', logCard);
                templateManager.updateTemplate();
            });
        });
    };

    /**
     * Build info to show at unit card in main
     * screen according to logged unit parameters.
     * 
     * @param {Object} params 
     * @returns {String}
     */
    const buildUnitCardContent = params => {
        let content = '';
        let userData = getLocalVar('userData');

        if (getLocalVar('hasImportedData')) {

            if (userData.CDFILIAL === userData.CDFILCENTPME) {
                content += `${i18n("UNIT_IS_MEALS_CENTER")}<br>`;
            } else if (!userData.CDFILCENTPME) {
                content += `${i18n("UNIT_HAVE_NO_MEALS_CENTER")}<br>`;
            }

            if (params) {
                if (params.IDUTILALMOX === 'S') {
                    content += `${i18n("UNIT_HAVE_STOCK_CTRL_WAREHOUSE")}<br>`;
                }

                if (params.IDUTILLOTE === 'S') {
                    content += `${i18n("UNIT_HAVE_BATCH_STOCK_CTRL")}<br>`;
                }
            }

        }

        if (!content.length) {
            content = i18n("NO_INFO_TO_SHOW");
        }

        return '<br>' + content;
    };

    /**
     * Build info to show at error log card in main
     * screen according to logged unit parameters.
     * 
     * @param {Object} params 
     * 
     * @returns {Promise<String>}
     */
    const buildLogCardContent = params => {
        return SchoolMealsUtil.getIDBVar('LogSolicitacao').then(log => {
            let content = '';

            if (log && log.length) {
                content += `<br>${i18n('LAST_ERROR')}<br>${log[0].DATAHORA}`;
            }

            if (!content.length) {
                content = i18n("NO_INFO_TO_SHOW");
            }

            return content;
        });
    };

    /**
     * Get logged unit related parameters
     * from 'Parametros' datasource.
     * 
     * @returns {Promise}
     */
    const getCurrentUnitParams = () => SchoolMealsUtil.getIDBVar('Parametros')
        .then((parameters) => parameters && parameters.find(param => param.CDFILIAL == getLocalVar('userData').CDFILIAL));
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DashboardController', DashboardController);
});


// FILE: js/controllers/DataExportController.js
var DataExportController = function(ScreenService, SchoolMealsUtil, templateManager, DataExportService, LoginService, DataImportService) {

    /**
     * @var service Reference to DataExportService.
     */
    const service = DataExportService;

    /**
     * Initialize exportation process if have 
     * imported data.
     */
    this.initDataExportProcess = () => {
        if (getLocalVar('hasImportedData')) {
            ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_EXPORT')).then(() => {
                exportData();
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
        }
    };

    /**
     * Export all changed data after validate if user
     * is active.
     */
    const exportData = () => {
        let promises = [
            validateActiveUser,
            updateInventory,
            updateRequestList,
            updateDeletedProducts,
            update_MOVOPSOL,
            updateEditedProducts,
            exportAllDataSources,
            afterExportDataSource
        ];

        SchoolMealsUtil.executePromisesSequentialy(promises);
    };

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     */
    const validateActiveUser = () => service.validateActiveUser().catch(() => {
        ScreenService.showMessage(ScreenService.i18n('INACTIVE_USER')).then(() => {
            LoginService.logout();
        });
    });

    /**
     * Update inventory data based on user changes.
     * Bind inventory's products to inventory register.
     * 
     * @return {Promise} 
     */
    const updateInventory = () => service.bindProductListToInventory();

    /**
     * Export all datasources from local to remote 
     * storage by doing a request and save data.
     * 
     * @return {Promise} 
     */
    const exportAllDataSources = () => service.exportAllDataSources();

    /**
     * Update request list data based on user changes.
     * Set extra requests registers.
     * 
     * @return {Promise} 
     */
    const updateRequestList = () => service.updateRequestList();

    /**
     * Synchronize deleted products.
     * 
     * @return {Promise} 
     */
    const updateDeletedProducts = () => service.removeDeletedProductsFromInexistentRequest();

    /**
     * update MOVOPSOL table log.
     * 
     * @return {Promise} 
     */
    const update_MOVOPSOL = () => service.removeLogRegisterFromInexistentRequest();

    /**
     * Update edited products.
     * 
     * @return {Promise} 
     */
    const updateEditedProducts = () => service.removeEditedProductsFromInexistentRequest();

    /**
     * After export dataSources, import again secondary
     * datasources using same paramters used in importation
     * routine.
     * 
     * @param {Array} exportResultDataset 
     * 
     * @returns {Promise}
     */
    const afterExportDataSource = exportResultDataset => service.afterExportDataSource(exportResultDataset, DataImportService);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataExportController', DataExportController);
});


// FILE: js/controllers/DataImportController.js
var DataImportController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, DataImportService, LoginService) {

    /**
     * Make all validations to check if is valid to 
     * import data from selected units in the form.
     */
    this.initDataImportProcess = () => {
        let dataImportForm = templateManager.container.getWidget('dataImport');
        let validationObj = Object.assign({}, dataImportForm.currentRow);

        validateUserRelatedUnits(validationObj).then(relatedUnits => {
            validationObj.relatedUnits = relatedUnits;
            validationObj.mappedRelatedUnits = relatedUnits.map(unit => unit.CDFILIAL);

            validateIntervalToImportData(dataImportForm)
                .then(hasImportedData)
                .then(validateActiveUser)
                .then(() => {
                    importData(validationObj);
                }).catch(error => {
                    console.warn(error);
                });
        });
    };

    /**
     * Validate if all selected units are related
     * to current user.
     * 
     * @param {Object} validationObj Data import parameters.
     * 
     * @return {Promise}
     */
    const validateUserRelatedUnits = validationObj => {
        let selectedUnits = validationObj.CDFILIAL;

        if (selectedUnits && selectedUnits.length) {
            return DataImportService.getUserRelatedUnits(getLocalVar('userData').CDOPERADOR, selectedUnits).then(units => {
                if (units.length !== selectedUnits.length) {
                    return ScreenService.confirmMessage(ScreenService.i18n('NOT_RELATED_UNITS')).then(() => units);
                } else {
                    return Promise.resolve(units);
                }
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n('SELECT_UNIT_TO_IMPORT'));
            return Promise.reject();
        }
    };

    /**
     * Validate selected interval to import data.
     * Selected interval is required, and 
     * must be less or equal 31 days.
     * 
     * @param {Widget} dataImportForm 
     */
    const validateIntervalToImportData = dataImportForm => {
        let start = dataImportForm.currentRow.INTERVAL_START;
        let end = dataImportForm.currentRow.INTERVAL_END;

        if (!start || !end) {
            ScreenService.showMessage(ScreenService.i18n('SELECT_COMPLETE_INTERVAL'));
            return Promise.reject();
        } else {
            start = moment(start);
            end = moment(end).add(1, "days");
            let diff = end.diff(start, "days");

            if (diff > 31) {
                ScreenService.showMessage(ScreenService.i18n('SELECT_31_DAYS_INTERVAL'));
                return Promise.reject();
            } else {
                return Promise.resolve();
            }
        }
    };

    /**
     * Check if already has imported data.
     * 
     * @return {Promise} resolved promise
     * if should continue importation process. 
     */
    const hasImportedData = () => {
        if (getLocalVar('hasImportedData')) {
            return ScreenService.confirmMessage(ScreenService.i18n('HAS_IMPORTED_DATA'));
        } else {
            return Promise.resolve();
        }
    };

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     * 
     * @return {Promise}
     */
    const validateActiveUser = () => {
        let userMail = getLocalVar('userData').DSEMAILOPER;
        let nrorg = getLocalVar('userData').NRORG;

        return LoginService.checkActiveUser(userMail, nrorg).then(isActive => {
            if (isActive) {
                return Promise.resolve();
            } else {
                ScreenService.showMessage(ScreenService.i18n('INACTIVE_USER'));
                LoginService.logout();
                return Promise.reject();
            }
        });
    };

    /**
     * Import application data according to parameters
     * selected in data import screen.
     * 
     * @param {Object} params Parameters to execute entire
     * process.
     */
    const importData = (params) => {
        return DataImportService.clearImportData() //@todo finalize
            .then(() => importRequestProductsData(params))
            .then(importProductsData)
            .then(importGeneralData)
            .then(finalizeImportProcess);
    };

    /**
     * Import Products of system requests from backend.
     * 
     * @param {Object} params 
     * 
     * @return {Promise}
     */
    const importRequestProductsData = params => {
        let requestedProductsParams = {
            nrorg: getLocalVar('userData').NRORG,
            start: moment(params.INTERVAL_START).format('DD/MM/YYYY'),
            end: moment(params.INTERVAL_END).format('DD/MM/YYYY'),
            period: 5
        };

        return DataImportService.importRequestProductsData(requestedProductsParams, params.relatedUnits).then(requestedProducts => {
            SchoolMealsUtil.setIDBVar('requestedProducts', requestedProducts, true);

            let reducedProds = requestedProducts.map(product => Object.assign({
                CDFILSOLI: product.CDFILSOLI,
                CDPRODUTO: product.CDPRODUTO,
                NMPRODUTO: product.NMPRODUTO
            }));

            removeDuplicateProducts(reducedProds);
            SchoolMealsUtil.setIDBVar('reducedUniqueRequestedProducts', reducedProds, true);

            return params;
        });
    };

    /**
     * Remove all duplicates of products in a array, using
     * SchoolMealsUtil -> spliceOnCondition function.
     * 
     * @param {Array} productArray 
     */
    const removeDuplicateProducts = productArray => {
        spliceOnCondition(productArray, product => {
            let ocurrences = 0;

            productArray.forEach(reduceProd => {
                let reduceProdData = [reduceProd.CDFILSOLI, reduceProd.CDPRODUTO];
                let currentProductData = [product.CDFILSOLI, product.CDPRODUTO];

                if (JSON.stringify(reduceProdData) == JSON.stringify(currentProductData)) {
                    ocurrences++;
                }
            });

            return ocurrences > 1;
        });
    };

    /**
     * Import system products data.
     * 
     * @param {Object} params Search data params.
     * 
     * @returns {Promise}
     */
    const importProductsData = params => {
        return DataImportService.importProductsData(params, params.relatedUnits).then(products => {
            let productsObject = {};

            products.forEach(product => {
                if (productsObject.hasOwnProperty(product.CDPRODUTO)) {
                    productsObject[product.CDPRODUTO].units.push(product.CDFILIAL);
                } else {
                    productsObject[product.CDPRODUTO] = product;
                    productsObject[product.CDPRODUTO].units = [product.CDFILIAL];
                }
            });

            SchoolMealsUtil.setIDBVar('productsDataset', products, true);
            SchoolMealsUtil.setIDBVar('productsDatasetObject', productsObject, true);

            params.productsData = productsObject;

            return params;
        });
    };

    /**
     * Import general data, related with working of 
     * all parts of system.
     * 
     * @param {Object} params 
     */
    const importGeneralData = params => {
        params.period = 5;

        return DataImportService.importGeneralData(params).then(generalData => {
            Object.keys(generalData).forEach(datasetName => {
                SchoolMealsUtil.setIDBVar(datasetName, generalData[datasetName], true);
            });

            return params;
        });

    };

    /**
     * Finalize the importation process of all system
     * necessary data.
     * 
     * @param {Object} params Data importation parameters.
     */
    const finalizeImportProcess = params => {
        setLocalVar('hasImportedData', true);
        setLocalVar('lastImport', moment());

        let userData = getLocalVar('userData');
        let currentUnit = userData.CDFILIAL;
        let msg = ScreenService.i18n('IMPORTED_DATA');

        userData = Object.assign(userData, {
            INTERVAL_START: params.INTERVAL_START,
            INTERVAL_END: params.INTERVAL_END
        });
        setLocalVar('userData', userData);

        if (!Object.keys(params.productsData).length) {
            msg += "<br>" + ScreenService.i18n('NO_IMPORTED_PRODUCT');
        }

        setUnitChangeDataSource(params);
        setSystemEmptyDataSources();

        if (params.mappedRelatedUnits.includes(currentUnit)) {
            SchoolMealsUtil.changeUnit(currentUnit);

            ScreenService.openWindow('dashboard').then(function() {
                ScreenService.showMessage(msg);
            });
        } else {
            setLocalVar('CDFILIAL', false);
            ScreenService.showMessage(msg).then(() => {
                ScreenService.openWindow('changeUnit').then(() => {
                    ScreenService.showMessage(ScreenService.i18n('PLEASE_CHANGE_UNIT'));
                });
            });
        }
    };

    /**
     * Set units dataSource, used to change
     * unit.
     * 
     * @param {Object} params 
     */
    const setUnitChangeDataSource = params => {
        let availableUnitsToChange = [];

        SchoolMealsUtil.getIDBVar('Units', true).then(units => {
            params.CDFILIAL.forEach(unitCode => {
                availableUnitsToChange.push(units.find(unit => unit.CDFILIAL == unitCode));
            });

            SchoolMealsUtil.setIDBVar('availableUnitsToChange', availableUnitsToChange, true);
        });
    };

    /**
     * Set all unseteds system empty dataSources.
     */
    const setSystemEmptyDataSources = () => {
        SchoolMealsUtil.setIDBVar('ProdutosSolicitacao', []);
        SchoolMealsUtil.setIDBVar('ProdutosInventario', []);
        SchoolMealsUtil.setIDBVar('ProdutosExcluidos', []);
        SchoolMealsUtil.setIDBVar('NovaSolicitacao', []);
        SchoolMealsUtil.setIDBVar('Movopsol', []);
        SchoolMealsUtil.setIDBVar('Inventarios', []);
        SchoolMealsUtil.setIDBVar('nextInventoryProductCode', {});
    };

    /**
     * On change date start, set end to
     * 30 days later.
     * 
     * @param {Object} field
     */
    this.onChangeDateIntervalStart = field => {
        const importForm = field.widget;
        let startDate = moment(field.value());

        startDate.add(30, 'days');

        importForm.currentRow.INTERVAL_END = startDate.format('YYYY-MM-DD');
        templateManager.updateTemplate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataImportController', DataImportController);
});


// FILE: js/controllers/ExportationErrorLogController.js
var ExportationErrorLogController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Open dependent registers of
     * selected error log entry.
     */
    this.openErrorLogDetails = () => {
        let errorDetails = templateManager.container.getWidget("errorProductList");
        let selectedReg = templateManager.container.currentItem;

        setFilterOnErrorDetail(selectedReg, errorDetails);
        UIManager.goToWidget(errorDetails);
    };

    /**
     * Set filter into register detail widget to 
     * show dependent registers related to error.
     * 
     * @param {Object} errorRegister 
     * @param {Object} errorWidget 
     */
    const setFilterOnErrorDetail = (errorRegister, errorWidget) => {
        errorWidget.dataSourceFilter = [{
            "name": "PARENT",
            "operator": "=",
            "value": errorRegister.ID
        }];
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ExportationErrorLogController', ExportationErrorLogController);
});


// FILE: js/controllers/LoginController.js
var LoginController = function(LoginService, ScreenService, SchoolMealsUtil, UIManager, templateManager) {
    var self = this;

    function checkLoginInfo(email, pass) {
        return (SchoolMealsUtil.emailValidation(email) && SchoolMealsUtil.passValidation(pass));
    }

    function setUserDataOnApplication(result) {
        let user = result.dataset.User;
        setLocalVar('isLogged', true);
        setLocalVar('userData', user);
        setLocalVar('CDFILIAL', user.CDFILIAL);
        SchoolMealsUtil.setIDBVar('Units', result.dataset.Units, true);

        LoginService.getProductsGroups().then(productsGroups => {
            SchoolMealsUtil.setIDBVar('productsGroups', productsGroups.dataset.GruposProduto, true);
        });

        LoginService.getProductMask().then(productMask => {
            setLocalVar('productMask', productMask);
        });
    }

    this.onEnter = function(widget) {
        this.setVisibilityForValidQRCode(widget);
        this.setUpDNS();
        let lastLogin = '';
        lastLogin = getLocalVar("userData") && getLocalVar("userData") != null ? getLocalVar("userData").DSEMAILOPER : lastLogin;
        setLocalVar('lastLogin', lastLogin);
        widget.currentRow.email = lastLogin;

        if (getLocalVar('isLogged') === true && getLocalVar("userData")) {
            ScreenService.openWindow('dashboard');
        } else {
            setLocalVar('isLogged', false);
            setLocalVar('userData', '');
        }
    };
    this.onAccountEnter = function(widget) {
        widget.newRow();
        widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
    };

    this.login = function(widget) {
        let email = widget.currentRow.email || '';
        let password = widget.currentRow.password || '';

        if (!checkLoginInfo(email, password)) {
            ScreenService.showMessage('Preencha os campos com email e senha!');
        } else {
            LoginService.login(email, password).then(function(result) {
                setUserDataOnApplication(result);
                ScreenService.openWindow('dashboard');
            }).catch(function(callback) {
                setLocalVar('isLogged', false);
                setLocalVar('userData', '');
                const errorMessage = (callback.data && callback.data.error) || callback.error || "Um erro inesperado aconteceu";
                ScreenService.showMessage(ScreenService.i18n(errorMessage), "error");
            });
        }
    };

    this.logout = function(skipQuestion = false) {
        if (!skipQuestion) {
            ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_EXIT')).then(() => {
                LoginService.logout();
            });
        } else {
            LoginService.logout();
        }
    };

    this.changeUnit = function(widget) {
        let newFilial = widget.currentRow.CDFILIAL;
        ScreenService.confirmMessage(ScreenService.i18n('Tem certeza que deseja trocar para filial ') + newFilial + "?", 'question',
            function() {
                let userData = getLocalVar('userData');
                userData.newFilial = newFilial;
                LoginService.changeUnit(userData).then(function(result) {
                    setUserDataOnApplication(result);
                }).catch(function(callback) {
                    self.logout();
                    ScreenService.showMessage(ScreenService.i18n(callback.data.error), "error");
                });
            },
            function() {
                widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
            });
    };

    this.saveQRCodeData = (qrCodeData) => {
        for (const qrCodePropertyKey in qrCodeData) {
            const qrCodeProperty = qrCodeData[qrCodePropertyKey];
            localStorage.setItem(qrCodePropertyKey, qrCodeProperty);
        }
    };

    this.validateQRCode = (qrCodeValue) => {
        let splitedData = qrCodeValue.split(';');
        if (!qrCodeValue || qrCodeValue.length === 0) {
            return Promise.reject("Invalid QRCode");
        }
        let qrCodeData = {
            serviceURL: splitedData[0],
            NRORG: splitedData[1]
        };

        let concantInURL = window.hasServiceInDNS ? "/backend/service/index.php" : "/backend/index.php";
        qrCodeData.serviceURL += concantInURL;
        templateManager.updateURL(qrCodeData.serviceURL);

        return new Promise((resolve, reject) => {
            LoginService.checkServerRepository(qrCodeData.NRORG).then(() => {
                qrCodeData.isQRCodeValid = true;
                resolve(qrCodeData);
            }).catch(() => {
                reject();
            });
        });
    };

    this.showQRCodeScanner = () => {
        let qrCodeWidget = templateManager.container.getWidget('qrCodeScanner');
        UIManager.goToWidgetByName('qrCodeScanner');
        this.readQRCode(qrCodeWidget);
        if ('QRScanner' in window) {
            qrCodeWidget.__showCameraPreview();
        }
    };

    this.readQRCode = function(qrCodeWidget) {
        qrCodeWidget.startScan(false).then((qrCodeValue) => {
            this.validateQRCode(qrCodeValue).then((qrCodeData) => {
                this.saveQRCodeData(qrCodeData);
                this.setVisibilityForValidQRCode();
                this.setUpDNS();
                UIManager.goBack();
            }).catch(() => {
                ScreenService.showMessage("QRCode Inválido");
                this.readQRCode(qrCodeWidget);
            });
        });
    };

    this.setVisibilityForValidQRCode = (widget) => {
        widget = widget || templateManager.container.getWidget('login');
        let validQRCode = localStorage.getItem('isQRCodeValid');
        if (validQRCode || widget.shouldIgnoreQrcode) {
            widget.fields.forEach(field => field.isVisible = true);
            let loginAction = widget.actions.find(action => action.name === 'login');
            loginAction.isVisible = true;
        }
    };

    this.setUpDNS = () => {
        let newURL = localStorage.getItem('serviceURL');
        if (newURL) {
            templateManager.updateURL(newURL);
        }
    };

    this.onManualQRCodeTyped = (typedValue) => {
        this.validateQRCode(typedValue).then((qrCodeData) => {
            this.saveQRCodeData(qrCodeData);
            this.setVisibilityForValidQRCode();
            this.setUpDNS();
            UIManager.goBack();
        }).catch((err) => {
            ScreenService.showMessage("QRCode Inválido");
        });
    };

    this.log = (string) => console.log(string);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});


// FILE: js/controllers/ManageInventoryController.js
var ManageInventoryController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, ManageInventoryService) {

    /**
     * @var service Reference to ManageInventoryService.
     */
    const service = ManageInventoryService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);
    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(service.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(service.showMessage('PLEASE_CHANGE_UNIT'));
        } else {
            checkIfIsValidCreateNewInventory().then(() => openScreenWithDashboardPushState('manageInventory'));
        }

    };

    /**
     * Check if is valid to create new inventory.
     * Is valid if: Current unit is not a meals center, 
     * if unit have a meals center, if have no stock control
     * by warehouse and have no batch stock control.
     * 
     * If is invalid, show a warning message to user, explaining
     * the error.
     * 
     * @return {Promise}
     */
    const checkIfIsValidCreateNewInventory = () => SchoolMealsUtil.getIDBVar('Parametros').then(parameters => {
        let valid = false;
        let msg = '';

        let filteredParams = parameters.find(param => param.CDFILIAL == getLocalVar('userData').CDFILIAL);

        if (service.isCurrentUnitMealsCenter()) {
            msg = ScreenService.i18n('UNIT_IS_MEALS_CENTER') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (!service.currentUnitHaveMealsCenter()) {
            msg = ScreenService.i18n('UNIT_HAVE_NO_MEALS_CENTER') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (filteredParams.IDUTILALMOX == 'S') {
            msg = ScreenService.i18n('UNIT_HAVE_STOCK_CTRL_WAREHOUSE') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else if (filteredParams.IDUTILLOTE == 'S') {
            msg = ScreenService.i18n('UNIT_HAVE_BATCH_STOCK_CTRL') + '. ' + ScreenService.i18n('CANNOT_CREATE_INVENTORY');
        } else {
            valid = true;
        }

        if (!valid)
            service.showMessage(msg);

        return valid;
    });

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Open formulary to add new inventory
     * register. Check if is valid to create new
     * inventory before open form.
     */
    this.openFormToAddInventory = () => {
        checkIfIsValidCreateNewInventory().then(valid => {
            if (valid) {
                const newInventoryForm = templateManager.container.getWidget('newInventoryForm');
                newInventoryForm.currentRow = {};
                UIManager.goToWidget(newInventoryForm);
            }
        });
    };

    /**
     * Save new inventory register created by user
     * if all data is filled correctly and there's no
     * inventory with same date.
     */
    this.saveNewInventory = () => {
        const newInventoryForm = templateManager.container.getWidget('newInventoryForm');

        if (validateInventoryToSave(newInventoryForm.currentRow)) {
            checkExistentInventory(newInventoryForm.currentRow).then(existsInventory => {
                if (!existsInventory) {
                    service.insertInventoryRegister(newInventoryForm.currentRow);
                }
            });
        }
    };

    /**
     * Check if all informations are filled,
     * if inventory date is less than or equal to
     * importation date.
     * 
     * Show to user a warning message if something
     * goes wrong.
     * 
     * @param {Object} inventoryRow Object row
     * containing inventory information.
     * 
     * @return {Boolean}
     */
    const validateInventoryToSave = inventoryRow => {
        let isValid = true;

        if (!service.isInventoryRequiredFieldsFilled(inventoryRow)) {
            service.showMessage('FILL_REQUIRED_FIELDS');
            isValid = false;
        } else if (!service.isInventoryDateValid(inventoryRow)) {
            isValid = false;
            service.showMessage('INVALID_INVENTORY_DATE');
        }

        return isValid;
    };

    /**
     * Check existance of a inventory date in local
     * and remote storage.
     * 
     * Show to user a warning message if something
     * goes wrong.
     * 
     * @param {Object} inventoryRow Object row
     * containing inventory information.
     * 
     * @return {Promise}
     */
    const checkExistentInventory = inventoryRow => {
        return service.existsInventory(inventoryRow, true).then(existsRemote => {

            if (existsRemote) {

                service.showMessage('ALREADY_EXISTS_INVENTORY_REMOTE');
                return Promise.resolve(existsRemote);

            } else {

                return service.existsInventory(inventoryRow, false).then(existsLocal => {

                    if (existsLocal)
                        service.showMessage('ALREADY_EXISTS_INVENTORY_LOCAL');
                    return existsLocal;
                });
            }

        });
    };

    /**
     * Open product list of user's selected inventory
     * register.
     */
    this.openInventoryProductList = () => {
        let currentItem = templateManager.container.currentItem;
        const productListWidget = templateManager.container.getWidget('inventoryProductList');

        productListWidget.label = currentItem.DTINVENTRETI + ' - ' + currentItem.DSINVENTRETI;
        setProductListFilter(productListWidget, currentItem).then(() => {
            UIManager.goToWidget(productListWidget);
        });
    };

    /**
     * Set filter on product list according to inventory
     * selected by user on inventory lists.
     * 
     * @param {Widget} productListWidget 
     * @param {Object} selectedInventory 
     * 
     * @returns {Promise}
     */
    const setProductListFilter = (productListWidget, selectedInventory) => {
        productListWidget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": selectedInventory.CDFILIAL
            },
            {
                "name": "NRINVENTRETI",
                "operator": "=",
                "value": selectedInventory.NRINVENTRETI
            }
        ];
        return productListWidget.reload();
    };

    /**
     * Open form to add new product related with
     * current inventory.
     */
    this.openFormToAddProduct = () => {
        const productForm = templateManager.container.getWidget('productForm');

        productForm.label = ScreenService.i18n('ADD_PRODUCT_LABEL');
        productForm.currentRow = {
            __is_new: true
        };
        productForm.getField('NMPRODUTO').isDisabled = false;

        UIManager.goToWidget(productForm);
    };

    /**
     * Open tabbed data to edit products
     * existent registers.
     */
    this.openProductEdit = () => {
        const productListWidget = templateManager.container.getWidget('inventoryProductList');
        const productEdit = templateManager.container.getWidget('productEdit');

        templateManager.container.fabVisibility = false;
        productEdit.dataSource = productListWidget.dataSource;

        UIManager.goToWidget(productEdit).then(() => {
            let index = productListWidget.dataSource.data.indexOf(templateManager.container.currentSubItem);
            productEdit.activateTab(index);
        });
    };

    /**
     * Get back from product edit widget.
     * Reload dataSource to refresh all unsaved
     * data.
     */
    this.getBackFromProductEdit = () => {
        const productListWidget = getWidget('inventoryProductList');
        productListWidget.dataSource.reload();
        setSaveButtonAcessibility(false);
        UIManager.goBack();
    };

    /**
     * Enable or disable save button.
     * 
     * @param {Boolean} enable
     */
    const setSaveButtonAcessibility = enable => {
        const productEdit = getWidget('productEdit');
        productEdit.getAction('saveProductEdit').isDisabled = !enable;
        templateManager.updateTemplate();
    };

    /**
     * Instructions to be executed when 
     * change active tab.
     * Reset save button acessibility
     * 
     * @param {Widget} productEditWidget 
     */
    this.onChangeActiveTab = productEditWidget => {
        setSaveButtonAcessibility(productEditWidget.getActiveTabData().changed);
    };

    /**
     * Instructions to be executed when change product
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeProductQuantity = () => {
        const productEditWidget = getWidget('productEdit');

        let activeData = productEditWidget.getActiveTabData();
        if (!activeData.changed) {
            activeData.changed = true;
            setSaveButtonAcessibility(true);
            activeData.tabLabel = activeData.tabLabel.replace("", "\u25CF ");
        }
    };

    /**
     * Save product register. If it's a new register
     * call function to insert new product.
     */
    this.saveProductForm = () => {
        const productForm = templateManager.container.getWidget('productForm');
        let inventory = productForm.currentRow;

        saveNewInventoryProduct(inventory);
    };

    /**
     * Save product register edited on tabbed
     * widget.
     */
    this.saveProductTabbedEdit = () => {
        const productEditWidget = getWidget('productEdit');
        saveEditedProduct(productEditWidget.getActiveTabData());
    };

    /**
     * Insert product register into related inventory
     * register. Show a sucess message and get back to
     * product list if nothing goes wrong.
     * 
     * @param {Object} product
     * 
     * @returns {Promise}
     */
    const saveNewInventoryProduct = product => {
        let currentInventory = templateManager.container.currentItem;

        return service.saveNewInventoryProduct(product, currentInventory).then(() => {
            const productListWidget = templateManager.container.getWidget('inventoryProductList');
            productListWidget.dataSource.reload();
            UIManager.goBack();
            ScreenService.notificationMessage('PRODUCT_INVENTORY_ADDED');
        });
    };

    /**
     * Save edited inventory's product register. 
     * 
     * @param {Object} product Object containing
     * product data.
     * 
     * @returns {Promise}
     */
    const saveEditedProduct = product => {
        return service.saveEditedProduct(product).then(() => {
            ScreenService.notificationMessage('PRODUCT_INVENTORY_EDITED');
            product.changed = false;
            setSaveButtonAcessibility(false);
            product.tabLabel = product.tabLabel.replace("\u25CF ", "");
        });
    };

    /**
     * Remove inventory register after user swipe 
     * card left and click 'yes' on question popup.
     * Check if is empty before remove and ask to user
     * to confirm so.
     */
    this.removeInventoryRegister = () => {
        let itemToExclude = templateManager.container.itemToExclude;

        service.checkIsInventoryEmpty(itemToExclude).then(isEmpty => {
            let remove = item => {
                service.removeInventoryRegister(item).then(() => {
                    afterRemoveInventory();
                });
            };

            if (isEmpty) {
                remove(itemToExclude);
            } else {
                ScreenService.confirmMessage('INVENTORY_HAS_PRODUCT_WANT_DELETE').then(() => {
                    remove(itemToExclude);
                });
            }
        });
    };

    /**
     * After remove inventory, show sucess notification
     * and reload datasource from inventory list widget
     * to show new set of registers.
     */
    const afterRemoveInventory = () => {
        ScreenService.notificationMessage(ScreenService.i18n('INVENTORY_REMOVED'));
        const inventoryList = templateManager.container.getWidget('manageInventory');
        inventoryList.reload();
    };

    /**
     * After remove selected inventorys, show sucess 
     * notification and reload datasource from inventory 
     * list widget to show new set of registers.
     */
    const afterRemoveSelectedInventory = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');

        ScreenService.notificationMessage(ScreenService.i18n('SELECTED_INVENTORY_REMOVED'));
        inventoryList.dataSource.clearCheckedRows();
        changeFabAction();
        inventoryList.reload();
    };

    /**
     * Remove inventory product register after user swipe 
     * card left and click 'yes' on question popup.
     */
    this.removeInventoryProductRegister = () => {
        let itemToExclude = templateManager.container.itemToExclude;

        ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_REMOVE_REG')).then(() => {
            service.removeInventoryProductRegister(itemToExclude).then(() => {
                ScreenService.notificationMessage(ScreenService.i18n('PRODUCT_INVENTORY_REMOVED'));
            });
        });
    };

    /**
     * Instructions to be executed after select
     * an inventory register.
     */
    this.afterSelectInventory = () => {
        changeFabAction();
    };

    /**
     * Instructions to be executed after unselect
     * an inventory register.
     */
    this.afterUnselectInventory = () => {
        changeFabAction();
    };

    /**
     * Change fab action between add action and exclusion
     * action when user long tap and select one or more
     * registers.
     */
    const changeFabAction = () => {
        const container = templateManager.container;
        const inventoryList = container.getWidget('manageInventory');

        if (inventoryList.hasCheckedRows()) {
            inventoryList.fabAction = "ManageInventoryController.removeSelectedInventorys();";
            container.fabActionIcon = "fa-trash-alt";
        } else {
            inventoryList.fabAction = "ManageInventoryController.openFormToAddInventory();";
            container.fabActionIcon = "fa-plus";
        }

        templateManager.updateTemplate();
    };

    /**
     * Remove selected inventorys registers.
     * Check if any of the selected registers have
     * products relateds with it.
     */
    this.removeSelectedInventorys = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');

        checkSelectedInventorysAreEmpty().then(inventorysAreEmpty => {
            let msg = inventorysAreEmpty ? 'WANT_REMOVE_SELECTED_ITEMS' : 'SELECTED_INVENTORY_HAS_PRODUCT_WANT_DELETE';

            ScreenService.confirmMessage(msg).then(() => {
                service.removeSelectedInventorys(inventoryList.getCheckedRows()).then(() => {
                    afterRemoveSelectedInventory();
                });
            });
        });
    };

    /**
     * Check if any of the selected inventorys
     * to be removed have products relateds with it.
     * 
     * @returns {Promise}
     */
    const checkSelectedInventorysAreEmpty = () => {
        const inventoryList = templateManager.container.getWidget('manageInventory');
        let selectedRegisters = inventoryList.getCheckedRows();

        let checkIsInventoryEmptyPromise = [];
        selectedRegisters.forEach(inventory => {
            checkIsInventoryEmptyPromise.push(service.checkIsInventoryEmpty(inventory));
        });

        return Promise.all(checkIsInventoryEmptyPromise).then(checkedInventorys => !checkedInventorys.includes(false));
    };

    /**
     * Instructions to be executed after select
     * an inventory's product register.
     * Hide add button and show exclusion button.
     */
    this.afterSelectInventoryProduct = () => {
        setActionVisibilityOnActiveWidget('addProduct', false);
        setActionVisibilityOnActiveWidget('removeSelectedProducts', true);
    };

    /**
     * Instructions to be executed after unselect
     * an inventory's product register.
     * Check if has checked rows. If not, show add
     * button and hide remove button.
     */
    this.afterUnselectInventoryProduct = () => {
        const productListWidget = templateManager.container.getWidget('inventoryProductList');

        if (!productListWidget.hasCheckedRows()) {
            setActionVisibilityOnActiveWidget('addProduct', true);
            setActionVisibilityOnActiveWidget('removeSelectedProducts', false);
        }
    };

    /**
     * Set visibility of actions in active widget to
     * parameter value.
     * 
     * @param {String} actionName
     * @param {Boolean} visible 
     */
    const setActionVisibilityOnActiveWidget = (actionName, visible) => {
        const productListWidget = templateManager.container.getActiveWidget();
        productListWidget.getAction(actionName).isVisible = visible;
        templateManager.updateTemplate();
    };

    /**
     * Instructions to be executed when enter product
     * list widget. Set addProduct button as visible and
     * remove button as invisible.
     */
    this.onEnterInventoryProductList = () => {
        setActionVisibilityOnActiveWidget('addProduct', true);
        setActionVisibilityOnActiveWidget('removeSelectedProducts', false);
    };

    /**
     * Remove selected registers of inventory's products.
     * After remove, show a success message, reload data and
     * reset buttons visibility.
     */
    this.removeSelectedProducts = () => {
        const productListWidget = templateManager.container.getActiveWidget();

        service.removeSelectedProducts(productListWidget.getCheckedRows()).then(() => {
            ScreenService.notificationMessage('SELECTED_PRODUCT_INVENTORY_REMOVED');
            productListWidget.reload();
            this.onEnterInventoryProductList();
        });
    };

    /**
     * Copy last exported inventory productset to the current
     * one.
     */
    this.copyLastExportedInventory = () => {
        let currentItem = templateManager.container.currentItem;
        let inventoryProductList = getWidget('inventoryProductList');

        ScreenService.confirmMessage('WANT_COPY_LAST_EXP_INVT').then(() => {
            service.copyLastExportedInventory(currentItem).then(() => {
                ScreenService.successNotification('SUCCESSFUL_OPERATION');
                inventoryProductList.dataSource.reload();
            }).catch((error) => {
                ScreenService.notificationMessage(error);
            });
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ManageInventoryController', ManageInventoryController);
});


// FILE: js/controllers/PurchaseRequestController.js
var PurchaseRequestController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, PurchaseRequestService) {

    /**
     * @var service Reference to PurchaseRequestService.
     */
    const service = PurchaseRequestService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);

    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(ScreenService.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(ScreenService.showMessage('PLEASE_CHANGE_UNIT'));
        } else {
            openScreenWithDashboardPushState('purchaseRequest');
        }

    };

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     * 
     * @param {String} screenName Name of container to open.
     * 
     * @returns {Promise}
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Open form to add new purchase request
     * register.
     */
    this.openFormToAddPurchaseRequest = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');
        newPurchaseRequestForm.currentRow = {};
        UIManager.goToWidget(newPurchaseRequestForm);
    };

    /**
     * Instructions to be executed when open form
     * to add new purchase request.
     */
    this.onEnterNewRequestForm = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');
        filterProductListByUnitParams(newPurchaseRequestForm);
        setNewPurchaseRequestValues(newPurchaseRequestForm);
        newPurchaseRequestForm.getField('DSJUSTSOLEXT').isVisible = false;
    };

    /**
     * Filter product list by product type = "E" when unit
     * has parameter inside 'Parametros' dataSource and
     * parameters attribute 'IDUTPRODPREF' is equal to 'N'.
     * 
     * @param {Widget} newPurchaseRequestForm Form widget to 
     * add new purchase request.
     */
    const filterProductListByUnitParams = newPurchaseRequestForm => {
        let loggedUnit = getLocalVar('userData').CDFILIAL;

        service.getUnitParameters(loggedUnit).then(parameterArray => {
            let productField = newPurchaseRequestForm.getField('NMPRODUTO');
            service.setAddProductFieldFilter(productField, parameterArray);
            productField.reload();
        });
    };

    /**
     * Set default values on new purchase request register's
     * current row.
     * 
     * @param {Widget} newPurchaseRequestForm 
     */
    const setNewPurchaseRequestValues = newPurchaseRequestForm => {
        newPurchaseRequestForm.currentRow.IDITMSOLEXTRA = 'N';
        newPurchaseRequestForm.currentRow.__is_new = true;
        /**
         * @todo
         * newPurchaseRequestForm.getField('DATE').min = moment().add(1, 'days').format('L');
         */
    };

    /**
     * Instructions to be executed when user toggle
     * extra product option. Change extra justification
     * field visibility.
     * 
     * @param {Widget} purchaseRequestForm Formulary widget
     * from add or edit register.
     */
    this.onToggleExtraProduct = (purchaseRequestForm, editing) => {
        toggleExtraJustificationField(purchaseRequestForm, editing);
    };

    /**
     * Set extra justification field visibility according
     * to product extra toggle button state and set value
     * to quantity justification field's value.
     * 
     * @param {Widget} purchaseRequestForm Formulary widget
     * from add or edit register.
     */
    const toggleExtraJustificationField = (purchaseRequestForm, editing) => {
        let extraJustificationField = purchaseRequestForm.getField('DSJUSTSOLEXT');
        let currentRegister = editing ? purchaseRequestForm.getActiveTabData() : purchaseRequestForm.currentRow;

        extraJustificationField.isVisible = currentRegister.PRODUTOEXTRA;

        if (extraJustificationField.isVisible) {
            currentRegister.DSJUSTSOLEXT = currentRegister.DSJUSTOPER;
        } else {
            currentRegister.DSJUSTSOLEXT = null;
        }

        if (editing) {
            extraJustificationField.updateDescriptionText();
        }

        templateManager.updateTemplate();
    };

    /**
     * Create new product purchase request with user's data
     * input if all data is valid to proceed.
     */
    this.createNewProductPurchaseRequest = () => {
        let newPurchaseRequestForm = getWidget('newPurchaseRequestForm');

        service.createNewProductPurchaseRequest(newPurchaseRequestForm.currentRow).then(() => {

            ScreenService.notificationMessage('PRODUCT_ADDED');
            UIManager.goBack();
            getWidget('purchaseRequestList').reload();

        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };

    /**
     * Instructions to be executed after select
     * a product request register.
     */
    this.afterSelectRequestRegister = () => {
        changeFabAction();
    };

    /**
     * Instructions to be executed after unselect
     * a product request register.
     */
    this.afterUnselectRequestRegister = () => {
        changeFabAction();
    };

    /**
     * Change fab action between add action and exclusion
     * action when user long tap and select one or more
     * registers.
     */
    const changeFabAction = () => {
        const container = templateManager.container;
        const purchaseRequestList = getWidget('purchaseRequestList');

        if (purchaseRequestList.hasCheckedRows()) {
            purchaseRequestList.fabAction = "PurchaseRequestController.removeSelectedProducts();";
            container.fabActionIcon = "fa-trash-alt";
        } else {
            purchaseRequestList.fabAction = "PurchaseRequestController.openFormToAddPurchaseRequest();";
            container.fabActionIcon = "fa-plus";
        }

        templateManager.updateTemplate();
    };

    /**
     * Remove  product register if is valid to.
     * If is not, show a message explaining what's wrong
     * with operation.
     * 
     * @param {Object} itemToExclude Object containing product
     * data to exclude.
     */
    const removeProductPurchaseRequest = itemToExclude => {
        return service.removeValidProductPurchaseRequest(itemToExclude)
            .then(() => {
                ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
                getWidget('purchaseRequestList').reload();
                return true;
            })
            .catch(errorCode => {
                ScreenService.showMessage(errorCode);
                return false;
            });
    };

    /**
     * Remove selected products purchase request
     * registers.
     */
    this.removeSelectedProducts = () => {
        const purchaseRequestList = getWidget('purchaseRequestList');

        ScreenService.confirmMessage('WANT_REMOVE_SELECTED_ITEMS').then(() => {
            service.removeSelectedProductsIfAreValid(purchaseRequestList.getCheckedRows()).then(() => {

                ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
                purchaseRequestList.reload();
                purchaseRequestList.dataSource.clearCheckedRows();
                changeFabAction();

            }).catch(errorCode => {
                ScreenService.showMessage(errorCode);
            });
        });
    };

    /**
     * Remove  swiped product register if is valid to.
     */
    this.removeSwipedProductPurchaseRequest = () => {
        let itemToExclude = templateManager.container.itemToExclude;
        removeProductPurchaseRequest(itemToExclude).then(proceed => {
            if (proceed) {
                templateManager.updateTemplate();
            }
        });
    };

    /**
     * Remove current product register if is valid to.
     */
    this.removeClickedProductPurchaseRequest = () => {
        ScreenService.confirmMessage('WANT_REMOVE_SELECTED_ITEMS').then(() => {
            removeProductPurchaseRequest(templateManager.container.currentItem).then(proceed => {
                if (proceed) {
                    UIManager.goBack();
                }
            });
        });
    };

    /**
     * Open widget to edit existent product purchase
     * request register.
     */
    this.openRequestEditFormulary = () => {
        let purchaseRequestList = getWidget('purchaseRequestList');
        let requestForm = getWidget('editPurchaseRequestForm');
        let currentProd = templateManager.container.currentItem;

        requestForm.dataSource = purchaseRequestList.dataSource;

        UIManager.goToWidget(requestForm).then(() => {
            let index = requestForm.dataSource.data.indexOf(currentProd);
            requestForm.activateTab(index);
            if (index == 0) {
                this.onChangeActiveTab();
            }
        });
    };

    /**
     * Initialize request edit form. Set Widget label,
     * currentRow and fields acessibility. Disable 
     * fields if product is invalid to edit.
     * 
     * @param {Widget} requestForm
     * @param {Object} currentProd
     * 
     * @returns {Promise}
     */
    const setupEditRequestForm = (requestForm, currentProd) => {

        return service.isValidProductToEdit(currentProd).then((isValid) => {
            let extraProdField = requestForm.getField('PRODUTOEXTRA');

            if (!currentProd._edited) {
                currentProd.PRODUTOEXTRA = !!currentProd.DSJUSTSOLEXT;
            }
            extraProdField.isDisabled = !isValid;

            let justExtraSolField = requestForm.getField('DSJUSTSOLEXT');
            justExtraSolField.isVisible = !!currentProd.DSJUSTSOLEXT;
            justExtraSolField.isDisabled = !isValid;

            requestForm.getField('QTAJUSTADA').isDisabled = !isValid;
            requestForm.getField('DSJUSTOPER').isDisabled = !isValid;

            requestForm.getAction('saveEditedProductPurchaseRequest').isDisabled = !isValid;

            templateManager.updateTemplate().then(() => {
                if (justExtraSolField.isVisible) {
                    justExtraSolField.updateDescriptionText();
                }
            });
        });
    };

    /**
     * save product purchase request edition with user's data
     * input if all data is valid to proceed.
     */
    this.saveEditedProductPurchaseRequest = () => {
        let editPurchaseRequestForm = getWidget('editPurchaseRequestForm');

        service.saveEditedProductPurchaseRequest(editPurchaseRequestForm.getActiveTabData()).then(() => {
            editPurchaseRequestForm.getActiveTabData()._edited = false;
            ScreenService.notificationMessage('PRODUCT_EDITED');
        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };

    /**
     * On change active tab, change what fields
     * are active.
     */
    this.onChangeActiveTab = () => {
        let requestForm = getWidget('editPurchaseRequestForm');
        let currentItem = requestForm.getActiveTabData();

        setupEditRequestForm(requestForm, currentItem);
    };

    /**
     * Toggle '_edited' property into tab register, used
     * for internal control of registers setup
     * (Like in 'setupEditRequestForm'
     * function).
     *
     * @param {Boolean} state - New State
     */
    this.toggleEditedStateOnCurrentRegister = (state) => {
        let editPurchaseRequestForm = getWidget('editPurchaseRequestForm');
        editPurchaseRequestForm.getActiveTabData()._edited = state;
    };

    /**
     * Open filter popup to allow user filter request list
     * using a date period.
     */
    this.openFilterPopUp = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.isVisible = true;
        UIManager.openPopUp(dateFilterPopup);
    };

    /**
     * Open filter popup to allow user filter request list
     * using a date period.
     */
    this.closeFilterPopUp = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.isVisible = false;
        UIManager.goBack();
    };

    /**
     * Confirm filter request using selected date
     * by user in form.
     */
    this.filterRequestListByDate = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        const purchaseRequestList = getWidget('purchaseRequestList');
        let selectedDate = dateFilterPopup.currentRow.FILTER_DATE;

        if (!!selectedDate) {
            purchaseRequestList.dataSource.data = purchaseRequestList.dataSource._data.filter(reg => reg.DTPROGENTR == selectedDate);
        } else {
            purchaseRequestList.dataSource.data = [...purchaseRequestList.dataSource._data];
        }
        ScreenService.notificationMessage('REQUEST_FILTERED');
        this.closeFilterPopUp();
    };

    /**
     * After load dataSource, set a backup property to use
     * in filter functions.
     */
    this.afterLoadDataSourcePurReqList = () => {
        const purchaseRequestList = getWidget('purchaseRequestList');
        purchaseRequestList.dataSource._data = [...purchaseRequestList.dataSource.data];
    };

    /**
     * Clear selected date into date filter popup.
     */
    this.clearDateFilter = () => {
        const dateFilterPopup = getWidget('dateFilterPurchaseRequest');
        dateFilterPopup.currentRow.FILTER_DATE = null;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('PurchaseRequestController', PurchaseRequestController);
});


// FILE: js/controllers/StaffManagementController.js
var StaffManagementController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager, StaffManagementService) {

    /**
     * @var service Reference to StaffManagementService.
     */
    const service = StaffManagementService;

    /**
     * Reference to Container's getWidget function. Return 
     * widget object by name.
     * 
     * @param {String} widgetName Name of widget to return.
     *
     * @returns {Object<Widget>} widget
     */
    const getWidget = widgetName => templateManager.container.getWidget(widgetName);

    /**
     * Check if is valid to init screen.
     * Test if have imported data, a logged 
     * unit, and check if is valid to create
     * new inventory registers.
     */
    this.initScreenIfIsValid = () => {

        if (!SchoolMealsUtil.hasImportedData()) {
            openScreenWithDashboardPushState('dataImport').then(ScreenService.showMessage('HAVE_TO_IMPORT_DATA'));
        } else if (!SchoolMealsUtil.hasLoggedUnit()) {
            openScreenWithDashboardPushState('changeUnit').then(ScreenService.showMessage('PLEASE_CHANGE_UNIT'));
        } else if (service.isLoggedUnitMealsCenter()) {
            ScreenService.showMessage('CANNOT_MANAGE_STAFF_IS_MEALS_CENTER');
        } else if (!service.loggedUnitHasMealsCenter()) {
            ScreenService.showMessage('CANNOT_MANAGE_STAFF_HAS_NO_MEALS_CENTER');
        } else {
            service.checkIfHasStaffInPeriod()
                .then(service.checkIfHasStaffInUnit)
                .then(() => openScreenWithDashboardPushState('staffManagement'))
                .catch(errorCode => ScreenService.showMessage(errorCode));
        }

    };

    /**
     * Open screen using ScreenService.openWindow and
     * push state using UIManager.pushState to correctly
     * go back to dashboard when back action is pressed.
     * 
     * @param {String} screenName Name of container to open.
     * 
     * @returns {Promise}
     */
    const openScreenWithDashboardPushState = screenName => {
        return ScreenService.openWindow(screenName).then(() => {
            UIManager.pushState(() => {
                ScreenService.openWindow('dashboard');
            });
        });
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service name using 
     * alphabetical order.
     * Set fab icon according to ordering direction.
     */
    this.sortServiceListByServiceName = () => {
        const staffManagement = getWidget('staffManagement');
        service.sortServiceListByServiceName(staffManagement.dataSource);
        staffManagement.container.fabActionIcon = !staffManagement.dataSource.orderingAscAlpha ? "fa-sort-alpha-up" : "fa-sort-alpha-down";
    };

    /**
     * Sort service list from staffManagement main
     * dataSource by service date.
     * Set fab icon according to ordering direction.
     */
    this.sortServiceListByDate = () => {
        const staffManagement = getWidget('staffManagement');

        service.sortServiceListByDate(staffManagement.dataSource);
        staffManagement.container.fabActionIcon = !staffManagement.dataSource.orderingAscDate ? "fa-sort-numeric-up" : "fa-sort-numeric-down";
    };

    /**
     * Open screen to edit staff registers.
     * Hide fab, get dataSource from staffListWidget
     * set default staff quantity label as '0', then
     * activate tab that contains clicked data.
     * 
     * @param {Object} clickedData 
     */
    this.openStaffEditScreen = clickedData => {
        const staffList = getWidget('staffManagement');
        const staffEditWidget = getWidget('staffEdit');

        templateManager.container.fabVisibility = false;
        staffEditWidget.dataSource = staffList.dataSource;
        staffEditWidget.getField('staffLabel').subLabel = '0';

        UIManager.goToWidget(staffEditWidget).then(() => {
            let index = staffList.dataSource.data.indexOf(clickedData);
            staffEditWidget.activateTab(index);
        });
    };

    /**
     * Instructions to be executed when change staff
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeStaffQuantity = () => {
        const staffEditWidget = getWidget('staffEdit');

        setSaveButtonAcessibility(true);

        if (!staffEditWidget.getActiveTabData().changed) {
            setStaffChangedData(staffEditWidget);
        }
        setStaffSum(staffEditWidget);
    };

    /**
     * Enable or disable save button.
     * 
     * @param {Boolean} enable
     */
    const setSaveButtonAcessibility = enable => {
        const staffEditWidget = getWidget('staffEdit');
        staffEditWidget.getAction('saveStaffEdit').isDisabled = !enable;
        templateManager.updateTemplate();
    };

    /**
     * Set changed as true in active tab data.
     * Add '●' character on changed data's label.
     * 
     * @param {Widget} staffEditWidget Staff Edit widget
     */
    const setStaffChangedData = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();

        activeData.changed = true;
        activeData.tabLabel = activeData.tabLabel.replace("", "\u25CF ");
    };

    /**
     * Set staff sum total in widget current tab;
     * 
     * @param {Widget} staffEditWidget Staff Edit widget
     */
    const setStaffSum = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();
        let sum = Number(activeData.QTCLIEFECPME) + Number(activeData.QTFUNEFECPME) + Number(activeData.QTDOAEFECPME);

        staffEditWidget.getField('staffLabel').subLabel = sum;
        activeData.QTCOMEFECPME = sum;
    };

    /**
     * Get back from staff edit widget.
     * Reload dataSource to refresh all unsaved
     * data.
     */
    this.getBackFromStaffEdit = () => {
        const staffList = getWidget('staffManagement');
        staffList.dataSource.reload();
        setSaveButtonAcessibility(false);
        UIManager.goBack();
    };

    /**
     * Save edited active staff register.
     * show success message after conclude operation.
     * Show an error message if something goes wrong.
     */
    this.saveStaffEdit = () => {
        const staffEditWidget = getWidget('staffEdit');
        let activeData = staffEditWidget.getActiveTabData();

        service.saveStaffEdit(activeData)
            .then(() => afterSaveStaffEdit(staffEditWidget))
            .then(() => ScreenService.notificationMessage('SETUP_CLIENT_QUANT'))
            .catch(errorCode => {
                ScreenService.showMessage(errorCode);
            });
    };

    /**
     * After save staff edited data, show 
     * success message and reset 'changed' property,
     * save button acessibility and unsaved data 
     * mark ('●');
     * 
     * @param {Widget} staffEditWidget
     */
    const afterSaveStaffEdit = staffEditWidget => {
        let activeData = staffEditWidget.getActiveTabData();

        ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
        activeData.changed = false;
        activeData.tabLabel = activeData.tabLabel.replace("\u25CF ", "");
        setSaveButtonAcessibility(false);
    };

    /**
     * Instructions to be executed when 
     * change active tab.
     * Reset save button acessibility
     * 
     * @param {Widget} staffEditWidget
     */
    this.onChangeActiveTab = staffEditWidget => {
        castStaffQuantitiesToNumber(staffEditWidget.getActiveTabData());
        setSaveButtonAcessibility(staffEditWidget.getActiveTabData().changed);
        setStaffSum(staffEditWidget);
    };

    /**
     * Cast quantities values to Number
     * 
     * @param {Object} staffData 
     */
    const castStaffQuantitiesToNumber = staffData => {
        staffData.QTFUNEFECPME = Number(staffData.QTFUNEFECPME);
        staffData.QTCLIEFECPME = Number(staffData.QTCLIEFECPME);
        staffData.QTCOMEFECPME = Number(staffData.QTCOMEFECPME);
        staffData.QTDOAEFECPME = Number(staffData.QTDOAEFECPME);
    };

    /**
     * Set filter on staffClientList widget dataSource
     * to bring related data. Then open widget.
     */
    this.openStaffClientList = () => {
        const staffEditWidget = getWidget('staffEdit');
        const staffClientList = getWidget('staffClientList');
        const staffList = getWidget('staffManagement');

        let staffData = staffEditWidget.getActiveTabData();

        filterStaffClientList(staffClientList, staffData);
        UIManager.goToWidget(staffClientList);
        staffEditWidget.currentActiveTabIndex = staffList.dataSource.data.indexOf(staffEditWidget.getActiveTabData());
    };

    /**
     * Build staff client list dataSourceFilter according
     * to active tab staff data.
     * 
     * @param {Widget} staffClientList 
     * @param {Object} staffData
     */
    const filterStaffClientList = (staffClientList, staffData) => {
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDFILIAL', staffData.CDFILIAL);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'DTEFETCLPME', staffData.DTCARDIAPME);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDSERVFILI', staffData.CDSERVFILI);
        SchoolMealsUtil.setValueOnFilter(staffClientList, 'CDTIPOCARD', staffData.CDTIPOCARD);
        staffClientList.reload();
    };

    /**
     * Open staff edit form in popup widget.
     * Set current row as clicked item and
     * backup row as a copy of it. If user cancel operation
     * then backup is restored, else 
     */
    this.openStaffEditPopUp = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        staffClientEdit.isVisible = true;
        staffClientEdit.currentRow = Object.assign({}, templateManager.container.currentItem);
        staffClientEdit.__backupRow = Object.assign({}, templateManager.container.currentItem);

        staffClientEdit.getField('staffLabel').subLabel = staffClientEdit.currentRow.TOTAL;

        UIManager.openPopUp(staffClientEdit);
    };

    /**
     * Save edited quantities of staff's client.
     * Set current item to edited current row.
     */
    this.saveStaffClientEdition = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        Object.assign(templateManager.container.currentItem, staffClientEdit.currentRow);
        UIManager.goBack();
        ScreenService.notificationMessage('QUANTITY_EDITED_SUCCESFUL');
    };

    /**
     * Cancel edited quantities of staff's client.
     * Reset current item to previous object.
     */
    this.cancelStaffClientEdition = () => {
        const staffClientEdit = getWidget('staffClientEdit');

        staffClientEdit.currentRow = Object.assign({}, staffClientEdit.__backupRow);
        UIManager.goBack();
    };

    /**
     * Instructions to be executed when change staff client
     * quantity. 
     * Set changed on current row and enable save button.
     */
    this.onChangeStaffClientQuantity = () => {
        const staffClientEdit = getWidget('staffClientEdit');
        castStaffClientQuantitiesToNumber(staffClientEdit.currentRow);
        setStaffClientSum(staffClientEdit);
    };

    /**
     * Cast client quantities values to Number
     * 
     * @param {Object} staffClientData
     */
    const castStaffClientQuantitiesToNumber = staffClientData => {
        staffClientData.QTCOMEFCLPME = Number(staffClientData.QTCOMEFCLPME);
        staffClientData.QTDOAEFCLPME = Number(staffClientData.QTDOAEFCLPME);
        staffClientData.QTFUNEFCLPME = Number(staffClientData.QTFUNEFCLPME);
    };

    /**
     * Set staff client sum total in popup widget;
     * 
     * @param {Widget} staffClientWidget Staff client Edit widget
     */
    const setStaffClientSum = staffClientWidget => {
        let currentRow = staffClientWidget.currentRow;
        let sum = Number(currentRow.QTDOAEFCLPME) + Number(currentRow.QTFUNEFCLPME) + Number(currentRow.QTCOMEFCLPME);

        staffClientWidget.getField('staffLabel').subLabel = sum;
        currentRow.TOTAL = sum;
    };

    /**
     * Save all staff client quantities on 
     * 'ClientesDoServicoEfetivos' dataSource if
     * is valid.
     * Client data is valid if sum of all client's
     * quantities is equal to service total staff
     * quantity.
     * 
     */
    this.saveStaffClientQuantities = () => {
        const staffClientList = getWidget('staffClientList');
        const staffEditWidget = getWidget('staffEdit');

        let staffClientListData = staffClientList.dataSource.data;
        let staffRegister = staffEditWidget.dataSource.data[staffEditWidget.currentActiveTabIndex];

        service.saveStaffClientQuantities(staffClientListData, staffRegister).then(() => {
            UIManager.goBack();

            ScreenService.notificationMessage('SUCCESSFUL_OPERATION');
            let activeDataIndex = staffEditWidget.dataSource.data.indexOf(staffEditWidget.getActiveTabData());
            staffEditWidget.activateTab(activeDataIndex);
            staffEditWidget.updateHeaderScroll();
        }).catch(errorCode => {
            ScreenService.showMessage(errorCode);
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('StaffManagementController', StaffManagementController);
});


// FILE: js/controllers/SupplynigAuthorizationController.js
var SupplynigAuthorizationController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Instructions to be executed when
     * enter widget.
     */
    this.onEnterSupplyingAuth = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.openWindow('dashboard').then(() => {
                ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
            });
        } else {
            setDefaultDataSourceFilter();
        }
    };

    /**
     * Sort widget data by code (NRAUTOFORN).
     */
    this.sortDataByCode = () => {
        const supplyingWidget = templateManager.container.getWidget('supplyingAuthorization');

        supplyingWidget.dataSource.orderingAscNumeric = !supplyingWidget.dataSource.orderingAscNumeric;
        supplyingWidget.container.fabActionIcon = !supplyingWidget.dataSource.orderingAscNumeric ? "fa-sort-numeric-up" : "fa-sort-numeric-down";

        supplyingWidget.dataSource.data.sort((a, b) => {
            let codeA = Number(a.NRAUTOFORN);
            let codeB = Number(b.NRAUTOFORN);
            let order = 0;

            if (supplyingWidget.dataSource.orderingAscNumeric) {
                order = codeA - codeB;
            } else {
                order = codeB - codeA;
            }

            return order;
        });
    };

    /**
     * Sort widget data by product name (NMPRODUTO).
     */
    this.sortDataByProductName = () => {
        const supplyingWidget = templateManager.container.getWidget('supplyingAuthorization');

        supplyingWidget.dataSource.orderingAscAlpha = !supplyingWidget.dataSource.orderingAscAlpha;
        supplyingWidget.container.fabActionIcon = !supplyingWidget.dataSource.orderingAscAlpha ? "fa-sort-alpha-up" : "fa-sort-alpha-down";

        supplyingWidget.dataSource.data.sort((a, b) => {
            let nameA = a.NMPRODUTO.toLowerCase();
            let nameB = b.NMPRODUTO.toLowerCase();
            let order = 0;

            order = (nameA < nameB) ? -1 : 1;

            if (!supplyingWidget.dataSource.orderingAscAlpha) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Show suppyling authorization data details.
     * 
     * @param {Object} data Clicked data.
     */
    this.showSupplyingAuthDetails = data => {
        const analyticsView = templateManager.container.getWidget('analyticsView');

        analyticsView.currentRow = Object.assign(data);
        analyticsView.getField('supplyingAuthCode').text = ScreenService.i18n('SUP_AUTH_LABEL') + ': ' + data.NRAUTOFORN;

        UIManager.goToWidget(analyticsView);
    };

    /**
     * Set default dataSourceFilter on main
     * screen. Filter data according to current
     * unit.
     */
    const setDefaultDataSourceFilter = () => {
        let supplyingAuthList = templateManager.container.getWidget('supplyingAuthorization');
        supplyingAuthList.dataSourceFilter = [{
            name: "CDFILDEST",
            operator: "=",
            value: getLocalVar('userData').CDFILIAL
        }];
        supplyingAuthList.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('SupplynigAuthorizationController', SupplynigAuthorizationController);
});

