// FILE: schoolmeals_components/dashboards/js/HorizontalListsDashBoardController.js
"use strict";

function HorizontalListsDashBoardController($scope, ApplicationContext, $element, $timeout) {
    const UIManager = ApplicationContext.UIManager;

    $scope.getWelcomeMessage = (widget) => {
        let welcomeMessage = widget.welcomeMessage;
        
        if(widget.currentRow){
            Object.keys(widget.currentRow).forEach(columnName=>{
                welcomeMessage = welcomeMessage.replace('__'+columnName+'__', widget.currentRow[columnName]);
            });
        }

        return welcomeMessage;
    };

    $scope.getUserName = () => {
        return getLocalVar('userData').NMOPERADOR + ',';
    };

    function getSourceLength(source) {
        return source ? source.length : 0;
    }

    $scope.executeControllerAction = (code) => {
        console.error('depreciated, '+code+ 'will not be executed');
    };

    $scope.formatData = (data) => {
        moment.locale('pt-br');
        return moment(data, 'DD/MM/YYYY hh:mi:ss').format('ddd, DD [de] MMM [às] HH:mm');
    };

    $scope.openWindow = (hor) => {
        UIManager.openContainer('listedProposalData', hor);
    };

    $scope.getTotal = (dataSource) => {
        return dataSource.length;
    };

    let render = function(filteredData, widget){
        widget.dataSource.data = filteredData;
        $scope.$apply();
    };
    
    let searchAdapter;
    $scope.initToList = (widget) => {
        
        searchAdapter = new SearchAdapter($element[0].querySelector('.search-input'), widget, 
        (filteredData) => render(filteredData, widget), 
        {
            local: true,
            dataSourceOrigin: widget.dataSource,
            dataSourcePropName: "__copyData",
            searchFields: widget.searchFields
        });
        
        widget.showSearch = () => {
            $scope.isSearching = true;
            widget.container.fabVisibility = false;
            widget.dataSource.__copyData = angular.copy(widget.dataSource.data);

            UIManager.pushState(() => {
                $scope.isSearching = false;
                widget.dataSource.data = angular.copy(widget.dataSource.__copyData);

                let inputElement = $element[0].querySelector('.search-input');
                inputElement.value = "";
                inputElement.blur();

                widget.container.fabVisibility = true;
                if('cordova' in window){
                    $scope.$apply();
                }
            });

            let input = $element[0].querySelector('.search-input');
            setTimeout(() => input.focus(), 10);
        }
    };

    $scope.goBack = () => UIManager.goBack();


    $scope.clickCard = cardData => {
        if(cardData.onClick){
            $scope.$eval(cardData.onClick, Object.assign({
                data: cardData
            }, ApplicationContext));
        } else if(cardData.windowName){
            UIManager.openContainer(cardData.windowName);
        }
    };

    $scope.onClickData = cardData => {
        if($scope.widget.onClickData){
            $scope.$eval($scope.widget.onClickData, Object.assign({
                data: cardData
            }, ApplicationContext));
        }
    };

    /**
     * Set horizontal dashboard events and properties.
     */
    (()=>{
        const horizontalDsbComponent = document.querySelector('.horizontal-lists-dash-board-component');

        if(horizontalDsbComponent){
            /**
             * Cancel previous timeout started when user touchended
             * screen.
             */
            horizontalDsbComponent.addEventListener("touchstart", ()=>{
                $timeout.cancel($scope.arrowTimeOutID);
            });

            /**
             * Check if user scroll to the end of the page. If in the end, 
             * so stop shake arrow, and rotate it horizontally. Else, apply normal
             * state to arrow;
             */
            horizontalDsbComponent.addEventListener("touchend", ()=>{
                $scope.arrowTimeOutID = $timeout(()=>{
                    $scope.endScroll = (horizontalDsbComponent.offsetHeight + horizontalDsbComponent.scrollTop) >= (horizontalDsbComponent.scrollHeight - 30);
                }, 500);
            });

            /**
             * Init shake arrow properties.
             */
            $scope.initArrowProperties = ()=>{
                $scope.endScroll = false;
            };  



            /**
             * Init main horizontal dashboard list.
             * 
             * @param {Object} horizontalDashboard
             */
            $scope.initMainHorizontalDashboard = horizontalDashboard =>{
                horizontalDashboard.dataSource[0].isActive = true;

                setHorizontalDashboardCardScrollEvents(horizontalDashboard);
            };

            /**
             * Cancel previous timeout started when user touchended
             * screen.
             * 
             * Check if user scroll between horizontal main cards, to 
             * change active status.
             * 
             * @param {Object} horizontalDashboard horizontal main dashboard
             * object.
             */
            const setHorizontalDashboardCardScrollEvents = horizontalDashboard => {
                const horizontalDsbMainCardScroll = document.querySelector('.horizontal-main-dashboard .itens-container');

                horizontalDsbMainCardScroll.addEventListener("touchstart", ()=>{
                    $timeout.cancel($scope.scrollMarkerTimeOutID);
                });
                
                horizontalDsbMainCardScroll.addEventListener("touchend", ()=>{
                    $scope.scrollMarkerTimeOutID = $timeout(()=>{
                        let currentCardIndex = horizontalDsbMainCardScroll.scrollLeft / horizontalDsbMainCardScroll.offsetWidth;
                        
                        horizontalDashboard.dataSource.forEach((card, index)=>{
                            card.isActive = Math.round(currentCardIndex) == index;
                        });
                    }, 600);
                });
            };

            $scope.toggleSecondaryDashboardScroll = () => {
                if($scope.endScroll){
                    horizontalDsbComponent.scrollTo(0,0);
                } else {
                    document.querySelector('.horizontal-secondary-dashboard').scrollIntoViewIfNeeded()
                }
            };
        }

    })();
}

// FILE: schoolmeals_components/dashboards/js/SearchAdapter.js
/**
 * 
 * @param {HTMLInputElement} input 
 * @param {any} restLoadingZhObject a object that contains a dataSource and dataSourceFilter
 */
function SearchAdapter(inputElement, restLoadingZhObject, render = () => {}, options = {}) {
    
    let activeRequests = [];
    let cacheBase = {};


    //TODO
    let cancelRequest = () => {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
    };

    this.options = options;

    let restKeyStrokeHandler = (event) => {
        if (activeRequests.length) {
            if (activeRequests[0].reqText === inputElement.value) {
                return;
            }
            else {
                cancelRequest();
            }
        }
        if (cacheBase[inputElement.value]) {
            render(cacheBase[inputElement.value]);
            return;
        }
        update();
    
    }

    let localKeyStrokeHandler = (event) => {
        let searchValue = inputElement.value || "";
        searchValue = searchValue.toLowerCase();
        let searchArray = angular.copy(this.options.dataSourceOrigin[this.options.dataSourcePropName]);
        const searchFields = this.options.searchFields;
        let filteredArray = searchArray.filter(dataSourceElement => {
            for(var i = searchFields.length; i--;){
                let currentString = dataSourceElement[searchFields[i]];
                if (currentString && currentString.toLowerCase().indexOf(searchValue) !== -1){
                    return true; 
                }
            }
        });
        
        render(filteredArray);
    }

    let keyStrokeStrategies = {
        "local": localKeyStrokeHandler,
        "rest": restKeyStrokeHandler
    }
    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    let keyStrokeHandler = (event) => {
        if (event.keyCode > 45 || event.keyCode === 8) {
            keyStrokeStrategies[this.options.keyStrokeStrategyName](event);
        }
    };

    // TODO
    /**
     * @returns {Promise<null>} 
     */
    let update = () => {
        return fetchData().then(refreshLastUpdateTime);
    };

    // TODO
    async function fetchData(filter) {
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        ScreenService.hideLoader();
        var newFilter = restLoadingZhObject.dataSourceFilter.slice(0); // make a copy of the zhDataSourceFilter to keep it pristine
        newFilter.push({
            name: options.searchName,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        }); // append our custom conditions into the new array
        var promisseToReturn = await restLoadingZhObject.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet) {
                cacheBase[deferredToCancelRequest.reqText] = resultSet.dataset[Object.keys(resultSet.dataset)[0]]; // do cache
            } else {
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        // ScreenService.hideLoader();
        return promisseToReturn;
    }


    this.bindListeners = () => {
        inputElement.addEventListener('keyup', keyStrokeHandler);
    };
    
    this.removeListeners = () => {
        inputElement.removeEventListener('keyup', keyStrokeHandler);
    }

    this.init = () => {
        this.bindListeners();
        if(this.options.local){ // if the local option is enabled the keyStrokeStrategyName will be overriden 
            this.options.keyStrokeStrategyName = 'local';
        } else {
            this.options.keyStrokeStrategyName = this.options.keyStrokeStrategyName || "rest";
        }
        
    };

    this.init();
}

// FILE: schoolmeals_components/fields/js/CheckboxFieldsController.js
'use strict';

function CheckboxFieldsController($scope) {

    function isNullorUndefined(itemToTest) {
        if (itemToTest === undefined || itemToTest === null) {
            return true;
        }
        return false;
    }

    $scope.getDisplayField = (field, data) => {
        let valueShow = field.valueShow;
        if (Array.isArray(valueShow)) {
            let ret = '';
            valueShow.forEach(v => {
                if(data[v]){
                    ret += data[v];
                    ret += ' ';
                }
            });
            return ret;
        } else {
            return data[field.valueShow];
        }
    };

    $scope.addOrRemoveOption = (field, data, $e) => {
        let curRow = $scope.widget.currentRow;
        let elem = $($e.currentTarget);
        let inpt = elem.children('input.chbox');
        inpt.attr('checked', true);
        let isChecked = inpt.prop("checked");

        if(isChecked){
            field.dataSource.addCheckedRows(data);
        } else {
            field.dataSource.removeCheckedRows(data);
        }
        curRow[field.name] = field.dataSource.checkedRows;
    };

    let _isInFieldDataArray = (elementToFind, arrayToSearch, strict = false) => {
        if(!strict){
            let found = false;
            let searchKeys = $scope.field.dataSource.getVO().localKey || [$scope.field.name];
            found = arrayToSearch.some((arrayElement) => 
                searchKeys.every(key => arrayElement[key] === elementToFind[key])
            );
            return found;
        } else {
            return (arrayToSearch.indexOf(elementToFind) !== -1);
        }
    };

    /**
     * @param {Array} rows array of objects or pks only
     * @param {boolean} strict 
     */
    $scope.field.setCheckedRows = (rows, strict = false) => {
        if(rows.length && typeof rows[0] !== 'object'){ // if a array of primitives is received 
            rows = rows.map(row => {
                let newRow = {};
                newRow[$scope.field.name] = row;
                return newRow
            })
        }
        $scope.field.dataSource.checkedRows = [];
        $scope.field.dataSource.addCheckedRows(rows);
        $scope.widget.currentRow[$scope.field.name] = $scope.field.dataSource.checkedRows;
        let checkboxElements = $scope.field.element[0].querySelectorAll('input.chbox[type=checkbox]');
        checkboxElements.forEach((checkboxElement, index) => {
            let checkboxData = $scope.field.dataSource.data[index];                
            checkboxElement.checked = _isInFieldDataArray(checkboxData, rows, strict);
        });
    };
    
}

// FILE: schoolmeals_components/fields/js/SelectAutocompleteController.js
'use strict';
function SelectAutocompleteController(
    $scope,
    $element,
    ScreenService,
    ZhUtil,
    maskEngine,
    restEngine,
    $q,
    ZhLogger,
    ApplicationContext
    ){

    /**
     * @type {Object}
     */
    var events = {};
    // document.querySelector('.zh-notification-messages').style.display = 'none';
    var activeRequests = [];
    /**
     * @type {Object}
     * a object that will precache the results of each query
     */
    var cacheBase = {};
    /**
     * the spinner element that indicates progress   
     * @type {HTMLElement}
     */
    var spinner = null;
    /**
     * a reference to the current highlited element of the list   
     * @type {HTMLElement}
     */
    var currentHighletedListElement = null;
    /**
     * @type {Object[]} 
     */
    var filteredAditionalOptions = $scope.field.aditionalOptions || [];
    /**
     * @type {Object[]} 
     */
    var filteredListContents = [];
    /** it will be initialized on the initializeData method
     * @type {listContents}
     */
    var listContents = null;
    /**
     * @type {HTMLElement}
     */
    var rootElement;
    /**
     * @type {HTMLElement}
     */
    var popUp;
    /**
     * @type  {HTMLElement}
     */
    var listElement;
    /**
     * @type {HTMLElement}
     */
    var inputElement;
    /**
     * @type {HTMLElement}
     */
    var zhLoader;
    /**
     * @type {HTMLElement}
     */
    var autoCompletePreviewElement;
    /**
     * @type {HTMLElement}
     */
    var labelElement;
    // /**
    //  * @type {HTMLElement}
    //  */
    // var headerLabelElement;
    /**
     * @type {HTMLElement}
     */
    var clearButtonElement;
    /**
     * @type {Object[]}
     */
    var fieldInfo = [];

    /**
     * an element that will be used as a basis to render each list element 
     * @type {HTMLElement}
     */
    var baseListRowToClone;

    var filterConditions = {
        name: "",
        value: "%%",
        operator: "LIKE_ALL"
    };

    var shouldPreventKeyUp = false;
    var lastUpdateTime;
    var updateRequestReference = null;
    var deferredToCancelRequest = false;

    var cacheFirstResultSet = function() {
        cacheBase[""] = listContents;
    };

    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    var keyStrokeHandler = function(event) {
        if (event.keyCode > 45 || event.keyCode === 8) {
            //console.log('%ckeyCode: ' + event.keyCode + "   key: '" + event.key + "'", 'color: orange');
            // var begining = performance.now(); // LOG TIME
            if (activeRequests.length) {
                if (activeRequests[0].reqText === inputElement.value) {
                    return;
                }
                else {
                    cancelRequest();
                }
            }
            if (cacheBase[inputElement.value]) {
                filteredListContents = cacheBase[inputElement.value];
                renderListElements(cacheBase[inputElement.value]);
                // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
                return;
            }
            filteredAditionalOptions = filterElements(inputElement.value, $scope.field.aditionalOptions || [], true);
            var filtered = filterElements(inputElement.value, null, true);
            renderListElements(filtered);
            // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
            // showAutoCompleteText();
            // var timeSinceUpdate = performance.now() - lastUpdateTime;
            // if (timeSinceUpdate >= 200) {
            //     update();
            // }
            // else {
            //     updateRequestReference = setTimeout(update, 200);
            // }
            update();
        }
        shouldPreventKeyUp = true;
    };

    /**
     * handles the command keys keypress, or propagates the event chain for the keyStrokeHandler to resolve
     * @param {KeyboardEvent} [event] 
     */
    var commandKeyStrokeHandler = function(event) {
        var code = event.keyCode;
        if (code > 45 || code === 8) {
            // do nothing and let the keyStrokeHandler handle the keyup event 
            shouldPreventKeyUp = false;
            return;
        }
        else if (code === 40 /* down arrow */ ) {
            if (currentHighletedListElement.parentElement.lastChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.nextElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 38 /* up arrow */ ) {
            if (currentHighletedListElement.parentElement.firstChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.previousElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 13) /* enter */ {
            confirm(null, currentHighletedListElement);
        }
        shouldPreventKeyUp = true;
    };

    /**
     * shows the first element of the filteredListContents array
     */
    var showAutoCompleteText = function() {
        autoCompletePreviewElement.value = getFormatedString(filteredListContents[0], 0);
    };

    var refreshLastUpdateTime = function() {
        lastUpdateTime = performance.now();
    };
    var hideSpinner = function() {
        spinner.classList.add('hidden');
    };
    var onRequestComplete = function() {
        hideSpinner();
        refreshLastUpdateTime();
    };

    /**
     * @returns {Promise<null>} 
     */
    var update = function() {
        if (updateRequestReference !== null) {
            clearTimeout(updateRequestReference);
            updateRequestReference = null;
        }
        filterConditions.value = "%" + inputElement.value + "%";
        return fetchData().then(refreshLastUpdateTime);
    };

    var cancelRequest = function() {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
    };

    var initiliazefieldInfo = function() {
        fieldInfo = [];
        filterConditions.name = "";
        if ($scope.field.fields) {
            for (var i = 0; i < $scope.field.fields.length; i++) { // TODO: use reverse for instead
                var label = ($scope.field.fields[i].label ? ($scope.field.fields[i].label + ": ") : "");
                fieldInfo.push({
                    "label": label,
                    "name": $scope.field.fields[i].name,
                    "mask": $scope.field.fields[i].mask || false
                });
                filterConditions.name += $scope.field.fields[i].name + '|';
            }
            filterConditions.name = filterConditions.name.substr(0, filterConditions.name.length - 1);
        }
        else {
            fieldInfo.push({
                "label": "" /*labelWithNoExtraFields*/ ,
                "name": $scope.field.name,
                "mask": $scope.field.mask || false
            });
            filterConditions.name = $scope.field.name;
        }
        $scope.field.dataSource.itemsPerPage = 100;
        $scope.field.dataSource.disableLoader = true;

        if($scope.field.events){
            let fieldOnChangeEvent = $scope.field.events.find(event => event.name === 'FieldOnChange');
            if(fieldOnChangeEvent){
                events.FieldOnChange = fieldOnChangeEvent.code;
            }
        }
    };
    $scope.field.initiliazefieldInfo = initiliazefieldInfo;
    var initiliazeDataSetter = function() {
        if ($scope.field.dataSource) {
            Object.defineProperty($scope.field.dataSource, 'data', {
                get: function() {
                    return listContents;
                },
                set: function(newValue) {
                    if (newValue === listContents) {
                        return;
                    }
                    newValue = [...filteredAditionalOptions].concat([...newValue]);
                    filteredListContents = listContents = newValue;
                    renderListElements(listContents);
                    if(!$scope.field.descriptionTextRendered){
                        let selectedElement = $scope.field.dataSource.data.find(dataSourceElement => {
                            return dataSourceElement[$scope.field.name] === $scope.__fieldRow;
                        });
                        if($scope.__fieldRow && selectedElement){
                            $scope.descriptionText = $scope.getDescriptionString(selectedElement);
                        }
                    }
                },
                enumerable: true,
                configurable: true
            });
        }

        

    };

    var doCache = function(cacheKey, cacheList) {
        cacheBase[cacheKey] = cacheList;
    };

    var initiliazeData = function() {
        if ($scope.field.dataSource.data) {
            filteredListContents = listContents = $scope.field.dataSource.data;
            let arrayToRender = listContents;
            if(!filteredAditionalOptions.every(aditionalOption => listContents.includes(aditionalOption))){
                listContents = filteredAditionalOptions.concat(listContents);
            }
            renderListElements(arrayToRender);
        }
        else {
            listContents = [];
        }
        if (!$scope.field.dataSource.dataSourceFilter) {
            $scope.field.dataSource.dataSourceFilter = [];
        }
    };

    let closeKeyboard = () => inputElement.blur();

    $scope.autoCompleteConstructor = function() {
        rootElement = $element.get(0);
        listElement = rootElement.querySelector('#element-list');
        inputElement = rootElement.querySelector('#content');
        autoCompletePreviewElement = rootElement.querySelector('#autocomplete-preview');
        clearButtonElement = rootElement.querySelector('#clear');
        clearButtonElement.addEventListener('click', clear);
        spinner = rootElement.querySelector('.select-autocomplete-spinner');
        popUp = rootElement.querySelector('.select-autocomplete-pop-up');
        inputElement.addEventListener('keyup', keyStrokeHandler);
        inputElement.addEventListener('keydown', commandKeyStrokeHandler);
        zhLoader = document.querySelector('.zh-background-loading');
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        let backButton = rootElement.querySelector('#back-button');
        backButton.addEventListener('click', ApplicationContext.UIManager.goBack);
        listElement.addEventListener('touchstart', closeKeyboard);
        inputElement.placeholder = "Pesquise " + $scope.field.label;

        baseListRowToClone = rootElement.querySelector('.select-autocomplete-row').cloneNode(true);
        baseListRowToClone.style.display = 'flex';
        if(!$scope.isMultiple) {
            let checkbox = baseListRowToClone.querySelector('input[type=checkbox]');
            checkbox.style.display = 'none';
        }
        
        if ($scope.field.transformOrigin) {
            popUp.style.transformOrigin = $scope.field.transformOrigin; 
        }

        if($scope.beforeSelectAutoCompleteInitialized) {
            $scope.beforeSelectAutoCompleteInitialized($scope);
        }
        initiliazefieldInfo();
        initiliazeData();
        initiliazeDataSetter();
        $scope.field.clearValue = $scope.field.clear = clear;

        $scope.field.setDescriptionText = newDescriptionText => {
            $scope.descriptionText = newDescriptionText;
            $scope.$apply();
        }

        if($scope.onSelectAutoCompleteInitialized) {
            $scope.onSelectAutoCompleteInitialized($scope);
        }
    };

    $scope.field.autoCompleteScope = $scope;

    /**
     * render new list
     * 
     * @param {Object[]} listElements an array of json objects to be rendered  
     */
    function renderListElements(listElements) {

        while (listElement.lastChild) {
            listElement.removeChild(listElement.lastChild);
        }
        for (var index = 0; index < 200/*8*/ && index < listElements.length; index++) {
            var newHtmlListElement = baseListRowToClone.cloneNode(true);
            newHtmlListElement.children[1].children[0].textContent = getFormatedString(listElements[index], 0);
            newHtmlListElement.children[1].children[1].children[0].textContent = getFormatedString(listElements[index], 1);
            newHtmlListElement.children[1].children[1].children[1].textContent = getFormatedString(listElements[index], 2);

            newHtmlListElement.realIndex = index;
            newHtmlListElement.addEventListener('click', confirm);
            listElement.appendChild(newHtmlListElement);
        }
        if (index > 0) {
            if (currentHighletedListElement)
                currentHighletedListElement.classList.remove('active');
            currentHighletedListElement = listElement.children[0];
            currentHighletedListElement.classList.add('active');
        }
    }


    /**
     * @param {Object} row - a object to extract the values for the display string
     * @returns {string} a string with the descriptionFields that are in the row concated with pipe
     */
    $scope.getDescriptionString = function(row){
        if($scope.field.descriptionFields){
            return  $scope.field.descriptionFields.map(descriptionField => row[descriptionField]).join(' | ');
        } else {
            return row[$scope.field.name];
        }
    }

    /**
     * returns an String with the field name concatenated, or an empty string if the object doesn't have that field  
     * @returns {String} 
     * @param {Obect} object 
     * @param {Number} index the index of the field that is being searched on the fieldInfo array 
     */
    function getFormatedString(object, index) {
        if (fieldInfo.length <= index && object) {
            return "";
        }
        else if (object.hasOwnProperty(fieldInfo[index].name)) {
            var value = object[fieldInfo[index].name];
            if (fieldInfo[index].mask) {
                value = Util.applyMask(value, fieldInfo[index].mask, maskEngine, ZhUtil);
            }
            return fieldInfo[index].label + value;
        }
        else {
            return "";
        }
    }

    /**
     * @param {string} stringToSearch 
     * @param {object[]} arraryToSearch
     * @param {boolean} [cleanReturn] - if is set to true the method doesn't affect 'filteredListContents' and returns a new array instead 
     */
    "use strict";
    function filterElements(stringToSearch, arraryToSearch, cleanReturn) {
        var workArray;
        arraryToSearch = arraryToSearch || listContents;
        if (!cleanReturn) {
            workArray = filteredListContents;
        }
        workArray = [];
        for (var i = 0; /*i < 9 && */ i < arraryToSearch.length; i++) {
            if (stringIsInJSON(arraryToSearch[i], stringToSearch))
                workArray.push(arraryToSearch[i]);
        }
        if (cleanReturn)
            return workArray;
    }

    /**
     * //TODO refactor docs// search all properties of a json object to find a string
     * @param {Object} object 
     * @param {String} stringToMatch 
     */
    function stringIsInJSON(object, stringToMatch) {
        stringToMatch = stringToMatch.toLowerCase();
        for (var i = fieldInfo.length; i--;) {
            if (('' + object[fieldInfo[i].name]).toLowerCase().indexOf(stringToMatch) !== -1)
                return true;
        }
        return false;
    }

    /** 
     * @param {MouseEvent} event - a event to extract the list html element
     * @param {HTMLElement} listElement - a list element to select
     */
    var confirm = function(event, listElement) {
        var element;
        var shouldForceDigest = true;
        if (event) {
            element = event.currentTarget;
        }
        else {
            element = listElement;
            shouldForceDigest = true;
        }
        var newValueObject = filteredListContents[element.realIndex];
        setValue(newValueObject);
        
        inputElement.blur();
        clearButtonElement.style.display = 'initial';
        ApplicationContext.UIManager.goBack();
        if (shouldForceDigest) {
            $scope.$apply();
        }
    };

    /**
     * the default value change handler
     */
    $scope.__onSelectAutoCompletedValueChanged = () => {
        $scope.fieldRow = $scope.__fieldRow;
    };


    /**
     * updates the UI and changes the internal label info vars 
     * @param {string} newLabel
     */
    $scope.field.setLabel = (newLabel) => {
        $scope.field.label = newLabel;
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        inputElement.placeholder = "Pesquise " + $scope.field.label;
    };

    $scope.field.setValue = (newValue) => {
        if(typeof newValue === 'object' && newValue !== null){
            setValue(newValue);
        } else if($scope.field.name && newValue){
            let setValueArgs = {};
            setValueArgs[$scope.field.name] = newValue;
            setValue(setValueArgs);
        }
    };

    /**
     * refreshes the ui, changes internal variables and trigger listeners
     * TODO: refactor this doc
     * @param {Object} newValue 
     */
    let setValue = function(newValue){

        if($scope.field.rowDump){
            $scope.__fieldRow = newValue;
        } else {
            let newValueString = newValue[$scope.field.name];
            $scope.__fieldRow = newValueString;
            inputElement.value = newValueString;
        }
        $scope.descriptionText = $scope.getDescriptionString(newValue);
        
        if($scope.onSelectAutoCompletedValueChanged){
            $scope.onSelectAutoCompletedValueChanged(newValue, $scope);
        } else {
            $scope.__onSelectAutoCompletedValueChanged();
        }

        insertOutData(newValue);
        if(events.FieldOnChange) {
            $scope.$eval(events.FieldOnChange, Object.assign({}, ApplicationContext, {
                args: {
                    owner: {field: $scope.field},
                    row: $scope.field.widget.currentRow
                }
            }));
        }
    }

    /**
     * clear the value of the current row, and the applied filter
     * @param {Boolean} [clearOutData] wheter or not the clear function should remove 
     * the values associated with this field on the OutData prop from the currentRow
     */
    function clear(clearOutData = false, forceUpdate = false) {
        if (activeRequests)
            cancelRequest();
        $scope.__fieldRow = null;
        $scope.__onSelectAutoCompletedValueChanged();
        inputElement.value = ""; 
        $scope.descriptionText = "";
        clearButtonElement.style.display = 'none';
        if (!cacheBase.hasOwnProperty('') || forceUpdate) {
            update();
        }
        else {
            $scope.field.dataSource.data = cacheBase[""];
        }
        if(clearOutData){
            destroyOutData();
        }
        if($scope.onClearAutoCompleteValue){
            $scope.onClearAutoCompleteValue($scope);
        }
    }

    /**
     * 
     * @param {Object} rowObject -- a ZhRowObject to extract the data and insert on the currentRow
     */
    function insertOutData(rowObject) {
        for (var key in $scope.field.outData) {
            var value = rowObject[key];
            $scope.field.widget.currentRow[$scope.field.outData[key]] = value;
        }
    }

    function destroyOutData() {
        for (var key in $scope.field.outData) {
            delete $scope.field.widget.currentRow[$scope.field.outData[key]];
        }
    }

    async function fetchData() {
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        ScreenService.hideLoader();

        var dataSourceFilter = $scope.field.dataSourceFilter || $scope.field.dataSource.dataSourceFilter || [];
        var newFilter = dataSourceFilter.slice(0); 

        newFilter.push({
            name: filterConditions.name,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        });

        var promisseToReturn = await $scope.field.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet && resultSet.dataset) {
                doCache(deferredToCancelRequest.reqText, 
                    filteredAditionalOptions.concat(resultSet.dataset[Object.keys(resultSet.dataset)[0]])
                );
            }
            else {
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        
        return promisseToReturn;
    }

    $scope.openPopUp = () => {
        popUp.classList.add('visible');
        inputElement.focus();
        
        ApplicationContext.UIManager.pushState(() => {
            $scope.closePopUp(); 
        });
        
        zhLoader.classList.add('invisible');
    };
    
    $scope.closePopUp = () => {
        popUp.classList.remove('visible');
        zhLoader.classList.remove('invisible');
    }

}


// FILE: schoolmeals_components/fields/js/SelectAutocompleteMultipleController.js
'use strict';
function SelectAutocompleteMultipleController(
    $scope,
    $element,
    ScreenService,
    ZhUtil,
    maskEngine,
    restEngine,
    $q,
    ZhLogger,
    ApplicationContext
    ){

    
    /**
     * @type {Array<String>}
     */
    let keys = [];
    /**
     * @type {Array<Object>}
     */
    $scope.__fieldRow = [];
    /**
     * @type {Object.<String, Object>}
     */
    $scope.selectedElements = {};
    /**
     * @type {Object}
     */
    var events = {};
    // document.querySelector('.zh-notification-messages').style.display = 'none';
    var activeRequests = [];

    /**
     * @type {Object}
     * a object that will precache the results of each query
     */
    var cacheBase = {};
    /**
     * the spinner element that indicates progress   
     * @type {HTMLElement}
     */
    var spinner = null;
    /**
     * a reference to the current highlited element of the list   
     * @type {HTMLElement}
     */
    var currentHighletedListElement = null;
    /**
     * @type {Object[]} 
     */
    var filteredAditionalOptions = $scope.field.aditionalOptions || [];
    /**
     * @type {Object[]} 
     */
    var filteredListContents = [];
    /** it will be initialized on the initializeData method
     * @type {listContents}
     */
    var listContents = null;
    /**
     * @type {HTMLElement}
     */
    var rootElement;
    /**
     * @type {HTMLElement}
     */
    var popUp;
    /**
     * @type  {HTMLElement}
     */
    var listElement;
    /**
     * @type {HTMLElement}
     */
    var inputElement;
    /**
     * @type {HTMLElement}
     */
    var zhLoader;
    /**
     * @type {HTMLElement}
     */
    var autoCompletePreviewElement;
    /**
     * @type {HTMLElement}
     */
    var labelElement;
    /**
     * @type {HTMLElement}
     */
    var clearButtonElement;
    /**
     * @type {Object[]}
     */
    var fieldInfo = [];

    /**
     * an element that will be used as a basis to render each list element 
     * @type {HTMLElement}
     */
    var baseListRowToClone;
    /**
     * the root element of the chips 
     * @type {HTMLDivElement}
     */
    let chipList;
    /**
     * an element for showing chips and opening the popup 
     * @type {HTMLDivElement}
     */
    let contentDisplay;

    var filterConditions = {
        name: "",
        value: "%%",
        operator: "LIKE_ALL"
    };

    var shouldPreventKeyUp = false;
    var lastUpdateTime;
    var updateRequestReference = null;
    var deferredToCancelRequest = false;

    var cacheFirstResultSet = function() {
        cacheBase[""] = listContents;
    };

    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    var keyStrokeHandler = function(event) {
        if (event.keyCode > 45 || event.keyCode === 8) {
            if (activeRequests.length) {
                if (activeRequests[0].reqText === inputElement.value) {
                    return;
                }
                else {
                    cancelRequest();
                }
            }
            if (cacheBase[inputElement.value]) {
                filteredListContents = cacheBase[inputElement.value];
                renderListElements(cacheBase[inputElement.value]);
                return;
            }
            filteredAditionalOptions = filterElements(inputElement.value, $scope.field.aditionalOptions || [], true);
            var filtered = filterElements(inputElement.value, null, true);
            renderListElements(filtered);
            update();
        }
        shouldPreventKeyUp = true;
    };

    /**
     * handles the command keys keypress, or propagates the event chain for the keyStrokeHandler to resolve
     * @param {KeyboardEvent} [event] 
     */
    var commandKeyStrokeHandler = function(event) {
        var code = event.keyCode;
        if (code > 45 || code === 8) {
            shouldPreventKeyUp = false;
            return;
        }
        else if (code === 40 /* down arrow */ ) {
            if (currentHighletedListElement.parentElement.lastChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.nextElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 38 /* up arrow */ ) {
            if (currentHighletedListElement.parentElement.firstChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.previousElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 13) /* enter */ {
            confirm(null, currentHighletedListElement);
        }
        shouldPreventKeyUp = true;
    };

    /**
     * shows the first element of the filteredListContents array
     */
    var showAutoCompleteText = function() {
        autoCompletePreviewElement.value = getFormatedString(filteredListContents[0], 0);
    };

    var refreshLastUpdateTime = function() {
        lastUpdateTime = performance.now();
    };
    var hideSpinner = function() {
        spinner.classList.add('hidden');
    };
    var onRequestComplete = function() {
        hideSpinner();
        refreshLastUpdateTime();
    };
    /**
     * @returns {Promise<null>} 
     */
    var update = function() {
        if (updateRequestReference !== null) {
            clearTimeout(updateRequestReference);
            updateRequestReference = null;
        }
        filterConditions.value = "%" + inputElement.value + "%";
        return fetchData().then(refreshLastUpdateTime);
    };

    var cancelRequest = function() {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
    };

    var initiliazefieldInfo = function() {
        if ($scope.field.fields) {
            for (var i = 0; i < $scope.field.fields.length; i++) { 
                var label = ($scope.field.fields[i].label ? ($scope.field.fields[i].label + ": ") : "");
                fieldInfo.push({
                    "label": label,
                    "name": $scope.field.fields[i].name,
                    "mask": $scope.field.fields[i].mask || false
                });
                filterConditions.name += $scope.field.fields[i].name + '|';
            }
            filterConditions.name = filterConditions.name.substr(0, filterConditions.name.length - 1);
        }
        else {
            fieldInfo.push({
                "label": "" /*labelWithNoExtraFields*/ ,
                "name": $scope.field.name,
                "mask": $scope.field.mask || false
            });
            filterConditions.name = $scope.field.name;
        }
        $scope.field.dataSource.itemsPerPage = 100000;
        $scope.field.dataSource.disableLoader = true;

        if($scope.field.events){
            let fieldOnChangeEvent = $scope.field.events.find(event => event.name === 'FieldOnChange');
            if(fieldOnChangeEvent){
                events.FieldOnChange = fieldOnChangeEvent.code;
            }
        }

        keys = $scope.field.dataSource.getVO().localKey;

    };

    var initiliazeDataSetter = function() {
        if ($scope.field.dataSource) {
            Object.defineProperty($scope.field.dataSource, 'data', {
                get: function() {
                    return listContents;
                },
                set: function(newValue) {
                    if (newValue === listContents) {
                        return;
                    }
                    newValue = [...filteredAditionalOptions].concat([...newValue]);
                    filteredListContents = listContents = newValue;
                    renderListElements(listContents);
                    if(!$scope.field.descriptionTextRendered){
                        let selectedElement = $scope.field.dataSource.data.find(dataSourceElement => {
                            return dataSourceElement[$scope.field.name] === $scope.__fieldRow;
                        });
                        if(selectedElement){
                            $scope.descriptionText = $scope.getDescriptionString(selectedElement);
                        }
                    }
                },
                enumerable: true,
                configurable: true
            });
        }

        

    };

    var doCache = function(cacheKey, cacheList) {
        if(cacheKey !== ""){
            cacheBase[cacheKey] = cacheList;
        }
    };

    var initiliazeData = function() {
        if ($scope.field.dataSource.data) {
            filteredListContents = listContents = $scope.field.dataSource.data;
            let arrayToRender = listContents;
            if(!filteredAditionalOptions.every(aditionalOption => listContents.includes(aditionalOption))){
                listContents = filteredAditionalOptions.concat(listContents);
            }
            renderListElements(arrayToRender);
        }
        else {
            listContents = [];
        }
        if (!$scope.field.dataSource.dataSourceFilter) {
            $scope.field.dataSource.dataSourceFilter = [];
        }
    };

    $scope.autoCompleteConstructor = function() {
        rootElement = $element.get(0);
        listElement = rootElement.querySelector('#element-list');
        inputElement = rootElement.querySelector('#content');
        autoCompletePreviewElement = rootElement.querySelector('#autocomplete-preview');
        clearButtonElement = rootElement.querySelector('#clear');
        clearButtonElement.addEventListener('click', clear);
        spinner = rootElement.querySelector('.select-autocomplete-spinner');
        popUp = rootElement.querySelector('.select-autocomplete-pop-up');
        chipList = rootElement.querySelector('#select-autocomplete-chips');
        contentDisplay = rootElement.querySelector('#select-autocomplete-content-display');
        inputElement.addEventListener('keyup', keyStrokeHandler);
        inputElement.addEventListener('keydown', commandKeyStrokeHandler);
        inputElement.placeholder = "Pesquise " + $scope.field.label;
        zhLoader = document.querySelector('.zh-background-loading');
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        let backButton = rootElement.querySelector('#back-button');
        let confirmButton = rootElement.querySelector('#confirm-button');
        backButton.addEventListener('click', ApplicationContext.UIManager.goBack);
        confirmButton.addEventListener('click', $scope.closePopUp);

        

        baseListRowToClone = rootElement.querySelector('.select-autocomplete-row').cloneNode(true);
        baseListRowToClone.style.display = 'flex';

        if($scope.beforeSelectAutoCompleteInitialized) {
            $scope.beforeSelectAutoCompleteInitialized($scope);
        }

        $scope.__fieldRow = $scope.__fieldRow || [];
        initiliazefieldInfo();
        initiliazeData();
        initiliazeDataSetter();
        if($scope.onSelectAutoCompleteMultipleInitialized) {
            $scope.onSelectAutoCompleteMultipleInitialized($scope);
        }


        chipList.addEventListener('click', removeChip);
    };

    let removeChip = event => {
        let clickedChip = event.target;
        let id = clickedChip.getAttribute('__hashkey');
        delete $scope.selectedElements[id];
        renderChips();
        Array.from(listElement.children).forEach(listRow => {
            if(listRow.__hashKey === id) {
                listRow.classList.remove('checked');
            }
        });
        if(!$scope.field.outData){
            $scope.__fieldRow = [];
            for(selectedElementKey in $scope.selectedElements){
                $scope.__fieldRow.push($scope.selectedElements[selectedElementKey][$scope.field.name]);
            }
        }
    };

    /**
     * render new list
     * 
     * @param {Array<Object>} listElements an array of objects to be rendered  
     */
    function renderListElements(listElements) {

        while (listElement.lastChild) {
            listElement.removeChild(listElement.lastChild);
        }
        for (var index = 0; /*index < 8 && */ index < listElements.length; index++) {
            var newHtmlListElement = baseListRowToClone.cloneNode(true);
            newHtmlListElement.children[1].children[0].textContent = getFormatedString(listElements[index], 0);
            newHtmlListElement.children[1].children[1].children[0].textContent = getFormatedString(listElements[index], 1);
            newHtmlListElement.children[1].children[1].children[1].textContent = getFormatedString(listElements[index], 2);

            if(isSelected(listElements[index])){
                newHtmlListElement.classList.add('checked');
            }
            newHtmlListElement.realIndex = index;
            newHtmlListElement.__hashKey = getHashKey(listElements[index]);
            newHtmlListElement.addEventListener('click', confirm);
            listElement.appendChild(newHtmlListElement);
        }
        if (index > 0) {
            if (currentHighletedListElement)
                currentHighletedListElement.classList.remove('active');
            currentHighletedListElement = listElement.children[0];
            currentHighletedListElement.classList.add('active');
        }
    }


    /**
     * @param {Object} [row] - a object to extract the values for the display string
     * @returns {string} a string with the descriptionFields that are in the row concated with pipe
     */
    $scope.getDescriptionString = function(row){
        if(row){
            let descriptionFields = $scope.field.descriptionChips || $scope.field.descriptionFields || [];
            return descriptionFields.map(descriptionField => row[descriptionField]).join(' | ');
        } else {
        let descriptionFields = ($scope.field.descriptionFields || []).concat($scope.field.name);
            let descriptionString = "";
            for(selectedElementKey in $scope.selectedElements){
                let selectedValue = $scope.selectedElements[selectedElementKey];
                descriptionString += 
                    descriptionFields.map(descriptionField => selectedValue[descriptionField]).join('|') + '🔝';
            }
            return descriptionString.slice(0, -1);
        }
    }

    /**
     * returns an String with the field name concatenated, or an empty string if the object doesn't have that field  
     * @returns {String} 
     * @param {Object} object 
     * @param {Number} index the index of the field that is being searched on the fieldInfo array 
     */
    function getFormatedString(object, index) {
        if (fieldInfo.length <= index && object) {
            return "";
        }
        else if (object.hasOwnProperty(fieldInfo[index].name)) {
            var value = object[fieldInfo[index].name];
            if (fieldInfo[index].mask) {
                value = Util.applyMask(value, fieldInfo[index].mask, maskEngine, ZhUtil);
            }
            return fieldInfo[index].label + value;
        }
        else {
            return "";
        }
    }

    /**
     * @param {String} stringToSearch 
     * @param {Array<Object>} arraryToSearch
     * @param {Boolean} [cleanReturn] - if is set to true the method doesn't affect 'filteredListContents' and returns a new array instead 
     */
    "use strict";
    function filterElements(stringToSearch, arraryToSearch, cleanReturn) {
        var workArray;
        arraryToSearch = arraryToSearch || listContents;
        if (!cleanReturn) {
            workArray = filteredListContents;
        }
        workArray = [];
        for (var i = 0; /*i < 9 && */ i < arraryToSearch.length; i++) {
            if (stringIsInJSON(arraryToSearch[i], stringToSearch))
                workArray.push(arraryToSearch[i]);
        }
        if (cleanReturn)
            return workArray;
    }

    /**
     * //TODO refactor docs// search all properties of a json object to find a string
     * @param {Object} object 
     * @param {String} stringToMatch 
     */
    function stringIsInJSON(object, stringToMatch) {
        stringToMatch = stringToMatch.toLowerCase();
        for (var i = fieldInfo.length; i--;) {
            if (('' + object[fieldInfo[i].name]).toLowerCase().indexOf(stringToMatch) !== -1)
                return true;
        }
        return false;
    }

    /** 
     * @param {MouseEvent} event - a event to extract the list html element
     * @param {HTMLElement} listElement - a list element to select
     */
    var confirm = function(event, listElement) {
        var element;
        var shouldForceDigest = true;
        if (event) {
            element = event.currentTarget;
        }
        else {
            element = listElement;
            shouldForceDigest = true;
        }
        var newValueObject = filteredListContents[element.realIndex];
        setValue(newValueObject, element);
        
        if (shouldForceDigest) {
            $scope.$apply();
        }
    };


    /**
     * @param {Object} dataSourceElement a ZhRowObject to generate a key
     */
    let getHashKey = (dataSourceElement) => {
        return keys.map(currentKey => dataSourceElement[currentKey]).join("");
    };

    /**
     * @param {Object} dataSourceElement a ZhRowObject
     */
    let isSelected = (dataSourceElement) => getHashKey(dataSourceElement) in $scope.selectedElements;

    
    let renderChips = function(){
        let selectedElementsKeys = Object.keys($scope.selectedElements);
        let elementChips = selectedElementsKeys.map(selectedElement => `
            <span class="select-autocomplete-chip" 
                __hashKey="${getHashKey($scope.selectedElements[selectedElement])}"
            > 
                ${$scope.getDescriptionString($scope.selectedElements[selectedElement])}
                <i class="fas fa-times-circle select-autocomplete-chip-close-button"></i>
            </span>
        `).join("");
        chipList.innerHTML = elementChips;

        contentDisplay.innerHTML = chipList.innerHTML;
        contentDisplay.scrollTo(0,0); 
    };

    /**
     * refreshes the ui, changes internal variables and trigger listeners
     * TODO: refactor this doc
     * @param {Object} newValue 
     * @param {HTMLElement} [element] 
     */
    let setValue = function(newValue, element){
        if(!isSelected(newValue)) {
            $scope.selectedElements[getHashKey(newValue)] = newValue;
            if(element) {
                element.classList.add('checked');
            }
        } else {
            delete $scope.selectedElements[getHashKey(newValue)];
            if(element) {
                element.classList.remove('checked');
            }
        }
        if(Object.keys($scope.selectedElements).length <= 0){
            clearButtonElement.style.display = 'initial';
        }

        if(!$scope.field.outData){
            $scope.__fieldRow = [];
            for(var selectedElementKey in $scope.selectedElements){
                if($scope.selectedElements.hasOwnProperty(selectedElementKey)){
                    $scope.__fieldRow.push($scope.selectedElements[selectedElementKey][$scope.field.name]);
                }
            }
        }

        if($scope.onSelectAutoCompletedValueChanged){
            $scope.onSelectAutoCompletedValueChanged(newValue, $scope);
        }

        if(events.FieldOnChange) {
            $scope.$eval(events.FieldOnChange, Object.assign({}, ApplicationContext, {
                args: {
                owner: {field: $scope.field},
                row: $scope.field.widget.currentRow
                }
            }));
        }

        renderChips();
    }

    /**
     * clear the value of the current row, and the applied filter
     * @param {MouseEvent} event 
     * @param {Boolean} [preventUpdate] wheter or not the clear function should reload the data 
     */
    function clear(event, preventUpdate) {
        if (activeRequests) cancelRequest();
        $scope.__fieldRow = null;
        inputElement.value = "";
        
        if (!cacheBase.hasOwnProperty('')) {
            update();
        }
        else {
            $scope.field.dataSource.data = cacheBase[""];
        }
        
        $scope.selectedElements = {};
        renderChips();
        destroyOutData();
        $scope.field.inputHasContent = false;
    }

    $scope.clear = ()=>{
        ScreenService.confirmMessage('Deseja limpar?').then(clear);
    };

    /**
     * 
     * @param {Object} rowObject -- a ZhRowObject to extract the data and insert on the currentRow
     */
    function insertOutData(rowObject) {
        for (var key in $scope.field.outData) {
            var value = rowObject[key];
            $scope.field.widget.currentRow[$scope.field.outData[key]] = value;
        }
    }

    function destroyOutData() {
        for (var key in $scope.field.outData) {
            delete $scope.field.widget.currentRow[$scope.field.outData[key]];
        }
    }

    async function fetchData() {
        
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        
        ScreenService.hideLoader();
        var dataSourceFilter = $scope.field.dataSourceFilter || 
                               $scope.field.dataSource.dataSourceFilter || [];
        var newFilter = dataSourceFilter.slice(0); 
        newFilter.push({
            name: filterConditions.name,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        });

        var promisseToReturn = await $scope.field.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet && resultSet.dataset) {
                doCache(deferredToCancelRequest.reqText, 
                    filteredAditionalOptions.concat(resultSet.dataset[Object.keys(resultSet.dataset)[0]])
                );
            }
            else {
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        
        return promisseToReturn;
    }

    $scope.openPopUp = () => {
        popUp.classList.add('visible');
        inputElement.focus();
        
        ApplicationContext.UIManager.pushState(() => {
            $scope.closePopUp(); 
        });
        
        zhLoader.classList.add('invisible');
    };
    
    $scope.closePopUp = () => {
        popUp.classList.remove('visible');
        zhLoader.classList.remove('invisible');
    }

    /**
     * the default value change handler
     */
    $scope.__onSelectAutoCompletedValueChanged = () => {
        $scope.fieldRow = $scope.__fieldRow;
    };
}


// FILE: schoolmeals_components/header/js/HeaderTemplateController.js
"use strict";

function HeaderTemplateController($scope, templateManager, ApplicationContext) {

    function getContainer() {
		return templateManager.container;
    }
    
    function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

    $scope.$watch(() => getContainer().getActiveWidget().name, () => {
        $scope.init();
    });
    
    $scope.$watch(() => getContainer().getActiveWidget().title, () => {
        updateTitle();
    });

    function updateTitle() {
        getContainer().title = getActiveWidget().title || getTitle();
    };


    $scope.init = () => {
        getContainer().title = getActiveWidget().title || getTitle();
        let header = document.getElementById('headerComponent');
        let headerActions = getActiveWidget().headerActions || getContainer().headerActions;
        if (headerActions && header) {
            let left = headerActions.left ? headerActions.left.length : 0;
            let right = headerActions.right ? headerActions.right.length : 0;
            let totalAc = [left, right];
            let coluns = '';

            for (let i = 0; i < totalAc.length; i++) {
                for (let j = 0; j < totalAc[i]; j++) {
                    coluns += '50px ';
                }
                coluns += i == 0 ? 'auto ' : '';
            }
            header.style.gridTemplateColumns = coluns;

            getContainer().openHeaderMenu = openHeaderMenu;
            getContainer().displayedActions = {};
            getContainer().displayedActions.left = headerActions && headerActions.left ? headerActions.left : [];
            getContainer().displayedActions.right = headerActions && headerActions.right ? headerActions.right : [];
            getContainer().displayedActions.menu = headerActions && headerActions.menuActions ? headerActions.menuActions : [];
            getContainer().fabVisibility = (getActiveWidget().fabAction || getActiveWidget().fabActions) ? true: false;
            getContainer().fabActionIcon = getActiveWidget().fabActionIcon || 'fp-add';
            $scope.showMenuActions = false;
            $scope.menuActions = getContainer().displayedActions.menu;
        }

		document.documentElement.style.setProperty('--headerHeight', (getContainer().showSubHeader ? "85px" : "51px")); 
		if(getContainer().subHeader === undefined){
			getContainer.subHeader = {};
		} else {
			if(getContainer().subHeader.evalText){
				getContainer().subHeader.text = $scope.$eval(getContainer().subHeader.evalText, ApplicationContext);
			}
		}
    };

    $scope.shouldShowHeader = () => {
        return getContainer().showHeader;
    };

    function getTitle() {
        return getActiveWidget().label || getContainer().name;
    };

    $scope.executeControllerAction = (code) => {
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
    
    $scope.executeFABAction = () => {
        let widget = getActiveWidget();
        if(widget.fabActions){
            $scope.fabActionsVisible = true;
            $scope.fabActions = widget.fabActions;
        } else if(widget.fabAction){
            let code = getActiveWidget().fabAction;
            $scope.executeControllerAction(code);
        }
    };

    $scope.closeFABActions = () => {
        $scope.fabActionsVisible = false;
        delete $scope.fabActions;
    };

    $scope.executeFABActionCode = (code) => {
        $scope.closeFABActions();
        $scope.executeControllerAction(code);
    };

    const openHeaderMenu = () => $scope.showMenuActions = true;

    $scope.closeHeaderMenu = () => $scope.showMenuActions = false;

    $scope.onMenuActionClick = (code,)=>{
        $scope.executeControllerAction(code);
        $scope.showMenuActions = false;
    };
    
    $scope.shouldShowSubHeader = () => $scope.shouldShowHeader() && getContainer().showSubHeader;
    
    getContainer().setSubHeaderVisibility = (visibility) => {
    	document.documentElement.style.setProperty('--headerHeight', (visibility ? "85px" : "51px"));
    	getContainer().showSubHeader = visibility;
    	$scope.$apply();
    };
    
    getContainer().setSubHeaderText = (text) => {
    	getContainer().subHeader.text = text;
    	$scope.$apply();
    };
}


// FILE: schoolmeals_components/lists/js/ListForGeneralUse.js
"use strict";

function ListForGeneralUse($scope, ApplicationContext, templateManager, ScreenService, $element) {
    
    $scope.testContent = (e, field) => {
        let fieldValue = e.currentTarget.value;
        field.inputHasContent = (fieldValue && fieldValue.length > 0 && fieldValue != "?");
    };

    $scope.testContentWithFieldValue = (field) => {
        field.inputHasContent = (field.value()!==undefined && field.value()!==null);
    };
    
    $scope.getFormatedFieldValue = (field, silm) => {
        let fieldValue = field.prefix || '';
        
        if(silm[field.name]!==undefined && silm[field.name]!==null){
            fieldValue += silm[field.name];
        }

        return fieldValue;
    };

    $scope.initAutocompleteField = field => {
        $scope.descriptionText = field.widget.currentRow[field.name];
        $scope.testContentWithFieldValue(field);
    };

    $scope.formatData = (data) => {
        let formattedDate = {};

        if(moment(data, 'DD/MM/YYYY hh:mi:ss').isValid()){
            formattedDate = moment(data, 'DD/MM/YYYY hh:mi:ss');
            moment.locale('pt-br');
        } else {
            formattedDate = moment(data);
        }
        
        return formattedDate.format('ddd, DD [de] MMM [de] YYYY');
    };

    $scope.setAvabilityStatus = (widget) => {
        if(widget.idForEnableButton){
            let hasDifStatus = widget.dataSource.data.filter(d => d[widget.idForEnableButton.name] === widget.idForEnableButton.value);
            if (hasDifStatus && hasDifStatus.lenght) {
                hasDifStatus.forEach(hds => {
                    hds.disabled = true;
                });
            }
        }
    };

    $scope.onSelectAutoCompletedValueChanged = (selectedValueObject, fieldScope) => {
        let field = fieldScope.field;
        field.inputHasContent = selectedValueObject && selectedValueObject !== {} && selectedValueObject !== null;
        if (field.inputHasContent) {
            fieldScope.__onSelectAutoCompletedValueChanged();
        }
    };

    let executeScopeCode = (code) => {
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };

    $scope.executeScopeCode = executeScopeCode;

    $scope.openDetail = (afterClickOnItemWidget, curItem) => {
        templateManager.container.currentItem = curItem;
        executeScopeCode(afterClickOnItemWidget);
    };

    $scope.initListWithConfirmationButton = (widget) => {
        initToList(widget);
    };

    $scope.goBack = () => ApplicationContext.UIManager.goBack();

    let render = function(filteredData, widget){
        widget.dataSource.data = filteredData;
        $scope.$apply();
    };

    let searchAdapter;
    const initToList = (widget) => {
        const UIManager = ApplicationContext.UIManager;
        
        searchAdapter = new SearchAdapter($element[0].querySelector('.search-input'), widget, 
        (filteredData) => render(filteredData, widget), 
        {
            local: true,
            dataSourceOrigin: widget.dataSource,
            dataSourcePropName: "__copyData",
            searchFields: widget.searchFields
        });
        
        widget.showSearch = () => {
            $scope.isSearching = true;
            widget.container.fabVisibility = false;
            widget.dataSource.__copyData = angular.copy(widget.dataSource.data);

            UIManager.pushState(() => {
                $scope.isSearching = false;
                widget.dataSource.data = angular.copy(widget.dataSource.__copyData);

                let inputElement = $element[0].querySelector('.search-input');
                inputElement.value = "";
                inputElement.blur();

                widget.container.fabVisibility = true;
                if('cordova' in window){
                    $scope.$apply();
                }
            });

            let input = $element[0].querySelector('.search-input');
            setTimeout(() => input.focus(), 10);
        }
    };

    $scope.concAction = (sim, e) => {
        e.stopPropagation();

        let widget = $scope.widget;
        widget.currentSimToConclude = sim;
        let msg = '<b>Concluir Simulação ' + sim.CDVERSAOPROPCOM + '?</b><br>';
        msg += '<span style="font-size:.8rem">Ao concluir, você não poderá alterar nenhuma informação.</span>';
        ScreenService.confirmMessage(msg, 'question', () => {
            executeScopeCode(widget.executeOnConclusion);
        });
    };

    $scope.setSizeClass = () => {
        let widget = $scope.widget.container.getActiveWidget();

        let hasHeader = widget.showHeader || widget.container.showHeader ? 50 : 0;
        let insideTabs = widget.isInsideTab ? 50 : 0;
        let actionsSize = widget.actions.filter(a => a.isVisible == true).length;

        let listNewSize = hasHeader + insideTabs + (actionsSize * 70);

        document.documentElement.style.setProperty('--simpleListSize', 'calc(100vh -' + listNewSize + 'px)');
    };

    $scope.startDeletion = (e, widget, item) => {
        let msg = 'Deseja realmente excluir este item?';
        let slided = e.currentTarget;

        widget.container.itemToExclude = item;
        slided.classList.add('beingDeleted');
        setTimeout(() => {
            ScreenService.confirmMessage(msg, 'question', () => {
                if (slided) {
                    slided.classList.remove('beingDeleted');
                }
                executeScopeCode(widget.excludeAction);
            }, () => {
                if (slided) {
                    slided.classList.remove('beingDeleted');
                }
            });
        }, 500);
    };

    $scope.itemLongTapAction = (tappedItem)=>{

        if($scope.widget.itemLongTapAction){

            $scope.$eval(
                $scope.widget.itemLongTapAction,
                Object.assign({}, ApplicationContext, {
                    args: {
                        owner: $scope.widget,
                        item: tappedItem
                    }
                })
            );

        } else if($scope.widget.allowSelectOnLongTap){
            $scope.selectLongTappedItem(tappedItem);
        }
    };

    $scope.selectLongTappedItem = item =>{
        if(item.__isSelected){
            if($scope.widget.beforeUnselectRow)
                $scope.widget.beforeUnselectRow();
            
            $scope.widget.dataSource.removeCheckedRows(item);

            if($scope.widget.afterUnselectRow)
                $scope.widget.afterUnselectRow();
        } else{
            if($scope.widget.beforeSelectRow)
                $scope.widget.beforeSelectRow();

            $scope.widget.dataSource.addCheckedRows(item);
            
            if($scope.widget.afterSelectRow)
                $scope.widget.afterSelectRow();
        }

    };

    $scope.toggleValue = field =>{
        field.widget.currentRow[field.name] = !field.widget.currentRow[field.name];
        if(field.change){
            field.change(field);
        }
    };
}

// FILE: schoolmeals_components/lists/js/ListOfSquareItens.js
"use strict";

function ListOfSquareItens($scope, ApplicationContext) {
    $scope.doSquareClickAction = (widget, target) => {
        let code = widget.squareClickAction;
        widget.squareClicked = target;
        
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
}

// FILE: schoolmeals_components/lists/js/ListWithTitleAndActions.js
"use strict";

function ListWithTitleAndActions($scope, ApplicationContext, templateManager, ScreenService) {

    let count = 5;

    $scope.openDetail = (afterClickOnItemWidget, curItem) => {
        templateManager.container.currentSubItem = curItem;
        let code = afterClickOnItemWidget;
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
    
    $scope.stopPropagation = e => {
        e.stopPropagation();
        count --;
        if(count < 0){
            count = 2;
            ScreenService.showMessage('TO_DELETE_PRESS_HOLD','exclamation');
        }
    };  
    
    $scope.doExcludeAction = (excludeAction, curItem) => {
        templateManager.container.itemToExclude = curItem;
        let code = excludeAction;
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };

    $scope.selectLongTappedItem = item =>{
        if(item.__isSelected){
            if($scope.widget.beforeUnselectRow)
                $scope.widget.beforeUnselectRow();
            
            $scope.widget.dataSource.removeCheckedRows(item);

            if($scope.widget.afterUnselectRow)
                $scope.widget.afterUnselectRow();
        } else{
            if($scope.widget.beforeSelectRow)
                $scope.widget.beforeSelectRow();

            $scope.widget.dataSource.addCheckedRows(item);
            
            if($scope.widget.afterSelectRow)
                $scope.widget.afterSelectRow();
        }

    };

    $scope.onClickSquareItem = data => {
        if($scope.widget.hasCheckedRows()){
            $scope.selectLongTappedItem(data);
        } else {
            $scope.openDetail($scope.widget.afterClickOnItemWidget, data);
        }
    };

    $scope.trashIconClick = (data, $event)=>{
        $event.stopPropagation();
        if(!$scope.widget.hasCheckedRows()){
            $scope.doExcludeAction($scope.widget.excludeAction, data);
        }
    };
}

// FILE: schoolmeals_components/loginWithQrCode/js/LoginTemplateController.js
"use strict";
function LoginTemplateController($scope, ApplicationContext) {
    $scope.testContent = (e, field) => {
        let fieldValue = e.currentTarget.value;
        field.inputHasContent = (fieldValue && fieldValue.length > 0);
    };
}

// FILE: schoolmeals_components/loginWithQrCode/js/QRScannerController.js
"use strict";
function QRScannerController($scope, ApplicationContext) {

    $scope.UIManager = ApplicationContext.UIManager;
    // TODO: make function scoped locally
    $scope.widget.__showCameraPreview = this.__showCameraPreview = () => {
        document.body.__backgroundCopy = document.body.style.background;
        document.body.style.background = 'transparent';
        // maybe an array isn't the best model, but it's o que tem para janta
        $scope.__widgetVisibilities = $scope.templateManager.container.widgets.map(widget => widget.isVisible); 
        ApplicationContext.UIManager.pushState(() => {
            document.body.style.background = document.body.__backgroundCopy;
            delete document.body.__backgroundCopy;
            for (let i = 0; i < $scope.templateManager.container.widgets.length; i++) {
                const widget = $scope.templateManager.container.widgets[i];
                widget.isVisible = $scope.__widgetVisibilities[i];
            }
            delete $scope.__widgetVisibilities;
            QRScanner.destroy();
            ApplicationContext.UIManager.goBack();
        });
    };

    // TODO: make function scoped locally
    $scope.widget.__hideCameraPreview = this.__hideCameraPreview = () => {
        ApplicationContext.UIManager.goBack();
    };

    // let bindVisibilityAccessFunctions = function bindVisibilityAccessFunctions() {
    //     Object.defineProperty($scope.widget, 'isVisible', {
    //         set: (newValue) => {
    //             $scope.widget._isVisible = this._isVisible = newValue;
    //         },
    //         get: () => $scope.widget._isVisible = this._isVisible,
    //         // enumerable: true,
    //         // configurable: true
    //     });
    // };
    // // constructor(){
    // // $scope.widget..$ = this.$scope = $scope;
    // $scope.widget._isVisible = this._isVisible = $scope.widget._isVisible;
    // bindVisibilityAccessFunctions();
    // }

    $scope.widget.startScan = this.startScan = (autoEnd = true) =>
        new Promise((resolve, reject) => {
            if('QRScanner' in window) {
                $scope.scanRejector = reject;
                QRScanner.show();
                QRScanner.scan((err, text) => {
                    if (err) {
                        reject(err);
                    } else {
                        resolve(text);
                    }
                    if(autoEnd){
                        this.endScan();
                    }
                });
            } else {
                console.warn("The application is not running in a native evironment. The QRScanner will not be triggered. Use the manual typing option.");
            }
        });


    $scope.widget.endScan = this.endScan = () => {
        if('QRScanner' in window) {
            QRScanner.destroy();
        }
    };

    let requestPermission = () =>
        new Promise((resolve, reject) => {
            if(QRScanner) {
                QRScanner.prepare((err, status) => {
                    if (err) {
                        reject(err);
                    } if (status.authorized) {
                        resolve(true);
                    } else if (status.denied) {
                        resolve(false);
                    } else {
                        resolve(false);
                        // needs better support
                        // we didn't get permission, but we didn't get permanently denied. (On
                        // Android, a denial isn't permanent unless the user checks the "Don't
                        // ask again" box.) We can ask again at the next relevant opportunity.
                    }
                });
            } else {
                reject('QRScanner is not avaliable');
            }
        });
    ;
    
    $scope.onManualInputCancel = () => {
        // $scope.manualInputPopupValue = "";
        $scope.manualInputActive = false;
    };

    $scope.onManualInputConfirm = (typedValue, autoEnd = false) => {
        if(autoEnd){
            this.endScan();
        }
        $scope.manualInputActive = false;
        // if($scope.scanRejector){
        //     $scope.scanRejector('manuallyTyped');
        //     delete $scope.scanRejector;
        // }
        if($scope.widget.onManualQRCodeTyped){
            $scope.$eval($scope.widget.onManualQRCodeTyped, 
                Object.assign(
                    {
                        typedValue: typedValue || ""
                    }, 
                    ApplicationContext
                )
            );
        }
    };
}

// FILE: schoolmeals_components/scrollerTabs/js/ScrollSnapController.js
function ScrollSnapController($scope, $element, $timeout){

    let activateTab = (tabIndex, forceDigest = false) => {
        if(tabIndex < $scope.widgets.length){
            let previousActiveWidget = $scope.widgets[$scope.currentTabActiveTabIndex]; 
            previousActiveWidget.isTabActive = false;
            $scope.currentTabActiveTabIndex = tabIndex;
            let activeWidget = $scope.widgets[tabIndex]; 
            activeWidget.isTabActive = true;
            activeWidget.activate();
            headerScroll(tabIndex);
            if(forceDigest){
                $scope.$apply();
            }
        }
    };

    let changeActiveTabByScrollV2 = () => {
        let tabsContainer = $element[0].querySelector('.s-tabbedContent');
        clearTimeout($scope.scrollTimeoutID);
        $scope.scrollTimeoutID = setTimeout(() => {
            let lastScroll = Math.floor(tabsContainer.scrollLeft);
            let newIndex = Math.round((lastScroll/ tabsContainer.scrollWidth) * $scope.widgets.length);
            activateTab(newIndex, false);
        }, 200);
    };

    function headerScroll(indexOfTab) {
        const tabHeader = $element[0].querySelector('.s-tabbedHeader');
        const offsetLeft = tabHeader.children[indexOfTab].offsetLeft;
        tabHeader.scrollLeft = (offsetLeft - window.innerWidth/3);
    }

    $scope.initListener = () => {
        if(!$scope.listenerAttached){
            const tabsContainer = $element[0].querySelector('.s-tabbedContent');
            tabsContainer.addEventListener('touchend', changeActiveTabByScrollV2 ,false);
            $scope.listenerAttached = true;
        }
    };

    $scope.init = () => {
        $scope.currentTabActiveTabIndex = 0;
        if($scope.widgets && $scope.widgets.length > 0){
            $scope.widgets[0].isTabActive = true;
            $scope.widgets[0].activate();
        }
    };

    $scope.clickOntabHeaderEvent = (tabIndex) => {
        const pageSize = window.innerWidth;
        const tabsContainer = $element[0].querySelector('.s-tabbedContent');
        tabsContainer.scroll({
            left: tabIndex * pageSize,
            behavior: "smooth"
        });
        activateTab(tabIndex);
    };
};

// FILE: schoolmeals_components/scrollerTabs/js/ScrollTabbedDataController.js
function ScrollTabbedDataController($scope, $element, $timeout, ApplicationContext){

    /**
     * Test field and set inputHasContent
     * according to value.
     */
    $scope.testContentWithFieldValue = (field) => {
        let currentItem = $scope.widget.dataSource.data[$scope.currentActiveTabIndex];
        field.inputHasContent = (currentItem[field.name]!==undefined && currentItem[field.name]!==null);
    };

    /**
     * Initiate template. Set 'dataSource' var
     * reference. Start touch event listener.
     */
    $scope.initScrollTabbedData = () => {
        $scope.dataSource = $scope.widget.dataSource; 
        $scope.initListener();
    };

    /**
     * Initiate autocomplete template. Set exibition value
     * reference.
     */
    $scope.onSelectAutoCompleteInitialized = fieldScope => {
        let activeIndex = $scope.widget.currentActiveTabIndex || 0;

        fieldScope.descriptionText = $scope.dataSource.data[activeIndex][fieldScope.field.name];
        $scope.testContentWithFieldValue(fieldScope.field);
    };

    /**
     * After select auto complete clear it's value, remove
     * the value from active register and related outData values.
     */
    $scope.onClearAutoCompleteValue = fieldScope => {
        let currentItem = $scope.widget.getActiveTabData();
        currentItem[fieldScope.field.name] = false;
        for (var key in fieldScope.field.outData) {
            delete currentItem[fieldScope.field.outData[key]];
        }
    };

	/**
     * Set 'updateDescriptioText' function into field instace.
	 */
	$scope.initAutoCompleteFunctions = field => {
        field.updateDescriptionText = (newDescription = null) => {
        	let fieldScope = field.autoCompleteScope;
			let activeTabData = $scope.widget.getActiveTabData();
        	
        	if(newDescription === null){
		        fieldScope.descriptionText = activeTabData[field.name];
        	} else {
		        fieldScope.descriptionText = newDescription;
        	}

        };
    };

    /**
     * Add event listener at touchend event to
     * change active tab by scrolling template.
     */
    $scope.initListener = () => {
        if(!$scope.listenerAttached){
            const tabsContainer = $element[0].querySelector('.s-tabbedContent');
            tabsContainer.addEventListener('touchend', changeActiveTabByScroll ,false);
            $scope.listenerAttached = true;
        }
    };

    /**
     * After ng-repeat set active tab according
     * to 'currentActiveTabIndex' property
     * in widget. If property is undefined, set
     * to first register
     */
    $scope.postRepeat = () => {
        let activeIndex = $scope.widget.currentActiveTabIndex || 0;
        $scope.currentActiveTabIndex = activeIndex;
        if($scope.dataSource.data && $scope.dataSource.data.length > 0){
            $scope.dataSource.data.forEach(reg=>reg.isTabActive = false);
            $scope.dataSource.data[activeIndex].isTabActive = true;
        }
    };

    /**
     * After click on tab header, scroll to 
     * correspondent tab container and then
     * set clicked register as active.
     * 
     * @param {Number} tabIndex Index of clicked
     * tab.
     */
    $scope.clickOntabHeaderEvent = (tabIndex) => {
        const pageSize = window.innerWidth;
        const tabsContainer = $element[0].querySelector('.s-tabbedContent');
        tabsContainer.scroll({
            left: tabIndex * pageSize,
            behavior: "smooth"
        });
        activateTab(tabIndex);
    };

    /**
     * Activate tab register according to index
     * passed as parameter, and deactivate previous
     * register.
     * 
     * @param {Number} tabIndex Index of register
     * to activate
     * @param {Boolean} forceDigest Forge angular
     * $apply.
     */
    let activateTab = (tabIndex, forceDigest = false) => {
        if(tabIndex < $scope.dataSource.data.length && tabIndex !== $scope.currentActiveTabIndex){
            if($scope.widget.beforeChangeActiveTab){
                $scope.executeScopeCode($scope.widget.beforeChangeActiveTab);
            }
            
            let previousActiveRegister = $scope.dataSource.data[$scope.currentActiveTabIndex]; 
            previousActiveRegister.isTabActive = false;
            $scope.currentActiveTabIndex = tabIndex;

            let activeRegister = $scope.dataSource.data[tabIndex]; 
            activeRegister.isTabActive = true;

            if($scope.widget.onChangeActiveTab){
                $scope.executeScopeCode($scope.widget.onChangeActiveTab);
            }

            $scope.widget.fields.forEach(field=>{
                $scope.testContentWithFieldValue(field);
            });

            headerScroll(tabIndex);
            if(forceDigest){
                $scope.$apply();
            }
        }
    };

    /**
     * Change active tab after scroll through
     * tab containers.
     */
    let changeActiveTabByScroll = () => {
        let tabsContainer = $element[0].querySelector('.s-tabbedContent');
        clearTimeout($scope.scrollTimeoutID);
        $scope.scrollTimeoutID = setTimeout(() => {
            let lastScroll = Math.floor(tabsContainer.scrollLeft);
            let newIndex = Math.round((lastScroll/ tabsContainer.scrollWidth) * $scope.dataSource.data.length);
            activateTab(newIndex, true);
        }, 200);
    };

    /**
     * Scroll header exibition to index passed as
     * parameter.
     * 
     * @param {Number} indexOfTab Index of register
     * to scroll.
     */
    function headerScroll(indexOfTab) {
        const tabHeader = $element[0].querySelector('.s-tabbedHeader');
        const offsetLeft = tabHeader.children[indexOfTab].offsetLeft;
        tabHeader.scrollLeft = (offsetLeft - window.innerWidth/3);
    }

    /**
     * Toggle value from toggle button in current register.
     * 
     * @param {Object} tabRegister 
     * @param {Object} Field 
     */
    $scope.toggleValue = (tabRegister, field, event) =>{
        tabRegister[field.name] = !tabRegister[field.name];
    };

    /**
     * Check if field has value
     */
    $scope.testContent = (e, field) => {
        let fieldValue = e.currentTarget.value;
        field.inputHasContent = (fieldValue && fieldValue.length > 0 && fieldValue != "?") && field.from != 'toggleButton';
    };

    /**
     * Activate tab register according to index
     * passed as parameter, and deactivate previous
     * register.
     * 
     * @param {Number} tabIndex Index of register
     * to activate
     * @param {Boolean} forceDigest Forge angular
     * $apply.
     */
    $scope.widget.activateTab = (index, forceDigest = true) => {
        const pageSize = window.innerWidth;
        const tabsContainer = $element[0].querySelector('.s-tabbedContent');
        tabsContainer.scroll({
            left: index * pageSize,
            behavior: "smooth"
        });
        activateTab(index, forceDigest)
    };

    /**
     * Get data from active tab.
     */
    $scope.widget.getActiveTabData = () => $scope.dataSource.data[$scope.currentActiveTabIndex];

	/**
	 * Update view of active header
	 */
    $scope.widget.updateHeaderScroll = () => headerScroll($scope.currentActiveTabIndex);

    /**
     * Eval code in App context
     * 
     * @param {String} code
     */
    $scope.executeScopeCode = code => {
        $scope.$eval(
            code,
            Object.assign(ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
};

