var ChangeUnitController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Instructions to be executed when enter
     * in main widget.
     */
    this.onEnterChangeUnit = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.openWindow('dashboard').then(function() {
                ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
            });
        } else {
            const userData = getLocalVar('userData');
            updateUnitLabel(userData);
        }
    };

    /**
     * Confirm unit change.
     * Set data of new unit in user data
     * at localStorage.
     */
    this.confirmChangeUnit = () => {
        const changeUnitWidget = templateManager.container.getWidget('changeUnit');
        let unitCode = changeUnitWidget.currentRow.CDFILIAL;
        let userData = getLocalVar('userData');

        if (unitCode) {
            if (userData.CDFILIAL == unitCode) {
                ScreenService.showMessage(ScreenService.i18n('ALREADY_LOGGED_UNIT'));
            } else {
                SchoolMealsUtil.changeUnit(unitCode).then(() => {
                    ScreenService.showMessage(ScreenService.i18n('SUCCESSFUL_OPERATION')).then(() => {
                        ScreenService.openWindow('dashboard');
                    });
                });
            }
        } else {
            ScreenService.showMessage(ScreenService.i18n('FILL_REQUIRED_FIELDS'));
        }
    };

    /**
     * Update logged unit label text.
     * 
     * @param {Object} unitData Object containing
     * unit name and code.
     */
    const updateUnitLabel = unitData => {
        const changeUnitWidget = templateManager.container.getWidget('changeUnit');
        changeUnitWidget.getField('loggedUnit').label = unitData.CDFILIAL + ' - ' + unitData.NMFILIAL;
        templateManager.updateTemplate();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ChangeUnitController', ChangeUnitController);
});
