var DataExportController = function(ScreenService, SchoolMealsUtil, templateManager, DataExportService, LoginService, DataImportService) {

    /**
     * @var service Reference to DataExportService.
     */
    const service = DataExportService;

    /**
     * Initialize exportation process if have 
     * imported data.
     */
    this.initDataExportProcess = () => {
        if (getLocalVar('hasImportedData')) {
            ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_EXPORT')).then(() => {
                exportData();
            });
        } else {
            ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
        }
    };

    /**
     * Export all changed data after validate if user
     * is active.
     */
    const exportData = () => {
        let promises = [
            validateActiveUser,
            updateInventory,
            updateRequestList,
            updateDeletedProducts,
            update_MOVOPSOL,
            updateEditedProducts,
            exportAllDataSources,
            afterExportDataSource
        ];

        SchoolMealsUtil.executePromisesSequentialy(promises);
    };

    /**
     * Check if current user is active on system.
     * If it's not, so should logout user.
     */
    const validateActiveUser = () => service.validateActiveUser().catch(() => {
        ScreenService.showMessage(ScreenService.i18n('INACTIVE_USER')).then(() => {
            LoginService.logout();
        });
    });

    /**
     * Update inventory data based on user changes.
     * Bind inventory's products to inventory register.
     * 
     * @return {Promise} 
     */
    const updateInventory = () => service.bindProductListToInventory();

    /**
     * Export all datasources from local to remote 
     * storage by doing a request and save data.
     * 
     * @return {Promise} 
     */
    const exportAllDataSources = () => service.exportAllDataSources();

    /**
     * Update request list data based on user changes.
     * Set extra requests registers.
     * 
     * @return {Promise} 
     */
    const updateRequestList = () => service.updateRequestList();

    /**
     * Synchronize deleted products.
     * 
     * @return {Promise} 
     */
    const updateDeletedProducts = () => service.removeDeletedProductsFromInexistentRequest();

    /**
     * update MOVOPSOL table log.
     * 
     * @return {Promise} 
     */
    const update_MOVOPSOL = () => service.removeLogRegisterFromInexistentRequest();

    /**
     * Update edited products.
     * 
     * @return {Promise} 
     */
    const updateEditedProducts = () => service.removeEditedProductsFromInexistentRequest();

    /**
     * After export dataSources, import again secondary
     * datasources using same paramters used in importation
     * routine.
     * 
     * @param {Array} exportResultDataset 
     * 
     * @returns {Promise}
     */
    const afterExportDataSource = exportResultDataset => service.afterExportDataSource(exportResultDataset, DataImportService);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('DataExportController', DataExportController);
});
