var LoginController = function(LoginService, ScreenService, SchoolMealsUtil, UIManager, templateManager) {
    var self = this;

    function checkLoginInfo(email, pass) {
        return (SchoolMealsUtil.emailValidation(email) && SchoolMealsUtil.passValidation(pass));
    }

    function setUserDataOnApplication(result) {
        let user = result.dataset.User;
        setLocalVar('isLogged', true);
        setLocalVar('userData', user);
        setLocalVar('CDFILIAL', user.CDFILIAL);
        SchoolMealsUtil.setIDBVar('Units', result.dataset.Units, true);

        LoginService.getProductsGroups().then(productsGroups => {
            SchoolMealsUtil.setIDBVar('productsGroups', productsGroups.dataset.GruposProduto, true);
        });

        LoginService.getProductMask().then(productMask => {
            setLocalVar('productMask', productMask);
        });
    }

    this.onEnter = function(widget) {
        this.setVisibilityForValidQRCode(widget);
        this.setUpDNS();
        let lastLogin = '';
        lastLogin = getLocalVar("userData") && getLocalVar("userData") != null ? getLocalVar("userData").DSEMAILOPER : lastLogin;
        setLocalVar('lastLogin', lastLogin);
        widget.currentRow.email = lastLogin;

        if (getLocalVar('isLogged') === true && getLocalVar("userData")) {
            ScreenService.openWindow('dashboard');
        } else {
            setLocalVar('isLogged', false);
            setLocalVar('userData', '');
        }
    };
    this.onAccountEnter = function(widget) {
        widget.newRow();
        widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
    };

    this.login = function(widget) {
        let email = widget.currentRow.email || '';
        let password = widget.currentRow.password || '';

        if (!checkLoginInfo(email, password)) {
            ScreenService.showMessage('Preencha os campos com email e senha!');
        } else {
            LoginService.login(email, password).then(function(result) {
                setUserDataOnApplication(result);
                ScreenService.openWindow('dashboard');
            }).catch(function(callback) {
                setLocalVar('isLogged', false);
                setLocalVar('userData', '');
                const errorMessage = (callback.data && callback.data.error) || callback.error || "Um erro inesperado aconteceu";
                ScreenService.showMessage(ScreenService.i18n(errorMessage), "error");
            });
        }
    };

    this.logout = function(skipQuestion = false) {
        if (!skipQuestion) {
            ScreenService.confirmMessage(ScreenService.i18n('WANT_TO_EXIT')).then(() => {
                LoginService.logout();
            });
        } else {
            LoginService.logout();
        }
    };

    this.changeUnit = function(widget) {
        let newFilial = widget.currentRow.CDFILIAL;
        ScreenService.confirmMessage(ScreenService.i18n('Tem certeza que deseja trocar para filial ') + newFilial + "?", 'question',
            function() {
                let userData = getLocalVar('userData');
                userData.newFilial = newFilial;
                LoginService.changeUnit(userData).then(function(result) {
                    setUserDataOnApplication(result);
                }).catch(function(callback) {
                    self.logout();
                    ScreenService.showMessage(ScreenService.i18n(callback.data.error), "error");
                });
            },
            function() {
                widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
            });
    };

    this.saveQRCodeData = (qrCodeData) => {
        for (const qrCodePropertyKey in qrCodeData) {
            const qrCodeProperty = qrCodeData[qrCodePropertyKey];
            localStorage.setItem(qrCodePropertyKey, qrCodeProperty);
        }
    };

    this.validateQRCode = (qrCodeValue) => {
        let splitedData = qrCodeValue.split(';');
        if (!qrCodeValue || qrCodeValue.length === 0) {
            return Promise.reject("Invalid QRCode");
        }
        let qrCodeData = {
            serviceURL: splitedData[0],
            NRORG: splitedData[1]
        };

        let concantInURL = window.hasServiceInDNS ? "/backend/service/index.php" : "/backend/index.php";
        qrCodeData.serviceURL += concantInURL;
        templateManager.updateURL(qrCodeData.serviceURL);

        return new Promise((resolve, reject) => {
            LoginService.checkServerRepository(qrCodeData.NRORG).then(() => {
                qrCodeData.isQRCodeValid = true;
                resolve(qrCodeData);
            }).catch(() => {
                reject();
            });
        });
    };

    this.showQRCodeScanner = () => {
        let qrCodeWidget = templateManager.container.getWidget('qrCodeScanner');
        UIManager.goToWidgetByName('qrCodeScanner');
        this.readQRCode(qrCodeWidget);
        if ('QRScanner' in window) {
            qrCodeWidget.__showCameraPreview();
        }
    };

    this.readQRCode = function(qrCodeWidget) {
        qrCodeWidget.startScan(false).then((qrCodeValue) => {
            this.validateQRCode(qrCodeValue).then((qrCodeData) => {
                this.saveQRCodeData(qrCodeData);
                this.setVisibilityForValidQRCode();
                this.setUpDNS();
                UIManager.goBack();
            }).catch(() => {
                ScreenService.showMessage("QRCode Inválido");
                this.readQRCode(qrCodeWidget);
            });
        });
    };

    this.setVisibilityForValidQRCode = (widget) => {
        widget = widget || templateManager.container.getWidget('login');
        let validQRCode = localStorage.getItem('isQRCodeValid');
        if (validQRCode || widget.shouldIgnoreQrcode) {
            widget.fields.forEach(field => field.isVisible = true);
            let loginAction = widget.actions.find(action => action.name === 'login');
            loginAction.isVisible = true;
        }
    };

    this.setUpDNS = () => {
        let newURL = localStorage.getItem('serviceURL');
        if (newURL) {
            templateManager.updateURL(newURL);
        }
    };

    this.onManualQRCodeTyped = (typedValue) => {
        this.validateQRCode(typedValue).then((qrCodeData) => {
            this.saveQRCodeData(qrCodeData);
            this.setVisibilityForValidQRCode();
            this.setUpDNS();
            UIManager.goBack();
        }).catch((err) => {
            ScreenService.showMessage("QRCode Inválido");
        });
    };

    this.log = (string) => console.log(string);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('LoginController', LoginController);
});
