var SupplynigAuthorizationController = function(ScreenService, SchoolMealsUtil, UIManager, templateManager) {

    /**
     * Instructions to be executed when
     * enter widget.
     */
    this.onEnterSupplyingAuth = () => {
        if (!getLocalVar('hasImportedData')) {
            ScreenService.openWindow('dashboard').then(() => {
                ScreenService.showMessage(ScreenService.i18n('HAVE_TO_IMPORT_DATA'));
            });
        } else {
            setDefaultDataSourceFilter();
        }
    };

    /**
     * Sort widget data by code (NRAUTOFORN).
     */
    this.sortDataByCode = () => {
        const supplyingWidget = templateManager.container.getWidget('supplyingAuthorization');

        supplyingWidget.dataSource.orderingAscNumeric = !supplyingWidget.dataSource.orderingAscNumeric;
        supplyingWidget.container.fabActionIcon = !supplyingWidget.dataSource.orderingAscNumeric ? "fa-sort-numeric-up" : "fa-sort-numeric-down";

        supplyingWidget.dataSource.data.sort((a, b) => {
            let codeA = Number(a.NRAUTOFORN);
            let codeB = Number(b.NRAUTOFORN);
            let order = 0;

            if (supplyingWidget.dataSource.orderingAscNumeric) {
                order = codeA - codeB;
            } else {
                order = codeB - codeA;
            }

            return order;
        });
    };

    /**
     * Sort widget data by product name (NMPRODUTO).
     */
    this.sortDataByProductName = () => {
        const supplyingWidget = templateManager.container.getWidget('supplyingAuthorization');

        supplyingWidget.dataSource.orderingAscAlpha = !supplyingWidget.dataSource.orderingAscAlpha;
        supplyingWidget.container.fabActionIcon = !supplyingWidget.dataSource.orderingAscAlpha ? "fa-sort-alpha-up" : "fa-sort-alpha-down";

        supplyingWidget.dataSource.data.sort((a, b) => {
            let nameA = a.NMPRODUTO.toLowerCase();
            let nameB = b.NMPRODUTO.toLowerCase();
            let order = 0;

            order = (nameA < nameB) ? -1 : 1;

            if (!supplyingWidget.dataSource.orderingAscAlpha) {
                order *= -1;
            }

            return order;
        });
    };

    /**
     * Show suppyling authorization data details.
     * 
     * @param {Object} data Clicked data.
     */
    this.showSupplyingAuthDetails = data => {
        const analyticsView = templateManager.container.getWidget('analyticsView');

        analyticsView.currentRow = Object.assign(data);
        analyticsView.getField('supplyingAuthCode').text = ScreenService.i18n('SUP_AUTH_LABEL') + ': ' + data.NRAUTOFORN;

        UIManager.goToWidget(analyticsView);
    };

    /**
     * Set default dataSourceFilter on main
     * screen. Filter data according to current
     * unit.
     */
    const setDefaultDataSourceFilter = () => {
        let supplyingAuthList = templateManager.container.getWidget('supplyingAuthorization');
        supplyingAuthList.dataSourceFilter = [{
            name: "CDFILDEST",
            operator: "=",
            value: getLocalVar('userData').CDFILIAL
        }];
        supplyingAuthList.reload();
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('SupplynigAuthorizationController', SupplynigAuthorizationController);
});
