var SchoolMealsUtil = function(localStorageEngine, restEngine, templateManager) {

    /**
     * Get factored widgets in container.
     */
    Container.prototype.getFactoredWidget = function(widgetName) {
        return this.factoredWidgets.find(widget => widget.name == widgetName);
    };

    /**
     * Prevent long click to open 
     */
    window.oncontextmenu = function(event) {
        event.preventDefault();
        event.stopPropagation();
        return false;
    };

    /**
     * Remove array element if testFunction results
     * become true.
     * 
     * @param {Array} array Array containing elements
     * @param {Function} testFunction Function to execute.
     */
    spliceOnCondition = function(array, testFunction) {
        var i = array.length;
        while (i--) {
            if (testFunction(array[i], i)) {
                array.splice(i, 1);
            }
        }
    };

    /**
     * Reads cookie from browser document memory.
     * 
     * @param {String} name Cookie name.
     * 
     * @return {String|Null}
     */
    this.readCookie = function(name) {
        var nameEQ = name + "=";
        var ca = document.cookie.split(';');
        for (var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') c = c.substring(1, c.length);
            if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length, c.length);
        }
        return null;
    };

    /**
     * Remove cookie from browser document memory.
     * 
     * @param {String} name Cookie name.
     */
    this.removeCookie = function(name) {
        document.cookie = name + "=;  path=/;";
        var d = new Date();
        d.setTime(d.getTime() + (0 * 60 * 1000));
        var expires = "expires=" + d.toUTCString();
        document.cookie = name + "=" + '' + ";" + expires + ";path=/";
    };

    /**
     * Check if a string is a valid e-mail address.
     * 
     * @param {String} email Email string.
     * 
     * @returns {Boolean}
     */
    this.emailValidation = function(email) {
        var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        return re.test(String(email).toLowerCase());
    };

    /**
     * 
     * @param {} pass
     * 
     * @returns {Boolean}
     */
    this.passValidation = function(pass) {
        return pass.length > 0;
    };

    /**
     * Get back to te previous screen at system
     * history.
     * 
     * @param {Widget} widget
     */
    this.backToPreviousScreen = function(widget) {
        let steps = widget.container.steps;
        if (steps && steps.length > 0) {
            let cont = 0;
            let cond = false;
            while (cont <= steps.length && !cond) {
                let step = steps[cont];
                if (step.active) {
                    cond = true;
                } else {
                    cont = cont + 1;
                }
            }
            if (cont > 0) {
                steps[cont].active = false;
                steps[cont - 1].active = true;
                widget.container.getWidget(steps[cont].name).isVisible = false;
                widget.container.getWidget(steps[cont - 1].name).activate();
                widget.container.getWidget(steps[cont - 1].name).isVisible = true;

                let footerPositioner = document.getElementsByClassName("footer-positioner")[0];
                let stepElements = document.getElementsByClassName("footer-wizard-step");
                let curStep = stepElements[cont - 1];
                let curStepWidth = parseInt(window.getComputedStyle(curStep, null).getPropertyValue("width"));
                footerPositioner.scrollLeft = curStep.offsetLeft - (curStepWidth / 3);
            }
        }
    };

    /**
     * Get max level from product code mask
     * of the product's tree.
     * 
     * @returns {Number} product code level.
     */
    this.getNivelMaxProduto = () => {
        var mascaraProduto = localStorage.getItem('CDPICTPROD');
        if (mascaraProduto) {
            return mascaraProduto.split('.').length;
        }
    };

    /**
     * Get max level from plates code mask
     * of the plate's tree.
     * 
     * @returns {Number} plates code level.
     */
    this.getNivelMaxPrato = () => {
        var mascaraProduto = localStorage.getItem('CDPICTPRAT');
        if (mascaraProduto) {
            return mascaraProduto.split('.').length;
        }
    };

    /**
     * Set value on dataSource of a widget according
     * to parameters.
     * 
     * @param {Widget} widget
     * @param {String} name column name to filter.
     * @param {*} value Value to use as filter.
     */
    this.setValueOnFilter = (widget, name, value) => {
        let simF = widget.dataSourceFilter.find(dsf => dsf.name === name);
        if (simF) {
            simF.value = value;
        } else {
            widget.dataSourceFilter.push({
                "name": name,
                "operator": "=",
                "value": value
            });
        }
    };

    /**
     * Returns Indexed DB var according to name.
     * Parse it to object if raw is falsy.
     * 
     * @param {String} varName Name of indexed DB key.
     * @param {Boolean} raw
     * 
     * @returns {Promise}
     */
    this.getIDBVar = (varName, raw = true) =>
        new Promise((resolve) => {
            localforage.getItem(varName, () => {}).then(item => {
                if (!raw) {
                    item = JSON.parse(item);
                }
                resolve(item);
            });
        });

    /**
     * Set Indexed DB var according to name.
     * Parse it to string if raw is truly.
     * 
     * @param {String} varName Name of indexed DB key.
     * @param {Boolean} raw
     * 
     * @returns {Promise}
     */
    this.setIDBVar = (varName, content, raw = true) => {
        if (!raw) {
            content = JSON.stringify(content);
        }
        return localforage.setItem(varName, content);
    };

    /**
     * Overwrite localStorageEngine __prepareData function to
     * verify if storaged data is string. Original function parse 
     * it regardless of type.
     * Performance of array data storagedData is better than string
     * data.
     */
    localStorageEngine._prepareData = function(storagedData) {
        let data = storagedData;

        if (typeof storagedData === "string") {
            data = JSON.parse(storagedData);
        }

        if (Util.isArray(data)) {
            data.forEach(item => {
                if (item) {
                    item.__is_new = !!item.__is_new;
                    return item;
                }
            });
        }

        return data;
    };

    /**
     * Clear all data from current user at indexed DB.
     */
    this.clearUserIDBData = () => {
        this.setIDBVar('Units', null, true);
        this.setIDBVar('productsGroups', null, true);
    };

    /**
     * Build filter array using object as reference.
     * 
     * @param {Object} filterObj Object containing data to
     * filter. Where key is column name and value is column
     * value.
     * 
     * @return {Array} Filter on Zeedhi Format
     */
    const buildFilterArray = filterObj => {
        let filterArray = [];

        Object.keys(filterObj).forEach(columnName => {
            filterArray.push({
                name: columnName,
                operator: '=',
                value: filterObj[columnName]
            });
        });

        return filterArray;
    };

    /**
     * Make a filter data request to route passed as argument.
     * 
     * @param {String} route
     * @param {Object} params Filter Data
     */
    this.doFilterDataRequest = (route, params) => restEngine.request(route, {
        "requestType": "FilterData",
        "filter": buildFilterArray(params)
    }, null, null, null, null, 300000).then(function(result) {
        return result.dataset;
    });

    /**
     * Change current logged working unit
     * to selected parameter unit.
     * 
     * @param {String} unitCode Unit code to
     * change.
     * 
     * @return {Promise}
     */
    this.changeUnit = unitCode => {
        let userData = getLocalVar('userData');

        return this.getIDBVar('Units', true).then(units => {
            let unitData = units.find(unit => unit.CDFILIAL == unitCode);

            if (unitData) {
                userData = Object.assign(userData, unitData);
            }

            setLocalVar('userData', userData);
            setLocalVar('CDFILIAL', unitCode);

            return userData;
        });
    };

    /**
     * Insert register into IDB var array.
     * If key not exists, return a warning to developer.
     * If key is not an array, return a warning to developer.
     * 
     * @param {String} dataSourceName Key name to insert.
     * 
     * @returns {Promise}
     */
    this.insertIntoDataSource = (dataSourceName, value) => {
        return this.getIDBVar(dataSourceName).then(dataSet => {
            let invalidDataset = isInvalidIDBDataSource(dataSet);

            if (!invalidDataset) {

                dataSet.push(value);
                return this.setIDBVar(dataSourceName, dataSet);

            } else
                return Promise.reject(invalidDataset);
        });
    };

    /**
     * Check if object is a valid array to manage
     * register at IDB.
     * 
     * Show a console warning to feedback developer
     * what's wrong.
     * 
     * @param {*} objToTest Object to check
     * 
     * @returns {Boolean} Is valid.
     */
    const isInvalidIDBDataSource = objToTest => {
        if (objToTest) {
            if (Array.isArray(objToTest)) {
                return false;
            } else {
                console.warn('IDB key does not reference an array.');
                return 'IDB_KEY_IS_NOT_ARRAY';
            }
        } else {
            console.warn('IDB key does not exists.');
            return 'IDB_KEY_NOT_EXISTS';
        }
    };

    /**
     * Remove itens from parameter dataSource. Target to be
     * removed can be a value or a test function. If it's a 
     * function, all registers that pass the test gonna be
     * removed.
     * 
     * @param {String} dataSourceName Name of working datasource.
     * @param {*|Function} target Exclusion target value, or test
     * function to remove more than a value.
     * 
     * @returns {Promise}
     */
    this.removeFromDataSource = (dataSourceName, target) => {
        let targetFunction = () => {};

        if (typeof target != 'function') {
            let _targetValue = Object.assign(target);
            targetFunction = dataValue => JSON.stringify(dataValue) == JSON.stringify(_targetValue);
        } else {
            targetFunction = target;
        }

        return this.getIDBVar(dataSourceName).then(dataSet => {
            let invalidDataset = isInvalidIDBDataSource(dataSet);

            if (!invalidDataset) {

                spliceOnCondition(dataSet, targetFunction);
                return this.setIDBVar(dataSourceName, dataSet).then(() => dataSet.filter(targetFunction).length);

            } else
                return Promise.reject(invalidDataset);
        });
    };

    /**
     * Check if user has a logged unit for the
     * sytem.
     * 
     * @return {Boolean}
     */
    this.hasLoggedUnit = () => {
        return getLocalVar('CDFILIAL');
    };

    /**
     * Check if user has imported data to system.
     * 
     * @return {Boolean}
     */
    this.hasImportedData = () => {
        return getLocalVar('hasImportedData');
    };

    /**
     * Execute array of promises sequentially, and
     * then return thenable from last promise;
     * 
     * Array.reduce method is used here to execute
     * all promise in sequence, without implement 
     * inherited .then's
     * 
     * @param {Array<Function>} promiseArray Array of
     * promises funcions.
     * 
     * @returns {Promise}
     */
    this.executePromisesSequentialy = promiseArray => promiseArray.reduce(
        (executedPromise, toExecutePromise) => executedPromise.then(promiseParam => toExecutePromise(promiseParam)),
        Promise.resolve()
    );

    /**
     * Show current logged unit into sub header
     * text span.
     */
    this.showUnitOnSubLabel = () => {
        let userData = getLocalVar('userData');
        templateManager.container.setSubHeaderText(`Unidade: ${userData.CDFILIAL} - ${userData.NMFILIAL}`);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('SchoolMealsUtil', SchoolMealsUtil);
});
