"use strict";

function HorizontalListsDashBoardController($scope, ApplicationContext, $element, $timeout) {
    const UIManager = ApplicationContext.UIManager;

    $scope.getWelcomeMessage = (widget) => {
        let welcomeMessage = widget.welcomeMessage;
        
        if(widget.currentRow){
            Object.keys(widget.currentRow).forEach(columnName=>{
                welcomeMessage = welcomeMessage.replace('__'+columnName+'__', widget.currentRow[columnName]);
            });
        }

        return welcomeMessage;
    };

    $scope.getUserName = () => {
        return getLocalVar('userData').NMOPERADOR + ',';
    };

    function getSourceLength(source) {
        return source ? source.length : 0;
    }

    $scope.executeControllerAction = (code) => {
        console.error('depreciated, '+code+ 'will not be executed');
    };

    $scope.formatData = (data) => {
        moment.locale('pt-br');
        return moment(data, 'DD/MM/YYYY hh:mi:ss').format('ddd, DD [de] MMM [às] HH:mm');
    };

    $scope.openWindow = (hor) => {
        UIManager.openContainer('listedProposalData', hor);
    };

    $scope.getTotal = (dataSource) => {
        return dataSource.length;
    };

    let render = function(filteredData, widget){
        widget.dataSource.data = filteredData;
        $scope.$apply();
    };
    
    let searchAdapter;
    $scope.initToList = (widget) => {
        
        searchAdapter = new SearchAdapter($element[0].querySelector('.search-input'), widget, 
        (filteredData) => render(filteredData, widget), 
        {
            local: true,
            dataSourceOrigin: widget.dataSource,
            dataSourcePropName: "__copyData",
            searchFields: widget.searchFields
        });
        
        widget.showSearch = () => {
            $scope.isSearching = true;
            widget.container.fabVisibility = false;
            widget.dataSource.__copyData = angular.copy(widget.dataSource.data);

            UIManager.pushState(() => {
                $scope.isSearching = false;
                widget.dataSource.data = angular.copy(widget.dataSource.__copyData);

                let inputElement = $element[0].querySelector('.search-input');
                inputElement.value = "";
                inputElement.blur();

                widget.container.fabVisibility = true;
                if('cordova' in window){
                    $scope.$apply();
                }
            });

            let input = $element[0].querySelector('.search-input');
            setTimeout(() => input.focus(), 10);
        }
    };

    $scope.goBack = () => UIManager.goBack();


    $scope.clickCard = cardData => {
        if(cardData.onClick){
            $scope.$eval(cardData.onClick, Object.assign({
                data: cardData
            }, ApplicationContext));
        } else if(cardData.windowName){
            UIManager.openContainer(cardData.windowName);
        }
    };

    $scope.onClickData = cardData => {
        if($scope.widget.onClickData){
            $scope.$eval($scope.widget.onClickData, Object.assign({
                data: cardData
            }, ApplicationContext));
        }
    };

    /**
     * Set horizontal dashboard events and properties.
     */
    (()=>{
        const horizontalDsbComponent = document.querySelector('.horizontal-lists-dash-board-component');

        if(horizontalDsbComponent){
            /**
             * Cancel previous timeout started when user touchended
             * screen.
             */
            horizontalDsbComponent.addEventListener("touchstart", ()=>{
                $timeout.cancel($scope.arrowTimeOutID);
            });

            /**
             * Check if user scroll to the end of the page. If in the end, 
             * so stop shake arrow, and rotate it horizontally. Else, apply normal
             * state to arrow;
             */
            horizontalDsbComponent.addEventListener("touchend", ()=>{
                $scope.arrowTimeOutID = $timeout(()=>{
                    $scope.endScroll = (horizontalDsbComponent.offsetHeight + horizontalDsbComponent.scrollTop) >= (horizontalDsbComponent.scrollHeight - 30);
                }, 500);
            });

            /**
             * Init shake arrow properties.
             */
            $scope.initArrowProperties = ()=>{
                $scope.endScroll = false;
            };  



            /**
             * Init main horizontal dashboard list.
             * 
             * @param {Object} horizontalDashboard
             */
            $scope.initMainHorizontalDashboard = horizontalDashboard =>{
                horizontalDashboard.dataSource[0].isActive = true;

                setHorizontalDashboardCardScrollEvents(horizontalDashboard);
            };

            /**
             * Cancel previous timeout started when user touchended
             * screen.
             * 
             * Check if user scroll between horizontal main cards, to 
             * change active status.
             * 
             * @param {Object} horizontalDashboard horizontal main dashboard
             * object.
             */
            const setHorizontalDashboardCardScrollEvents = horizontalDashboard => {
                const horizontalDsbMainCardScroll = document.querySelector('.horizontal-main-dashboard .itens-container');

                horizontalDsbMainCardScroll.addEventListener("touchstart", ()=>{
                    $timeout.cancel($scope.scrollMarkerTimeOutID);
                });
                
                horizontalDsbMainCardScroll.addEventListener("touchend", ()=>{
                    $scope.scrollMarkerTimeOutID = $timeout(()=>{
                        let currentCardIndex = horizontalDsbMainCardScroll.scrollLeft / horizontalDsbMainCardScroll.offsetWidth;
                        
                        horizontalDashboard.dataSource.forEach((card, index)=>{
                            card.isActive = Math.round(currentCardIndex) == index;
                        });
                    }, 600);
                });
            };

            $scope.toggleSecondaryDashboardScroll = () => {
                if($scope.endScroll){
                    horizontalDsbComponent.scrollTo(0,0);
                } else {
                    document.querySelector('.horizontal-secondary-dashboard').scrollIntoViewIfNeeded()
                }
            };
        }

    })();
}