/**
 * 
 * @param {HTMLInputElement} input 
 * @param {any} restLoadingZhObject a object that contains a dataSource and dataSourceFilter
 */
function SearchAdapter(inputElement, restLoadingZhObject, render = () => {}, options = {}) {
    
    let activeRequests = [];
    let cacheBase = {};


    //TODO
    let cancelRequest = () => {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
    };

    this.options = options;

    let restKeyStrokeHandler = (event) => {
        if (activeRequests.length) {
            if (activeRequests[0].reqText === inputElement.value) {
                return;
            }
            else {
                cancelRequest();
            }
        }
        if (cacheBase[inputElement.value]) {
            render(cacheBase[inputElement.value]);
            return;
        }
        update();
    
    }

    let localKeyStrokeHandler = (event) => {
        let searchValue = inputElement.value || "";
        searchValue = searchValue.toLowerCase();
        let searchArray = angular.copy(this.options.dataSourceOrigin[this.options.dataSourcePropName]);
        const searchFields = this.options.searchFields;
        let filteredArray = searchArray.filter(dataSourceElement => {
            for(var i = searchFields.length; i--;){
                let currentString = dataSourceElement[searchFields[i]];
                if (currentString && currentString.toLowerCase().indexOf(searchValue) !== -1){
                    return true; 
                }
            }
        });
        
        render(filteredArray);
    }

    let keyStrokeStrategies = {
        "local": localKeyStrokeHandler,
        "rest": restKeyStrokeHandler
    }
    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    let keyStrokeHandler = (event) => {
        if (event.keyCode > 45 || event.keyCode === 8) {
            keyStrokeStrategies[this.options.keyStrokeStrategyName](event);
        }
    };

    // TODO
    /**
     * @returns {Promise<null>} 
     */
    let update = () => {
        return fetchData().then(refreshLastUpdateTime);
    };

    // TODO
    async function fetchData(filter) {
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        ScreenService.hideLoader();
        var newFilter = restLoadingZhObject.dataSourceFilter.slice(0); // make a copy of the zhDataSourceFilter to keep it pristine
        newFilter.push({
            name: options.searchName,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        }); // append our custom conditions into the new array
        var promisseToReturn = await restLoadingZhObject.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet) {
                cacheBase[deferredToCancelRequest.reqText] = resultSet.dataset[Object.keys(resultSet.dataset)[0]]; // do cache
            } else {
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        // ScreenService.hideLoader();
        return promisseToReturn;
    }


    this.bindListeners = () => {
        inputElement.addEventListener('keyup', keyStrokeHandler);
    };
    
    this.removeListeners = () => {
        inputElement.removeEventListener('keyup', keyStrokeHandler);
    }

    this.init = () => {
        this.bindListeners();
        if(this.options.local){ // if the local option is enabled the keyStrokeStrategyName will be overriden 
            this.options.keyStrokeStrategyName = 'local';
        } else {
            this.options.keyStrokeStrategyName = this.options.keyStrokeStrategyName || "rest";
        }
        
    };

    this.init();
}