'use strict';
function SelectAutocompleteController(
    $scope,
    $element,
    ScreenService,
    ZhUtil,
    maskEngine,
    restEngine,
    $q,
    ZhLogger,
    ApplicationContext
    ){

    /**
     * @type {Object}
     */
    var events = {};
    // document.querySelector('.zh-notification-messages').style.display = 'none';
    var activeRequests = [];
    /**
     * @type {Object}
     * a object that will precache the results of each query
     */
    var cacheBase = {};
    /**
     * the spinner element that indicates progress   
     * @type {HTMLElement}
     */
    var spinner = null;
    /**
     * a reference to the current highlited element of the list   
     * @type {HTMLElement}
     */
    var currentHighletedListElement = null;
    /**
     * @type {Object[]} 
     */
    var filteredAditionalOptions = $scope.field.aditionalOptions || [];
    /**
     * @type {Object[]} 
     */
    var filteredListContents = [];
    /** it will be initialized on the initializeData method
     * @type {listContents}
     */
    var listContents = null;
    /**
     * @type {HTMLElement}
     */
    var rootElement;
    /**
     * @type {HTMLElement}
     */
    var popUp;
    /**
     * @type  {HTMLElement}
     */
    var listElement;
    /**
     * @type {HTMLElement}
     */
    var inputElement;
    /**
     * @type {HTMLElement}
     */
    var zhLoader;
    /**
     * @type {HTMLElement}
     */
    var autoCompletePreviewElement;
    /**
     * @type {HTMLElement}
     */
    var labelElement;
    // /**
    //  * @type {HTMLElement}
    //  */
    // var headerLabelElement;
    /**
     * @type {HTMLElement}
     */
    var clearButtonElement;
    /**
     * @type {Object[]}
     */
    var fieldInfo = [];

    /**
     * an element that will be used as a basis to render each list element 
     * @type {HTMLElement}
     */
    var baseListRowToClone;

    var filterConditions = {
        name: "",
        value: "%%",
        operator: "LIKE_ALL"
    };

    var shouldPreventKeyUp = false;
    var lastUpdateTime;
    var updateRequestReference = null;
    var deferredToCancelRequest = false;

    var cacheFirstResultSet = function() {
        cacheBase[""] = listContents;
    };

    /**
     * schedule new http requests and filter the current elements
     * @param {KeyboardEvent} [event] 
     */
    var keyStrokeHandler = function(event) {
        if (event.keyCode > 45 || event.keyCode === 8) {
            //console.log('%ckeyCode: ' + event.keyCode + "   key: '" + event.key + "'", 'color: orange');
            // var begining = performance.now(); // LOG TIME
            if (activeRequests.length) {
                if (activeRequests[0].reqText === inputElement.value) {
                    return;
                }
                else {
                    cancelRequest();
                }
            }
            if (cacheBase[inputElement.value]) {
                filteredListContents = cacheBase[inputElement.value];
                renderListElements(cacheBase[inputElement.value]);
                // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
                return;
            }
            filteredAditionalOptions = filterElements(inputElement.value, $scope.field.aditionalOptions || [], true);
            var filtered = filterElements(inputElement.value, null, true);
            renderListElements(filtered);
            // //console.log('rendering : ' + (performance.now() - begining)); // LOG TIME
            // showAutoCompleteText();
            // var timeSinceUpdate = performance.now() - lastUpdateTime;
            // if (timeSinceUpdate >= 200) {
            //     update();
            // }
            // else {
            //     updateRequestReference = setTimeout(update, 200);
            // }
            update();
        }
        shouldPreventKeyUp = true;
    };

    /**
     * handles the command keys keypress, or propagates the event chain for the keyStrokeHandler to resolve
     * @param {KeyboardEvent} [event] 
     */
    var commandKeyStrokeHandler = function(event) {
        var code = event.keyCode;
        if (code > 45 || code === 8) {
            // do nothing and let the keyStrokeHandler handle the keyup event 
            shouldPreventKeyUp = false;
            return;
        }
        else if (code === 40 /* down arrow */ ) {
            if (currentHighletedListElement.parentElement.lastChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.nextElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 38 /* up arrow */ ) {
            if (currentHighletedListElement.parentElement.firstChild !== currentHighletedListElement) {
                currentHighletedListElement.classList.remove('active');
                currentHighletedListElement = currentHighletedListElement.previousElementSibling;
                currentHighletedListElement.classList.add('active');
            }
        }
        else if (code === 13) /* enter */ {
            confirm(null, currentHighletedListElement);
        }
        shouldPreventKeyUp = true;
    };

    /**
     * shows the first element of the filteredListContents array
     */
    var showAutoCompleteText = function() {
        autoCompletePreviewElement.value = getFormatedString(filteredListContents[0], 0);
    };

    var refreshLastUpdateTime = function() {
        lastUpdateTime = performance.now();
    };
    var hideSpinner = function() {
        spinner.classList.add('hidden');
    };
    var onRequestComplete = function() {
        hideSpinner();
        refreshLastUpdateTime();
    };

    /**
     * @returns {Promise<null>} 
     */
    var update = function() {
        if (updateRequestReference !== null) {
            clearTimeout(updateRequestReference);
            updateRequestReference = null;
        }
        filterConditions.value = "%" + inputElement.value + "%";
        return fetchData().then(refreshLastUpdateTime);
    };

    var cancelRequest = function() {
        var numberOfCanceledRequests = activeRequests.length;
        ZhLogger.setLevel(999);
        ZhLogger.previousLevel = ZhLogger.level;
        while (activeRequests.length) {
            activeRequests.pop().resolve();
        }
    };

    var initiliazefieldInfo = function() {
        fieldInfo = [];
        filterConditions.name = "";
        if ($scope.field.fields) {
            for (var i = 0; i < $scope.field.fields.length; i++) { // TODO: use reverse for instead
                var label = ($scope.field.fields[i].label ? ($scope.field.fields[i].label + ": ") : "");
                fieldInfo.push({
                    "label": label,
                    "name": $scope.field.fields[i].name,
                    "mask": $scope.field.fields[i].mask || false
                });
                filterConditions.name += $scope.field.fields[i].name + '|';
            }
            filterConditions.name = filterConditions.name.substr(0, filterConditions.name.length - 1);
        }
        else {
            fieldInfo.push({
                "label": "" /*labelWithNoExtraFields*/ ,
                "name": $scope.field.name,
                "mask": $scope.field.mask || false
            });
            filterConditions.name = $scope.field.name;
        }
        $scope.field.dataSource.itemsPerPage = 100;
        $scope.field.dataSource.disableLoader = true;

        if($scope.field.events){
            let fieldOnChangeEvent = $scope.field.events.find(event => event.name === 'FieldOnChange');
            if(fieldOnChangeEvent){
                events.FieldOnChange = fieldOnChangeEvent.code;
            }
        }
    };
    $scope.field.initiliazefieldInfo = initiliazefieldInfo;
    var initiliazeDataSetter = function() {
        if ($scope.field.dataSource) {
            Object.defineProperty($scope.field.dataSource, 'data', {
                get: function() {
                    return listContents;
                },
                set: function(newValue) {
                    if (newValue === listContents) {
                        return;
                    }
                    newValue = [...filteredAditionalOptions].concat([...newValue]);
                    filteredListContents = listContents = newValue;
                    renderListElements(listContents);
                    if(!$scope.field.descriptionTextRendered){
                        let selectedElement = $scope.field.dataSource.data.find(dataSourceElement => {
                            return dataSourceElement[$scope.field.name] === $scope.__fieldRow;
                        });
                        if($scope.__fieldRow && selectedElement){
                            $scope.descriptionText = $scope.getDescriptionString(selectedElement);
                        }
                    }
                },
                enumerable: true,
                configurable: true
            });
        }

        

    };

    var doCache = function(cacheKey, cacheList) {
        cacheBase[cacheKey] = cacheList;
    };

    var initiliazeData = function() {
        if ($scope.field.dataSource.data) {
            filteredListContents = listContents = $scope.field.dataSource.data;
            let arrayToRender = listContents;
            if(!filteredAditionalOptions.every(aditionalOption => listContents.includes(aditionalOption))){
                listContents = filteredAditionalOptions.concat(listContents);
            }
            renderListElements(arrayToRender);
        }
        else {
            listContents = [];
        }
        if (!$scope.field.dataSource.dataSourceFilter) {
            $scope.field.dataSource.dataSourceFilter = [];
        }
    };

    let closeKeyboard = () => inputElement.blur();

    $scope.autoCompleteConstructor = function() {
        rootElement = $element.get(0);
        listElement = rootElement.querySelector('#element-list');
        inputElement = rootElement.querySelector('#content');
        autoCompletePreviewElement = rootElement.querySelector('#autocomplete-preview');
        clearButtonElement = rootElement.querySelector('#clear');
        clearButtonElement.addEventListener('click', clear);
        spinner = rootElement.querySelector('.select-autocomplete-spinner');
        popUp = rootElement.querySelector('.select-autocomplete-pop-up');
        inputElement.addEventListener('keyup', keyStrokeHandler);
        inputElement.addEventListener('keydown', commandKeyStrokeHandler);
        zhLoader = document.querySelector('.zh-background-loading');
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        let backButton = rootElement.querySelector('#back-button');
        backButton.addEventListener('click', ApplicationContext.UIManager.goBack);
        listElement.addEventListener('touchstart', closeKeyboard);
        inputElement.placeholder = "Pesquise " + $scope.field.label;

        baseListRowToClone = rootElement.querySelector('.select-autocomplete-row').cloneNode(true);
        baseListRowToClone.style.display = 'flex';
        if(!$scope.isMultiple) {
            let checkbox = baseListRowToClone.querySelector('input[type=checkbox]');
            checkbox.style.display = 'none';
        }
        
        if ($scope.field.transformOrigin) {
            popUp.style.transformOrigin = $scope.field.transformOrigin; 
        }

        if($scope.beforeSelectAutoCompleteInitialized) {
            $scope.beforeSelectAutoCompleteInitialized($scope);
        }
        initiliazefieldInfo();
        initiliazeData();
        initiliazeDataSetter();
        $scope.field.clearValue = $scope.field.clear = clear;

        $scope.field.setDescriptionText = newDescriptionText => {
            $scope.descriptionText = newDescriptionText;
            $scope.$apply();
        }

        if($scope.onSelectAutoCompleteInitialized) {
            $scope.onSelectAutoCompleteInitialized($scope);
        }
    };

    $scope.field.autoCompleteScope = $scope;

    /**
     * render new list
     * 
     * @param {Object[]} listElements an array of json objects to be rendered  
     */
    function renderListElements(listElements) {

        while (listElement.lastChild) {
            listElement.removeChild(listElement.lastChild);
        }
        for (var index = 0; index < 200/*8*/ && index < listElements.length; index++) {
            var newHtmlListElement = baseListRowToClone.cloneNode(true);
            newHtmlListElement.children[1].children[0].textContent = getFormatedString(listElements[index], 0);
            newHtmlListElement.children[1].children[1].children[0].textContent = getFormatedString(listElements[index], 1);
            newHtmlListElement.children[1].children[1].children[1].textContent = getFormatedString(listElements[index], 2);

            newHtmlListElement.realIndex = index;
            newHtmlListElement.addEventListener('click', confirm);
            listElement.appendChild(newHtmlListElement);
        }
        if (index > 0) {
            if (currentHighletedListElement)
                currentHighletedListElement.classList.remove('active');
            currentHighletedListElement = listElement.children[0];
            currentHighletedListElement.classList.add('active');
        }
    }


    /**
     * @param {Object} row - a object to extract the values for the display string
     * @returns {string} a string with the descriptionFields that are in the row concated with pipe
     */
    $scope.getDescriptionString = function(row){
        if($scope.field.descriptionFields){
            return  $scope.field.descriptionFields.map(descriptionField => row[descriptionField]).join(' | ');
        } else {
            return row[$scope.field.name];
        }
    }

    /**
     * returns an String with the field name concatenated, or an empty string if the object doesn't have that field  
     * @returns {String} 
     * @param {Obect} object 
     * @param {Number} index the index of the field that is being searched on the fieldInfo array 
     */
    function getFormatedString(object, index) {
        if (fieldInfo.length <= index && object) {
            return "";
        }
        else if (object.hasOwnProperty(fieldInfo[index].name)) {
            var value = object[fieldInfo[index].name];
            if (fieldInfo[index].mask) {
                value = Util.applyMask(value, fieldInfo[index].mask, maskEngine, ZhUtil);
            }
            return fieldInfo[index].label + value;
        }
        else {
            return "";
        }
    }

    /**
     * @param {string} stringToSearch 
     * @param {object[]} arraryToSearch
     * @param {boolean} [cleanReturn] - if is set to true the method doesn't affect 'filteredListContents' and returns a new array instead 
     */
    "use strict";
    function filterElements(stringToSearch, arraryToSearch, cleanReturn) {
        var workArray;
        arraryToSearch = arraryToSearch || listContents;
        if (!cleanReturn) {
            workArray = filteredListContents;
        }
        workArray = [];
        for (var i = 0; /*i < 9 && */ i < arraryToSearch.length; i++) {
            if (stringIsInJSON(arraryToSearch[i], stringToSearch))
                workArray.push(arraryToSearch[i]);
        }
        if (cleanReturn)
            return workArray;
    }

    /**
     * //TODO refactor docs// search all properties of a json object to find a string
     * @param {Object} object 
     * @param {String} stringToMatch 
     */
    function stringIsInJSON(object, stringToMatch) {
        stringToMatch = stringToMatch.toLowerCase();
        for (var i = fieldInfo.length; i--;) {
            if (('' + object[fieldInfo[i].name]).toLowerCase().indexOf(stringToMatch) !== -1)
                return true;
        }
        return false;
    }

    /** 
     * @param {MouseEvent} event - a event to extract the list html element
     * @param {HTMLElement} listElement - a list element to select
     */
    var confirm = function(event, listElement) {
        var element;
        var shouldForceDigest = true;
        if (event) {
            element = event.currentTarget;
        }
        else {
            element = listElement;
            shouldForceDigest = true;
        }
        var newValueObject = filteredListContents[element.realIndex];
        setValue(newValueObject);
        
        inputElement.blur();
        clearButtonElement.style.display = 'initial';
        ApplicationContext.UIManager.goBack();
        if (shouldForceDigest) {
            $scope.$apply();
        }
    };

    /**
     * the default value change handler
     */
    $scope.__onSelectAutoCompletedValueChanged = () => {
        $scope.fieldRow = $scope.__fieldRow;
    };


    /**
     * updates the UI and changes the internal label info vars 
     * @param {string} newLabel
     */
    $scope.field.setLabel = (newLabel) => {
        $scope.field.label = newLabel;
        let headerLabelElement = rootElement.querySelector('.select-autocomplete-field-label');
        headerLabelElement.textContent = $scope.field.label;
        inputElement.placeholder = "Pesquise " + $scope.field.label;
    };

    $scope.field.setValue = (newValue) => {
        if(typeof newValue === 'object' && newValue !== null){
            setValue(newValue);
        } else if($scope.field.name && newValue){
            let setValueArgs = {};
            setValueArgs[$scope.field.name] = newValue;
            setValue(setValueArgs);
        }
    };

    /**
     * refreshes the ui, changes internal variables and trigger listeners
     * TODO: refactor this doc
     * @param {Object} newValue 
     */
    let setValue = function(newValue){

        if($scope.field.rowDump){
            $scope.__fieldRow = newValue;
        } else {
            let newValueString = newValue[$scope.field.name];
            $scope.__fieldRow = newValueString;
            inputElement.value = newValueString;
        }
        $scope.descriptionText = $scope.getDescriptionString(newValue);
        
        if($scope.onSelectAutoCompletedValueChanged){
            $scope.onSelectAutoCompletedValueChanged(newValue, $scope);
        } else {
            $scope.__onSelectAutoCompletedValueChanged();
        }

        insertOutData(newValue);
        if(events.FieldOnChange) {
            $scope.$eval(events.FieldOnChange, Object.assign({}, ApplicationContext, {
                args: {
                    owner: {field: $scope.field},
                    row: $scope.field.widget.currentRow
                }
            }));
        }
    }

    /**
     * clear the value of the current row, and the applied filter
     * @param {Boolean} [clearOutData] wheter or not the clear function should remove 
     * the values associated with this field on the OutData prop from the currentRow
     */
    function clear(clearOutData = false, forceUpdate = false) {
        if (activeRequests)
            cancelRequest();
        $scope.__fieldRow = null;
        $scope.__onSelectAutoCompletedValueChanged();
        inputElement.value = ""; 
        $scope.descriptionText = "";
        clearButtonElement.style.display = 'none';
        if (!cacheBase.hasOwnProperty('') || forceUpdate) {
            update();
        }
        else {
            $scope.field.dataSource.data = cacheBase[""];
        }
        if(clearOutData){
            destroyOutData();
        }
        if($scope.onClearAutoCompleteValue){
            $scope.onClearAutoCompleteValue($scope);
        }
    }

    /**
     * 
     * @param {Object} rowObject -- a ZhRowObject to extract the data and insert on the currentRow
     */
    function insertOutData(rowObject) {
        for (var key in $scope.field.outData) {
            var value = rowObject[key];
            $scope.field.widget.currentRow[$scope.field.outData[key]] = value;
        }
    }

    function destroyOutData() {
        for (var key in $scope.field.outData) {
            delete $scope.field.widget.currentRow[$scope.field.outData[key]];
        }
    }

    async function fetchData() {
        var deferredToCancelRequest = $q.defer();
        deferredToCancelRequest.reqText = inputElement.value;
        activeRequests.push(deferredToCancelRequest);
        spinner.classList.remove('hidden');
        ScreenService.hideLoader();

        var dataSourceFilter = $scope.field.dataSourceFilter || $scope.field.dataSource.dataSourceFilter || [];
        var newFilter = dataSourceFilter.slice(0); 

        newFilter.push({
            name: filterConditions.name,
            value: "%" + inputElement.value + "%",
            operator: "LIKE_ALL"
        });

        var promisseToReturn = await $scope.field.dataSource.load(
            newFilter,
            null,
            null,
            () => {},
            () => {}, -1,
            deferredToCancelRequest.promise,
            () => {}
        ).then(function(resultSet) {
            if (resultSet && resultSet.dataset) {
                doCache(deferredToCancelRequest.reqText, 
                    filteredAditionalOptions.concat(resultSet.dataset[Object.keys(resultSet.dataset)[0]])
                );
            }
            else {
                ZhLogger.setLevel(ZhLogger.previousLevel);
            }
            deferredToCancelRequest = false;
        }, function(errorArg) {}
        ).finally(function() {
            hideSpinner();
        });
        
        return promisseToReturn;
    }

    $scope.openPopUp = () => {
        popUp.classList.add('visible');
        inputElement.focus();
        
        ApplicationContext.UIManager.pushState(() => {
            $scope.closePopUp(); 
        });
        
        zhLoader.classList.add('invisible');
    };
    
    $scope.closePopUp = () => {
        popUp.classList.remove('visible');
        zhLoader.classList.remove('invisible');
    }

}
