"use strict";

function HeaderTemplateController($scope, templateManager, ApplicationContext) {

    function getContainer() {
		return templateManager.container;
    }
    
    function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

    $scope.$watch(() => getContainer().getActiveWidget().name, () => {
        $scope.init();
    });
    
    $scope.$watch(() => getContainer().getActiveWidget().title, () => {
        updateTitle();
    });

    function updateTitle() {
        getContainer().title = getActiveWidget().title || getTitle();
    };


    $scope.init = () => {
        getContainer().title = getActiveWidget().title || getTitle();
        let header = document.getElementById('headerComponent');
        let headerActions = getActiveWidget().headerActions || getContainer().headerActions;
        if (headerActions && header) {
            let left = headerActions.left ? headerActions.left.length : 0;
            let right = headerActions.right ? headerActions.right.length : 0;
            let totalAc = [left, right];
            let coluns = '';

            for (let i = 0; i < totalAc.length; i++) {
                for (let j = 0; j < totalAc[i]; j++) {
                    coluns += '50px ';
                }
                coluns += i == 0 ? 'auto ' : '';
            }
            header.style.gridTemplateColumns = coluns;

            getContainer().openHeaderMenu = openHeaderMenu;
            getContainer().displayedActions = {};
            getContainer().displayedActions.left = headerActions && headerActions.left ? headerActions.left : [];
            getContainer().displayedActions.right = headerActions && headerActions.right ? headerActions.right : [];
            getContainer().displayedActions.menu = headerActions && headerActions.menuActions ? headerActions.menuActions : [];
            getContainer().fabVisibility = (getActiveWidget().fabAction || getActiveWidget().fabActions) ? true: false;
            getContainer().fabActionIcon = getActiveWidget().fabActionIcon || 'fp-add';
            $scope.showMenuActions = false;
            $scope.menuActions = getContainer().displayedActions.menu;
        }

		document.documentElement.style.setProperty('--headerHeight', (getContainer().showSubHeader ? "85px" : "51px")); 
		if(getContainer().subHeader === undefined){
			getContainer.subHeader = {};
		} else {
			if(getContainer().subHeader.evalText){
				getContainer().subHeader.text = $scope.$eval(getContainer().subHeader.evalText, ApplicationContext);
			}
		}
    };

    $scope.shouldShowHeader = () => {
        return getContainer().showHeader;
    };

    function getTitle() {
        return getActiveWidget().label || getContainer().name;
    };

    $scope.executeControllerAction = (code) => {
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
    
    $scope.executeFABAction = () => {
        let widget = getActiveWidget();
        if(widget.fabActions){
            $scope.fabActionsVisible = true;
            $scope.fabActions = widget.fabActions;
        } else if(widget.fabAction){
            let code = getActiveWidget().fabAction;
            $scope.executeControllerAction(code);
        }
    };

    $scope.closeFABActions = () => {
        $scope.fabActionsVisible = false;
        delete $scope.fabActions;
    };

    $scope.executeFABActionCode = (code) => {
        $scope.closeFABActions();
        $scope.executeControllerAction(code);
    };

    const openHeaderMenu = () => $scope.showMenuActions = true;

    $scope.closeHeaderMenu = () => $scope.showMenuActions = false;

    $scope.onMenuActionClick = (code,)=>{
        $scope.executeControllerAction(code);
        $scope.showMenuActions = false;
    };
    
    $scope.shouldShowSubHeader = () => $scope.shouldShowHeader() && getContainer().showSubHeader;
    
    getContainer().setSubHeaderVisibility = (visibility) => {
    	document.documentElement.style.setProperty('--headerHeight', (visibility ? "85px" : "51px"));
    	getContainer().showSubHeader = visibility;
    	$scope.$apply();
    };
    
    getContainer().setSubHeaderText = (text) => {
    	getContainer().subHeader.text = text;
    	$scope.$apply();
    };
}
