"use strict";

function ListForGeneralUse($scope, ApplicationContext, templateManager, ScreenService, $element) {
    
    $scope.testContent = (e, field) => {
        let fieldValue = e.currentTarget.value;
        field.inputHasContent = (fieldValue && fieldValue.length > 0 && fieldValue != "?");
    };

    $scope.testContentWithFieldValue = (field) => {
        field.inputHasContent = (field.value()!==undefined && field.value()!==null);
    };
    
    $scope.getFormatedFieldValue = (field, silm) => {
        let fieldValue = field.prefix || '';
        
        if(silm[field.name]!==undefined && silm[field.name]!==null){
            fieldValue += silm[field.name];
        }

        return fieldValue;
    };

    $scope.initAutocompleteField = field => {
        $scope.descriptionText = field.widget.currentRow[field.name];
        $scope.testContentWithFieldValue(field);
    };

    $scope.formatData = (data) => {
        let formattedDate = {};

        if(moment(data, 'DD/MM/YYYY hh:mi:ss').isValid()){
            formattedDate = moment(data, 'DD/MM/YYYY hh:mi:ss');
            moment.locale('pt-br');
        } else {
            formattedDate = moment(data);
        }
        
        return formattedDate.format('ddd, DD [de] MMM [de] YYYY');
    };

    $scope.setAvabilityStatus = (widget) => {
        if(widget.idForEnableButton){
            let hasDifStatus = widget.dataSource.data.filter(d => d[widget.idForEnableButton.name] === widget.idForEnableButton.value);
            if (hasDifStatus && hasDifStatus.lenght) {
                hasDifStatus.forEach(hds => {
                    hds.disabled = true;
                });
            }
        }
    };

    $scope.onSelectAutoCompletedValueChanged = (selectedValueObject, fieldScope) => {
        let field = fieldScope.field;
        field.inputHasContent = selectedValueObject && selectedValueObject !== {} && selectedValueObject !== null;
        if (field.inputHasContent) {
            fieldScope.__onSelectAutoCompletedValueChanged();
        }
    };

    let executeScopeCode = (code) => {
        $scope.$eval(
            code,
            Object.assign({}, ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };

    $scope.executeScopeCode = executeScopeCode;

    $scope.openDetail = (afterClickOnItemWidget, curItem) => {
        templateManager.container.currentItem = curItem;
        executeScopeCode(afterClickOnItemWidget);
    };

    $scope.initListWithConfirmationButton = (widget) => {
        initToList(widget);
    };

    $scope.goBack = () => ApplicationContext.UIManager.goBack();

    let render = function(filteredData, widget){
        widget.dataSource.data = filteredData;
        $scope.$apply();
    };

    let searchAdapter;
    const initToList = (widget) => {
        const UIManager = ApplicationContext.UIManager;
        
        searchAdapter = new SearchAdapter($element[0].querySelector('.search-input'), widget, 
        (filteredData) => render(filteredData, widget), 
        {
            local: true,
            dataSourceOrigin: widget.dataSource,
            dataSourcePropName: "__copyData",
            searchFields: widget.searchFields
        });
        
        widget.showSearch = () => {
            $scope.isSearching = true;
            widget.container.fabVisibility = false;
            widget.dataSource.__copyData = angular.copy(widget.dataSource.data);

            UIManager.pushState(() => {
                $scope.isSearching = false;
                widget.dataSource.data = angular.copy(widget.dataSource.__copyData);

                let inputElement = $element[0].querySelector('.search-input');
                inputElement.value = "";
                inputElement.blur();

                widget.container.fabVisibility = true;
                if('cordova' in window){
                    $scope.$apply();
                }
            });

            let input = $element[0].querySelector('.search-input');
            setTimeout(() => input.focus(), 10);
        }
    };

    $scope.concAction = (sim, e) => {
        e.stopPropagation();

        let widget = $scope.widget;
        widget.currentSimToConclude = sim;
        let msg = '<b>Concluir Simulação ' + sim.CDVERSAOPROPCOM + '?</b><br>';
        msg += '<span style="font-size:.8rem">Ao concluir, você não poderá alterar nenhuma informação.</span>';
        ScreenService.confirmMessage(msg, 'question', () => {
            executeScopeCode(widget.executeOnConclusion);
        });
    };

    $scope.setSizeClass = () => {
        let widget = $scope.widget.container.getActiveWidget();

        let hasHeader = widget.showHeader || widget.container.showHeader ? 50 : 0;
        let insideTabs = widget.isInsideTab ? 50 : 0;
        let actionsSize = widget.actions.filter(a => a.isVisible == true).length;

        let listNewSize = hasHeader + insideTabs + (actionsSize * 70);

        document.documentElement.style.setProperty('--simpleListSize', 'calc(100vh -' + listNewSize + 'px)');
    };

    $scope.startDeletion = (e, widget, item) => {
        let msg = 'Deseja realmente excluir este item?';
        let slided = e.currentTarget;

        widget.container.itemToExclude = item;
        slided.classList.add('beingDeleted');
        setTimeout(() => {
            ScreenService.confirmMessage(msg, 'question', () => {
                if (slided) {
                    slided.classList.remove('beingDeleted');
                }
                executeScopeCode(widget.excludeAction);
            }, () => {
                if (slided) {
                    slided.classList.remove('beingDeleted');
                }
            });
        }, 500);
    };

    $scope.itemLongTapAction = (tappedItem)=>{

        if($scope.widget.itemLongTapAction){

            $scope.$eval(
                $scope.widget.itemLongTapAction,
                Object.assign({}, ApplicationContext, {
                    args: {
                        owner: $scope.widget,
                        item: tappedItem
                    }
                })
            );

        } else if($scope.widget.allowSelectOnLongTap){
            $scope.selectLongTappedItem(tappedItem);
        }
    };

    $scope.selectLongTappedItem = item =>{
        if(item.__isSelected){
            if($scope.widget.beforeUnselectRow)
                $scope.widget.beforeUnselectRow();
            
            $scope.widget.dataSource.removeCheckedRows(item);

            if($scope.widget.afterUnselectRow)
                $scope.widget.afterUnselectRow();
        } else{
            if($scope.widget.beforeSelectRow)
                $scope.widget.beforeSelectRow();

            $scope.widget.dataSource.addCheckedRows(item);
            
            if($scope.widget.afterSelectRow)
                $scope.widget.afterSelectRow();
        }

    };

    $scope.toggleValue = field =>{
        field.widget.currentRow[field.name] = !field.widget.currentRow[field.name];
        if(field.change){
            field.change(field);
        }
    };
}