function ScrollTabbedDataController($scope, $element, $timeout, ApplicationContext){

    /**
     * Test field and set inputHasContent
     * according to value.
     */
    $scope.testContentWithFieldValue = (field) => {
        let currentItem = $scope.widget.dataSource.data[$scope.currentActiveTabIndex];
        field.inputHasContent = (currentItem[field.name]!==undefined && currentItem[field.name]!==null);
    };

    /**
     * Initiate template. Set 'dataSource' var
     * reference. Start touch event listener.
     */
    $scope.initScrollTabbedData = () => {
        $scope.dataSource = $scope.widget.dataSource; 
        $scope.initListener();
    };

    /**
     * Initiate autocomplete template. Set exibition value
     * reference.
     */
    $scope.onSelectAutoCompleteInitialized = fieldScope => {
        let activeIndex = $scope.widget.currentActiveTabIndex || 0;

        fieldScope.descriptionText = $scope.dataSource.data[activeIndex][fieldScope.field.name];
        $scope.testContentWithFieldValue(fieldScope.field);
    };

    /**
     * After select auto complete clear it's value, remove
     * the value from active register and related outData values.
     */
    $scope.onClearAutoCompleteValue = fieldScope => {
        let currentItem = $scope.widget.getActiveTabData();
        currentItem[fieldScope.field.name] = false;
        for (var key in fieldScope.field.outData) {
            delete currentItem[fieldScope.field.outData[key]];
        }
    };

	/**
     * Set 'updateDescriptioText' function into field instace.
	 */
	$scope.initAutoCompleteFunctions = field => {
        field.updateDescriptionText = (newDescription = null) => {
        	let fieldScope = field.autoCompleteScope;
			let activeTabData = $scope.widget.getActiveTabData();
        	
        	if(newDescription === null){
		        fieldScope.descriptionText = activeTabData[field.name];
        	} else {
		        fieldScope.descriptionText = newDescription;
        	}

        };
    };

    /**
     * Add event listener at touchend event to
     * change active tab by scrolling template.
     */
    $scope.initListener = () => {
        if(!$scope.listenerAttached){
            const tabsContainer = $element[0].querySelector('.s-tabbedContent');
            tabsContainer.addEventListener('touchend', changeActiveTabByScroll ,false);
            $scope.listenerAttached = true;
        }
    };

    /**
     * After ng-repeat set active tab according
     * to 'currentActiveTabIndex' property
     * in widget. If property is undefined, set
     * to first register
     */
    $scope.postRepeat = () => {
        let activeIndex = $scope.widget.currentActiveTabIndex || 0;
        $scope.currentActiveTabIndex = activeIndex;
        if($scope.dataSource.data && $scope.dataSource.data.length > 0){
            $scope.dataSource.data.forEach(reg=>reg.isTabActive = false);
            $scope.dataSource.data[activeIndex].isTabActive = true;
        }
    };

    /**
     * After click on tab header, scroll to 
     * correspondent tab container and then
     * set clicked register as active.
     * 
     * @param {Number} tabIndex Index of clicked
     * tab.
     */
    $scope.clickOntabHeaderEvent = (tabIndex) => {
        const pageSize = window.innerWidth;
        const tabsContainer = $element[0].querySelector('.s-tabbedContent');
        tabsContainer.scroll({
            left: tabIndex * pageSize,
            behavior: "smooth"
        });
        activateTab(tabIndex);
    };

    /**
     * Activate tab register according to index
     * passed as parameter, and deactivate previous
     * register.
     * 
     * @param {Number} tabIndex Index of register
     * to activate
     * @param {Boolean} forceDigest Forge angular
     * $apply.
     */
    let activateTab = (tabIndex, forceDigest = false) => {
        if(tabIndex < $scope.dataSource.data.length && tabIndex !== $scope.currentActiveTabIndex){
            if($scope.widget.beforeChangeActiveTab){
                $scope.executeScopeCode($scope.widget.beforeChangeActiveTab);
            }
            
            let previousActiveRegister = $scope.dataSource.data[$scope.currentActiveTabIndex]; 
            previousActiveRegister.isTabActive = false;
            $scope.currentActiveTabIndex = tabIndex;

            let activeRegister = $scope.dataSource.data[tabIndex]; 
            activeRegister.isTabActive = true;

            if($scope.widget.onChangeActiveTab){
                $scope.executeScopeCode($scope.widget.onChangeActiveTab);
            }

            $scope.widget.fields.forEach(field=>{
                $scope.testContentWithFieldValue(field);
            });

            headerScroll(tabIndex);
            if(forceDigest){
                $scope.$apply();
            }
        }
    };

    /**
     * Change active tab after scroll through
     * tab containers.
     */
    let changeActiveTabByScroll = () => {
        let tabsContainer = $element[0].querySelector('.s-tabbedContent');
        clearTimeout($scope.scrollTimeoutID);
        $scope.scrollTimeoutID = setTimeout(() => {
            let lastScroll = Math.floor(tabsContainer.scrollLeft);
            let newIndex = Math.round((lastScroll/ tabsContainer.scrollWidth) * $scope.dataSource.data.length);
            activateTab(newIndex, true);
        }, 200);
    };

    /**
     * Scroll header exibition to index passed as
     * parameter.
     * 
     * @param {Number} indexOfTab Index of register
     * to scroll.
     */
    function headerScroll(indexOfTab) {
        const tabHeader = $element[0].querySelector('.s-tabbedHeader');
        const offsetLeft = tabHeader.children[indexOfTab].offsetLeft;
        tabHeader.scrollLeft = (offsetLeft - window.innerWidth/3);
    }

    /**
     * Toggle value from toggle button in current register.
     * 
     * @param {Object} tabRegister 
     * @param {Object} Field 
     */
    $scope.toggleValue = (tabRegister, field, event) =>{
        tabRegister[field.name] = !tabRegister[field.name];
    };

    /**
     * Check if field has value
     */
    $scope.testContent = (e, field) => {
        let fieldValue = e.currentTarget.value;
        field.inputHasContent = (fieldValue && fieldValue.length > 0 && fieldValue != "?") && field.from != 'toggleButton';
    };

    /**
     * Activate tab register according to index
     * passed as parameter, and deactivate previous
     * register.
     * 
     * @param {Number} tabIndex Index of register
     * to activate
     * @param {Boolean} forceDigest Forge angular
     * $apply.
     */
    $scope.widget.activateTab = (index, forceDigest = true) => {
        const pageSize = window.innerWidth;
        const tabsContainer = $element[0].querySelector('.s-tabbedContent');
        tabsContainer.scroll({
            left: index * pageSize,
            behavior: "smooth"
        });
        activateTab(index, forceDigest)
    };

    /**
     * Get data from active tab.
     */
    $scope.widget.getActiveTabData = () => $scope.dataSource.data[$scope.currentActiveTabIndex];

	/**
	 * Update view of active header
	 */
    $scope.widget.updateHeaderScroll = () => headerScroll($scope.currentActiveTabIndex);

    /**
     * Eval code in App context
     * 
     * @param {String} code
     */
    $scope.executeScopeCode = code => {
        $scope.$eval(
            code,
            Object.assign(ApplicationContext, {
                args: {
                    owner: $scope.widget
                }
            })
        );
    };
};
